# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2011 <Davide Vescovini> <davide.vescovini@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

################################################################################
############# IMPORTAZIONE MODULI  BASE ########################################
################################################################################
from gi.repository import Gtk
import os.path
################################################################################
### FINESTRA DI SELEZIONE OPZIONI GENERICA #####################################
################################################################################

class Dialog(Gtk.Dialog):
    def __init__(self, titolo, options, buttons=None, op_keys_names=None):
        self.__old_options = options
        if buttons == "apply":
            buttons = (Gtk.STOCK_APPLY, Gtk.ResponseType.ACCEPT)
        elif buttons == "save":
            buttons = (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                    Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT)
        else:
            buttons = (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                    Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT)
        # creo una copia del dizionario per opter ripristinare le opzioni base
        self.options = dict()
        self.dialog = Gtk.Dialog(titolo, None, Gtk.DialogFlags.MODAL |
                                 Gtk.DialogFlags.DESTROY_WITH_PARENT, buttons)
        self.dialog.set_resizable(False)
        self.dialog.set_position(Gtk.WindowPosition.CENTER)
        # Create a orizontal box
        hbox = Gtk.HBox(False, 2)
        self.dialog.vbox.pack_start(hbox, False, False, 2)
        hbox.show()
        # Create a vertical box for buttons
        vbox = Gtk.VBox(False, 2)
        hbox.pack_start(vbox, False, False, 2)
        vbox.show()
        # Create a separator
        vsep = Gtk.VSeparator()
        hbox.pack_start(vsep, False, False, 0)
        vsep.show()
        # Create a vertical box for Gtk.RadioButton
        vbox1 = Gtk.VBox(False, 2)
        vbox.pack_start(vbox1, False, False, 2)
        vbox1.show()
        # Create a vertical box for Gtk.CheckButton
        vbox2 = Gtk.VBox(False, 2)
        vbox.pack_start(vbox2, False, False, 2)
        vbox2.show()
        # Create a vertical box for Gtk.Entry
        vbox3 = Gtk.VBox(False, 2)
        hbox.pack_start(vbox3, False, False, 2)
        vbox3.show()
        # imposta una "radioaction" per eventuali pulsanti di scelta
        radiobutton_group = Gtk.RadioButton(group=None, label=None, use_underline=True)
        # set the option dictionary and create the buttons
        for key, value in options.items():
            self.options[key] = value
            if op_keys_names is not None:
                text = op_keys_names.get(key, str(key))
            else:
                text = str(key)
            # se il valore è un intero VERO/FALSO crea un gruppo di Gtk.RadioButton
            if type(value) == tuple:
                button = Gtk.RadioButton(group=radiobutton_group, label=text, use_underline=True)
                button.set_active(value[0])
                button.connect("toggled", self.callback_radiobutton, key)
                vbox1.pack_start(button, False, False, 2)
                button.show()
            # se il valore è VERO/FALSO usa un Gtk.CheckButton
            elif type(value) == bool:
                button = Gtk.CheckButton(text.replace("_", " "))
                button.set_active(value)
                button.connect("toggled", self.callback_button, key)
                vbox2.pack_start(button, False, False, 2)
                button.show()
            # se il valore è un file usa un Gtk.FileChooser
            elif type(value) == str and os.path.exists(os.path.dirname(value)):
                label = Gtk.Label("<i>" + text.replace("_", " ") + ":" + "</i>")
                label.set_use_markup(True)
                # crea il pulsante per la scelta file
                button = Gtk.FileChooserButton('Select Directory')
                button.connect("file-set", self.callback_filechooser, key)
                vbox3.pack_start(label, False, False, 2)
                vbox3.pack_start(button, False, False, 2)
                # imposta i filtri per i file
                filter = Gtk.FileFilter()
                filter.set_name("All files")
                filter.add_pattern("*")
                button.add_filter(filter)
                # filtro per i file database sqlite3
                filter1 = Gtk.FileFilter()
                filter1.set_name("Database SQLITE")
                filter1.add_mime_type("application/x-sqlite3")
                filter1.add_pattern("*.db")
                button.add_filter(filter1)
                button.set_filter(filter1)
                # stabilisce se stiamo gestendo un file o una directory
                if os.path.isfile(value):
                    button.set_filename(value)
                    button.select_filename(value)
                elif os.path.isdir(value):
                    button.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
                    button.set_current_folder(value)
                    button.set_current_folder_uri(value)
                else: 
                    button.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
                    button.set_current_folder(value)
                
                # mostra i widget
                label.show()
                button.show()
            # se il valore è una stringa usa un Gtk.entry
            elif type(value) == str:
                label = Gtk.Label("<i>" + text.replace("_", " ") + ":" + "</i>")
                label.set_use_markup(True)
                entry = Gtk.Entry()
                entry.set_text(value)
                entry.set_icon_from_stock(1, Gtk.STOCK_SAVE)
                entry.connect("activate", self.callback_entry, key)
                entry.connect("icon-press", self.callback_entry_icon, key)
                vbox3.pack_start(label, False, False, 2)
                vbox3.pack_start(entry, False, False, 2)
                label.show()
                entry.show()
            # se il valore è un numero intero utilizza uno SpinButton
            elif type(value) == int:
                label = Gtk.Label("<i>" + text.replace("_", " ") + ":" + "</i>")
                label.set_use_markup(True)
                adj = Gtk.Adjustment(value=value, lower=0, upper=value*10, step_incr=1, page_incr=0, page_size=0)
                button = Gtk.SpinButton(adjustment=adj, climb_rate=1, digits=0)
                button.connect("value_changed", self.callback_spinbutton, key)
                hboxSpin = Gtk.HBox(False, 2)
                hboxSpin.pack_start(label, False, False, 2)
                hboxSpin.pack_start(button, False, False, 2)
                vbox2.pack_start(hboxSpin, False, False, 2)
                label.show()
                button.show()
                hboxSpin.show()
            # se il valore è una lista utilizza una Gtk.entry
            elif type(value) == list:
                stringa = str()
                for data in value:
                    stringa += data + ","
                # imposta l'etichetta e il valore della Gtk.entry
                label = Gtk.Label("<i>" + text.replace("_", " ") + ":" + "</i>")
                label.set_use_markup(True)
                entry = Gtk.Entry()
                entry.set_text(stringa)
                entry.set_icon_from_stock(1, Gtk.STOCK_SAVE)
                entry.connect("icon-press", self.callback_entry_list, key)
                vbox3.pack_start(label, False, False, 2)
                vbox3.pack_start(entry, False, False, 2)
                label.show()
                entry.show()
################################################################################
# Avvio dell finestra di dialgo ################################################
################################################################################
    def run(self):
        response = self.dialog.run()
        if response == Gtk.ResponseType.ACCEPT:
            options = self.options
        elif response == Gtk.ResponseType.CANCEL:
            options = self.__old_options
        elif response == Gtk.ResponseType.CLOSE:
            options = self.__old_options
        else: options = self.__old_options
        self.dialog.destroy()
        return options
################################################################################
# Funzioni richiamate quando di immettono dati nell'interfaccia grafica ########
################################################################################
    def callback_radiobutton(self, widget, data=None):
        if widget.get_active():
            self.options[data] = (1,)
        else: self.options[data] = (0,)
    def callback_button(self, widget, data=None):
        self.options[data] = widget.get_active()
    def callback_spinbutton(self, widget, data=None):
        self.options[data] = int(widget.get_value())
    def callback_entry(self, widget, data=None):
        self.options[data] = widget.get_text()
    def callback_entry_icon(self, widget,icon_pos, event, data=None):
        self.options[data] = widget.get_text()
    def callback_filechooser(self, widget, data=None):
        self.options[data] = widget.get_filename()
    def callback_entry_list (self, widget, icon_pos, event, data=None):
        self.options[data] = widget.get_text().split(",")
        for par in self.options[data]:
            par = par.rstrip(" ").lstrip(" ")
            if par == "": self.options[data].remove(par)
