# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 <Davide Vescovini> <davide.vescovini@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

######################################################
########### MODULI E LIBRERIE BASE ###################
######################################################
import argparse #(python 2.7)
from gi.repository import Gtk # pylint: disable=E0611
from preventares import PreventaresWindow
from preventares_lib import set_up_logging, get_version
######################################################
########### MODULI PER INTERNAZIONALIZZAZIONE ########
######################################################
import gettext
from gettext import gettext as _
gettext.textdomain('preventares')
######################################################
########### SUPPORTO PER LINEA DI COMANDO ############
######################################################
def parse_options():
    """Support for command line options"""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--version', action='version', 
        version= "%prog {0}".format(PreventaresWindow.VERSION))
    parser.add_argument(
        "-d", "--dump-mdb", metavar="DATABASE_.MDB", dest="dump_mdb",
        help=_("Dump .mdb price list"))
    parser.add_argument(
        "-l", "--import-prevwin-archive", metavar="PREVWIN_DATABASE", dest="convert_archive",
        help=_("Import Prevwin .mdb archive into PreventARES archive"))
    parser.add_argument(
        "-L", "--import-prevwin-price-list", metavar="PREVWIN_DATABASE", dest="convert_computo",
        help=_("Import Prevwin .mdb price list into PreventARES archive"))
    parser.add_argument(
        "-m", "--import-ANALIST-archive", metavar="ANALIST_DATABASE", dest="import_ANALIST",
        help=_("Convert 'ANALIST software Tariffario' price list (.mdb) into PreventARES archive"))
    parser.add_argument(
        "-s", "--show-database", metavar="PreventARES_DATABASE", dest="show_database",
        help=_("Show the tables content for PreventARES database"))
    parser.add_argument(
        "-S", "--shell", metavar="PreventARES_DATABASE", dest="shell",
        help=_("Minimal shell for Preventa_lib. Use ':memory:' to open in memory database"))
    parser.add_argument(
        "-v", "--verbose", action="count", dest="verbose",
        help=_("Show debug messages (-vv debugs preventares_lib also)"))
    parser.add_argument(
        "-t", "--export-txt", metavar="PreventARES_DATABASE", dest="export_txt",
        help=_("Export database to txt file"))
    parser.add_argument(
        "-X", "--import-xml", metavar="XMLFILE", dest="import_xml",
        help=_("Convert XML (SIX format) price list  into PreventARES archive"))
    parser.add_argument(
        "--logfile", dest="logfile", metavar="LOG_FILENAME",
        help=_("write debug messages to LOGFILE"))
    parser.add_argument(
        "--outfile", dest="output_filename", metavar="OUTPUT_FILENAME",
        help=_("override the standard OUTPUT filename and set it to OUTPUT FILENAME"))
    parser.add_argument(
        "-F", "--convert-finpolo", metavar="TXTFILE", dest="finpolo",
        help=_("Convert Finpolo price list into standard METEL file"))
    # parse arguments and setup logging
    options = parser.parse_args()
    set_up_logging(options)
    return options

################################################################################
############ MAIN PROCESS - CON SUPPORTO x RIGA DI COMANDO E LOGGING ###########
################################################################################
def main():
    'constructor for your class instances'
    options = parse_options()
    # CLI commmand to convert Prevwin archive.
    if options.convert_archive is not None:
        from preventares import import_elettro_mdb
        if options.output_filename is None:
            FILE_ARCHIVIO = options.convert_archive.replace(".", "_") + ".db"
        else: FILE_ARCHIVIO = options.output_filename
        # esegue l'importazione del file usando la libreria preventa-lib.py
        import_elettro_mdb.import_archivio(options.convert_archive, FILE_ARCHIVIO)
    ############################################################################
    # CLI commmand to convert Prevwin computo.
    if options.convert_computo is not None:
        from preventares import import_elettro_mdb
        if options.output_filename is None:
            FILE_ARCHIVIO = options.convert_computo.replace(".", "_") + ".db"
        else: FILE_ARCHIVIO = options.output_filename
        # esegue l'importazione del file usando la libreria preventa-lib.py
        import_elettro_mdb.import_preventivo(options.convert_computo, FILE_ARCHIVIO)
    ############################################################################
    # CLI commmand to convert Dump .mdb database.
    if options.dump_mdb is not None:
        from preventares import import_elettro_mdb
        # esegue l'importazione del file usando la libreria preventa-lib.py
        import_elettro_mdb.dump_mdb(options.dump_mdb)
    ############################################################################
    # CLI commmand to convert and import ANALIST software .mdb database.
    if options.import_ANALIST is not None:
        from preventares import import_elettro_mdb
        if os.path.isfile(options.import_ANALIST):
            import_elettro_mdb.import_preventivo_mdb_ANALIST(options.import_ANALIST)
        else:
            print _("""Errore: Il file '%s' è inesistente.""" % options.import_ANALIST)
    ############################################################################
    # CLI commmand to print the database content to the standard output (terminal).
    if options.show_database is not None:
        from preventares_lib import preventa_lib
        if os.path.isfile(options.show_database):
                preventa_lib.show(options.show_database)
        else:
            print _("""Errore: Il file '%s' è inesistente.""" % options.show_database)
    ############################################################################
    # CLI commmand to open preventa_lib shell.
    if options.shell is not None:
        from preventares_lib import preventa_lib
        if os.path.isfile(options.shell):
            preventa_lib.shell(options.shell)
        elif options.shell == ":memory:":
            preventa_lib.shell()
    ############################################################################
    # Export all database to txt file
    if options.export_txt is not None:
        from preventares_lib import preventa_lib
        if options.output_filename is None:
            FILENAME = options.export_txt.replace(".", "_") + ".txt"
        else: FILENAME = options.output_filename
        if os.path.isfile(options.export_txt):
                preventivo = preventa_lib.Preventivo(options.export_txt)
                preventivo.export_to_dumpfile(FILENAME)
                preventivo.connection_shutdown()
        else:
           print _("""Errore: Il file '%s' è inesistente.""" % options.export_txt)
    ############################################################################
    # Import xml file
    if options.import_xml is not None:
        from preventares_lib import preventa_lib
        if options.output_filename is None:
            DATABASE = options.import_xml.replace(".", "_") + ".db"
        else: DATABASE = options.output_filename
        if os.path.isfile(options.import_xml):
            preventivo = preventa_lib.Preventivo(DATABASE)
            preventivo.import_XML(options.import_xml)
            preventivo.connection_shutdown()
        else:
            print _("""Errore: Il file '%s' è inesistente.""" % options.import_xml)
    ############################################################################
    # Convert Finpolo file into standard METEL file
    if options.finpolo is not None:
        from preventares import import_elettro_mdb
        if os.path.isfile(options.finpolo):
            import_elettro_mdb.TranslateFinpoloFile(options.finpolo)
        else:
            print _("""Errore: Il file '%s' è inesistente.""" % options.finpolo)
    ############################################################################
    # Run the application if command line option are not called
    if not options.import_xml \
       and not options.export_txt \
       and not options.show_database \
       and not options.import_ANALIST \
       and not options.dump_mdb \
       and not options.shell \
       and not options.finpolo:
        # Run the application.
        window = PreventaresWindow.PreventaresWindow()
        window.show()
        Gtk.main()
