#!/usr/bin/python
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2011 <Davide Vescovini> <davide.vescovini@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

####################################################################################
############# IMPORTAZIONE MODULI  BASE ############################################
####################################################################################
import sys
import os
import urllib
from gi.repository import Gtk, Gdk, GObject, Pango
################################################################################
############ SUPPORTO INTERNAZIONALIZZAZIONE ###################################
################################################################################
import gettext
from gettext import gettext as _
gettext.textdomain('preventares')
################################################################################
############ IMPOSTAZIONI LOGGINGS #############################################
################################################################################
import logging
logger = logging.getLogger('preventares')
################################################################################
########### VARIABILI DA INIZIALIZZARE #########################################
################################################################################
# Add project root directory (enable symlink, and trunk execution).
PROJECT_ROOT_DIRECTORY = os.path.abspath(
    os.path.dirname(os.path.dirname(os.path.realpath(sys.argv[0]))))

if (os.path.exists(os.path.join(PROJECT_ROOT_DIRECTORY, 'preventares'))
    and PROJECT_ROOT_DIRECTORY not in sys.path):
    sys.path.insert(0, PROJECT_ROOT_DIRECTORY)
    os.putenv('PYTHONPATH', PROJECT_ROOT_DIRECTORY) # for subprocesses

################################################################################
############# IMPORTAZIONE MODULI ##############################################
################################################################################
from preventares import (AboutPreventaresDialog, PrintoptionDialog, 
                         ModifyDialog, OptionChooserDialog, ManodoperaDialog,
                         MetelWindow, preventa_print)
from preventares.FileChooserDialog import FileChooserDialog
from preventares_lib import metel_lib
from preventares_lib import preventa_lib as Preventa
from preventares_lib.helpers import get_builder
from preventares_lib.preventaresconfig import (get_print_settings_file,
                                               get_config_path,
                                               get_archive_path,
                                               get_archive_filename,
                                               get_metel_filename,
                                               get_data_path)

from preventares_lib.libreoffice_preventa import PreventaConvertToWriter
#Try adding AppIndicator. Will work after "quickly add indicator"
try:
    from preventares import indicator
except:
    indicator = False
################################################################################
############# COSTANTI E IMPOSTAZIONI DI DEFAULT ###############################
################################################################################
# impostazioni dell'interfaccia grafica (globali x tutto il programma)
DEFAULT_SETTINGS = {"maximize_window_at_startup": False,
                    "load_main_archive": False,
                    "load_epu_chapter": False,
                    "show_toolbar": True,
                    "show_empty_category": False,
                    "show_empty_chapter": False,
                    "show_column_desc_voce": True,
                    "show_column_materiali": True,
                    "show_column_ricarico": True,
                    "show_column_sicurezza": True,
                    "show_column_tempo_inst": True,
                    "show_column_contract": True,
                    "show_column_desc_voce_lavorazioni": True,
                    "show_column_desc_estesa_lavorazioni": True,
                    "show_column_materiali_lavorazioni": True,
                    "show_column_ricarico_lavorazioni": True,
                    "show_column_tempo_inst_lavorazioni": True,
                    "show_column_sicurezza_lavorazioni": True,
                    "show_mask_article_full": True,
                    "archive_notebook_orizontal": False,
                    "character_nr_for_descr_cod_from_descr_estesa": 10,
                    "new_articolo_name": "Nuovo articolo",
                    "archive_pathname": get_archive_path(),
                    "main_archive_filename": get_archive_filename(),
                    "metel_archive_filename": get_metel_filename(),
                    "default_working_directory": get_config_path()}
# traduzione delle impostazioni UI_DEFAULT_SETTINGS per l'interfaccia grafica
SETTINGS_TRANSLATION = {"maximize_window_at_startup": _("Massimizza la finestra all'avvio"),
                    "load_main_archive": _("Carica l'archivio principale"),
                    "load_epu_chapter": _("Visualizzazione ad albero Epu"),
                    "show_toolbar": _("Mostra la Tooolbar"),
                    "show_empty_category": _("Visualizza categorie vuote"),
                    "show_empty_chapter": _("Visualizza capitoli vuoti"),
                    "show_column_desc_voce": _("Mostra la colonna Descrizione Voce"),
                    "show_column_materiali": _("Mostra la colonna Materiali"),
                    "show_column_ricarico": _("Mostra la colonna Ricarico"),
                    "show_column_sicurezza": _("Mostra la colonna Sicurezza"),
                    "show_column_tempo_inst": _("Mostra la colonna Tempo"),
                    "show_column_contract": _("Mostra le colonne Tipo e Categoria di Appalto"),
                    "show_column_desc_voce_lavorazioni": _("Mostra la colonna Descr.Voce tab 'Epu'"),
                    "show_column_desc_estesa_lavorazioni": _("Mostra la colonna Descr.Estesa tab 'Epu'"),
                    "show_column_materiali_lavorazioni": _("Mostra la colonna Materiali tab 'Epu'"),
                    "show_column_ricarico_lavorazioni": _("Mostra la colonna Ricarico tab 'Epu'"),
                    "show_column_tempo_inst_lavorazioni": _("Mostra la colonna Tempo tab 'Epu'"),
                    "show_column_sicurezza_lavorazioni": _("Mostra la colonna Sicurezza tab 'Epu'"),
                    "show_mask_article_full": _("Mostra descr. nella maschera di mod. articolo"),
                    "archive_notebook_orizontal": _("Notebook Archivio Orizzontale"),
                    "character_nr_for_descr_cod_from_descr_estesa": _("N° max di caratteri per generazione descr. Voce da descr. estesa"),
                    "new_articolo_name": _("Nome di default per nuovo articolo"),
                    "archive_pathname": _("Percorso per i file di Archivio"),
                    "main_archive_filename": _("File per Archivio Principale"),
                    "metel_archive_filename": _("File per Archivio METEL"),
                    "default_working_directory": _("Cartella di Lavoro")}
################################################################################
###### CLASSE COMPUTO WINDOWS ##################################################
################################################################################
class ComputoWindow(Gtk.Window):
    __gtype_name__ = "ComputoWindow"
    
    # To construct a new instance of this method, the following notable 
    # methods are called in this order:
    # __new__(cls)
    # __init__(self)
    # finish_initializing(self, builder)
    # __init__(self)
    #
    # For this reason, it's recommended you leave __init__ empty and put
    # your inialization code in finish_intializing

    def __new__(cls, type = None, filename=None, filearchivio = None, 
                filelistino=None, preventa_settings=None, computo_settings=None):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated ComputoWindow object.
        """
        builder = get_builder('ComputoWindow')
        new_object = builder.get_object("computo_window")
        new_object.finish_initializing(builder, filename, filearchivio, filelistino,
                                       preventa_settings, computo_settings)
        return new_object

    def __init__(self, filename=None, filearchivio=None, filelistino=None, 
                 preventa_settings=None, computo_settings=None):
        self.__filename = filename # nome del file del progetto
        self.__filearchivio = filearchivio # nome del file archivio principale
        self.__filelistino = filelistino # nome del file listino METEL
        self.__preventa_settings = preventa_settings #impostazioni per la libreria 'preventa'
        self.__computo_settings = computo_settings #impostazioni UI 'ComputoWindow'

    def finish_initializing(self, builder, filename, filearchivio, filelistino, 
                            preventa_settings, computo_settings):
        """Called while initializing this instance in __new__
        finish_initalizing should be called after parsing the UI definition
        and creating a ComputoWindow object with it in order to finish
        initializing the start of the new ComputoWindow instance."""

        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.builder.connect_signals(self)
        #Inizializza i nomi dei file e le impostazioni di programma e libreria
        self.__filename = filename # nome del file del progetto
        self.__filearchivio = filearchivio # nome del file archivio principale
        self.__filelistino = filelistino # nome del file listino METEL
        self.__preventa_settings = preventa_settings #impostazioni per la libreria 'preventa'
        self.__computo_settings = computo_settings #impostazioni UI 'ComputoWindow'

        #AppIndicator support
        # use 'quickly add indicator' to get started
        # self is passed so methods of this class can be called from indicator.py
        # Comment to disable appindicator
        if indicator:
            self.indicator = indicator.new_application_indicator(self)
        # self.indicator is an appindicator instance.
        # learn more about it here http://LINK-to-AppIndicator-Docs

        ###################################
        # Imposta le opzioni dell'Applicazione
        ###################################
        # impostazioni iniziali
        global settings
        settings = DEFAULT_SETTINGS
        # se i parametri iniziali sono validi sostituisci i valori di default
        if self.__computo_settings is not None and type(self.__computo_settings) is dict:
            for key in self.__computo_settings.keys():
                if key in settings:
                    settings[key] = self.__computo_settings[key]

        ###################################
        # inizializzo il computo e la libreria di preventivazione
        ###################################
        # se il filename è nullo (None) il preventivo viene caricato in memoria
        # la libreria esegue i propri controlli se i parametri iniziali sono non validi
        self.preventivo = Preventa.Preventivo(self.__filename, self.__preventa_settings)
        self.db = self.preventivo.get_database()

        ###################################
        # inizializzo l'Archivio Principale
        ###################################
        try:
            self.__archivio_principale = Preventa.Preventivo(self.__filearchivio,
                                                             self.__preventa_settings)
        except: 
            self.message(self, "errore", "Impossibile aprire il file '%s'. Archivio Principale non disponibile." % self.__filearchivio)
            self.__archivio_principale = None
            #else: self.on_destroy(self)
        # imposto l'archivio di base uguale a quello principale
        self.archivio = self.__archivio_principale
        ###################################
        # inizializzo le VARIABILI base e l'articolo selezionato
        ###################################
        self.prymary_key = 0 #valore univoco di accesso alle righe del database computo
        self.articolo_computo = None
        self.listino_key = 0 #valore univoco di accesso alle righe del database listino
        self.articolo_listino = None
        self.articolo_epu = None #articolo di EPU in memoria
        self.lista_articoli_computo = None
        self.lista_articoli_epu = None
        self.lista_articoli_listino = None
        self.capitoli = (0,0,0)
        self.categorie = (0,0,0)
        self.contract_type = (None,None)
        #mantiene in memoria la lista delle categorie, supercategorie e capitoli
        self.lista_cap = (1,1,1)
        self.lista_cat = (1,1,1)
        # variabili da finestra listini METEL
        self.metel = None #finesta di dialgo METEL non inizializzata
        self.articolo_metel = None

        ###################################
        # imposta il nome della finestra uguale al filename
        ###################################
        self.set_title(_("Computo: %s" % self.preventivo.get_filename()))
        ###################################
        # impostazioni per pannelli verticali e laterali
        ###################################
        # sistema i pannelli laterali se necessario
        self.pannello_verticale_computo = builder.get_object("hpaned1")
        self.pannello_verticale_epu = builder.get_object("paned1")
        self.pannello_orizzontale = builder.get_object("paned2")
        self.pannello_verticale_lav = builder.get_object("vpaned4")

        ###################################
        # impostazioni per maschera codice
        ###################################
        # riferimento ai frame (cornici) contenenti widget
        self.frame_capitoli = self.builder.get_object("frame4") #capitoli epu
        self.frame_categorie = self.builder.get_object("frame5") #capitoli computo
        self.frame_dati_articolo = builder.get_object("frame6")
        self.frame_elementi_analisi = builder.get_object("frame7")
        self.frame_descrizioni_voce_codice = builder.get_object("frame8")
        # imposta la cella tipo per il render del testo
        cell = Gtk.CellRendererText()
        # imposta il colore base grigio e rosso
        self.colour_lred = Gdk.RGBA(0, 0, 0)
        self.colour_lred.parse("#FF9999") # light red
        self.colour_gray = Gdk.RGBA(0, 0, 0)
        self.colour_gray.parse("#e5e5e5")
        StateType = Gtk.StateType.NORMAL
        # imposta i listsore e le treeview
        self.tariffa = builder.get_object("entry1")
        # imposto la casella tariffa con colore di sfondo grigio chiaro
        self.tariffa.override_background_color(StateType, self.colour_gray)
        self.tariffa.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.tariffa.connect("activate", self.callback_activate_entry)
        self.descrizione_codice = builder.get_object("entry4")
        self.descrizione_codice.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.descrizione_voce = builder.get_object("entry5")
        self.descrizione_voce.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.descrizione_estesa = builder.get_object("textbuffer_desc_estesa")
        self.descrizione_estesa.connect("changed", self.Attiva_pulsante_aggiornamento)
        # imposto la casella quantita con colore di sfondo grigio chiaro
        self.quantita = builder.get_object("entry7")
        self.quantita.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.quantita.connect("activate", self.callback_activate_entry)
        self.quantita.override_background_color(StateType, self.colour_gray)
        # impostazione del ComboBox unità di misura
        self.liststore_um = Gtk.ListStore(str)
        self.liststore_um.elenco_um = ["n.","cad.", "corpo", "m", "mq", "mc", "kg"]
        for a in self.liststore_um.elenco_um:
            self.liststore_um.append((a,))
        table = builder.get_object("table4")
        self.unita_misura = Gtk.ComboBox.new_with_model_and_entry(self.liststore_um)
        self.unita_misura.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.unita_misura.set_entry_text_column(0)
        table.attach(self.unita_misura, 1, 2, 1, 2)
        # imposto la casella ricarico
        self.ricarico = builder.get_object("spinbutton2")
        self.ricarico.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.tempo_inst = builder.get_object("spinbutton3")
        self.tempo_inst.connect("changed", self.Attiva_pulsante_aggiornamento)
        # imposto il costo_materiali con colore di sfondo grigio chiaro
        self.costo_materiali = builder.get_object("entry2")
        self.costo_materiali.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.costo_materiali.connect("icon-press", self.callback_icon_press_entry)
        self.costo_materiali.connect("activate", self.callback_activate_entry)
        self.costo_materiali.override_background_color(StateType, self.colour_gray)
        # imposto il prezzo unitario con colore di sfondo rosso
        self.prezzo_unitario = builder.get_object("entry3")
        self.prezzo_unitario.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.prezzo_unitario.connect("icon-press", self.callback_icon_press_entry)
        self.prezzo_unitario.connect("activate", self.callback_activate_entry)
        self.prezzo_unitario.override_background_color(StateType, self.colour_lred)
        self.sicurezza = builder.get_object("spinbutton4")
        self.sicurezza.connect("changed", self.Attiva_pulsante_aggiornamento)
        # capitoli
        self.liststore_supercapitolo = Gtk.ListStore(str, str, str, str)
        self.box_supercapitolo = builder.get_object("combobox6")
        self.box_supercapitolo.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.box_supercapitolo.pack_start(cell, True)
        self.box_supercapitolo.add_attribute(cell, 'text', 1)
        self.box_supercapitolo.set_model(self.liststore_supercapitolo)
        self.liststore_capitolo = Gtk.ListStore(str, str, str, str)
        self.box_capitolo = builder.get_object("combobox7")
        self.box_capitolo.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.box_capitolo.pack_start(cell, True)
        self.box_capitolo.add_attribute(cell, 'text', 1)
        self.box_capitolo.set_model(self.liststore_capitolo)
        self.liststore_subcapitolo = Gtk.ListStore(str, str, str, str)
        self.box_subcapitolo = builder.get_object("combobox8")
        self.box_subcapitolo.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.box_subcapitolo.pack_start(cell, True)
        self.box_subcapitolo.add_attribute(cell, 'text', 1)
        self.box_subcapitolo.set_model(self.liststore_subcapitolo)
        # categorie
        self.liststore_supercategoria = Gtk.ListStore(str, str, str, str)
        self.box_supercategoria = builder.get_object("combobox9")
        self.box_supercategoria.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.box_supercategoria.pack_start(cell, True)
        self.box_supercategoria.add_attribute(cell, 'text', 1)
        self.box_supercategoria.set_model(self.liststore_supercategoria)
        self.liststore_categoria = Gtk.ListStore(str, str, str, str)
        self.box_categoria = builder.get_object("combobox10")
        self.box_categoria.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.box_categoria.pack_start(cell, True)
        self.box_categoria.add_attribute(cell, 'text', 1)
        self.box_categoria.set_model(self.liststore_categoria)
        self.liststore_subcategoria = Gtk.ListStore(str, str, str, str)
        self.box_subcategoria = builder.get_object("combobox11")
        self.box_subcategoria.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.box_subcategoria.pack_start(cell, True)
        self.box_subcategoria.add_attribute(cell, 'text', 1)
        self.box_subcategoria.set_model(self.liststore_subcategoria)
        # costi materiali 1-4 (extra)
        self.costo_mat_1 = builder.get_object("entry11")
        self.costo_mat_1.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.costo_mat_2 = builder.get_object("entry12")
        self.costo_mat_2.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.costo_mat_3 = builder.get_object("entry13")
        self.costo_mat_3.connect("changed", self.Attiva_pulsante_aggiornamento)
        self.costo_mat_4 = builder.get_object("entry14")
        self.costo_mat_4.connect("changed", self.Attiva_pulsante_aggiornamento)
        # ricerca in computo
        self.liststore_campo_ricerca = Gtk.ListStore(str)
        self.box_campo_ricerca = builder.get_object("combobox1")
        self.box_campo_ricerca.pack_start(cell, True)
        self.box_campo_ricerca.add_attribute(cell, 'text', 0)
        self.box_campo_ricerca.set_model(self.liststore_campo_ricerca)
        self.liststore_campo_ricerca.append(["Tariffa"])
        self.liststore_campo_ricerca.append(["Descrizione Codice"])
        self.liststore_campo_ricerca.append(["Descrizione Voce"])
        self.liststore_campo_ricerca.append(["Descrizione Estesa"])
        self.box_campo_ricerca.set_active(0)
        self.entry_chiave_ricerca = builder.get_object("entry6")
        self.entry_chiave_ricerca.connect("activate", self.ricerca_preventivo)
        self.entry_chiave_ricerca.connect("icon-press", self.ricerca_preventivo)
        # lavorazioni
        self.entry_tariffa_lavorazioni = builder.get_object("entry8")
        self.entry_tariffa_lavorazioni.override_background_color(StateType, self.colour_gray)
        # computo
        self.entry_tariffa_computo = builder.get_object("entry15")
        self.entry_tariffa_computo.override_background_color(StateType, self.colour_gray)

        ###########################################
        # impostazioni per widget NOTEBOOK
        ###########################################
        # Pagine del notebook: 0= dati generali, 1= Computo, 2= Epu
        self.notebook = builder.get_object("notebook1")
        # memorizza il Gtk.Container per poter effettuare la visualizzazione
        # della treeview listino al cambiamento del tab 'Epu' (vedi: def notebook_page_changed)
        self.lavorazioni_scrolledwindow_listino = builder.get_object("scrolledwindow8")
        self.computo_scrolledwindow_listino = builder.get_object("scrolledwindow4")
        # memorizza i Gtk.Container la visualizzazione (reparent) della maschera di modifica articolo
        self.mask_art = builder.get_object("vbox14")
        self.mask_art.hide()
        self.computo_expander = builder.get_object("expander1")
        self.computo_expander.connect("activate", self.on_expander_activated, 
            self.mask_art)
        self.lav_expander = builder.get_object("expander2")
        self.lav_expander.connect("activate", self.on_expander_activated, 
            self.mask_art)
        # memorizza i Gtk.Container per il 'reparent' della maschera dell'Epu
        self.box_treeview_epu = builder.get_object("vbox21")
        self.vbox_laterale_sx_tab_computo = builder.get_object("box4")
        self.vbox_laterale_sx_tab_epu = builder.get_object("box20")
        # imposta le icone delle linguette 'tab'
        icon_size = Gtk.IconSize.BUTTON
        img_dati_generali = builder.get_object("image9")
        img_dati_generali.set_from_stock('preventa-dati-generali', icon_size)
        img_computo = builder.get_object("image6")
        img_computo.set_from_stock('preventa-computo-metrico', icon_size)
        img_epu = builder.get_object("image7")
        img_epu.set_from_stock('preventa-elenco-prezzi', icon_size)
        img_cat_computo = builder.get_object("image28")
        img_cat_computo.set_from_stock('preventa-computo-metrico', icon_size)
        img_cat_epu = builder.get_object("image27")
        img_cat_epu.set_from_stock('preventa-elenco-prezzi', icon_size)
        # setup the archive notebook icons
        img_cat = builder.get_object("image29")
        img_cat.set_from_stock('preventa-computo-metrico', icon_size)
        img_arc1 = builder.get_object("image30")
        img_arc1.set_from_stock('preventa-archive', icon_size)
        img_cap = builder.get_object("image31")
        img_cap.set_from_stock('preventa-elenco-prezzi', icon_size)
        img_arc2 = builder.get_object("image32")
        img_arc2.set_from_stock('preventa-archive', icon_size)
        # get a reference to the labels and notebooks
        self.label_cat = builder.get_object("label26")
        self.label_cap = builder.get_object("label22")
        self.label_arc1 = builder.get_object("label52")
        self.label_arc2 = builder.get_object("label23")
        self.notebook_3 = builder.get_object("notebook3")
        self.notebook_4 = builder.get_object("notebook4")
        
        ###################################
        # impostazioni per DATI GENERALI
        ###################################
        self.dg_ricarico = builder.get_object("spinbutton1")
        self.dg_manodopera = builder.get_object("entry9")
        self.dg_sicurezza = builder.get_object("spinbutton5")
        self.dg_valuta = builder.get_object("entry10")
        self.dg_nome_lavoro = builder.get_object("textbuffer_nome_lavoro")
        self.dg_nome_lavoro.connect("changed", self.Attiva_pulsanti_dati_generali_anagrafica)
        self.dg_cliente = builder.get_object("textbuffer_cliente")
        self.dg_cliente.connect("changed", self.Attiva_pulsanti_dati_generali_anagrafica)
        self.dg_indirizzo = builder.get_object("textbuffer_indirizzo")
        self.dg_indirizzo.connect("changed", self.Attiva_pulsanti_dati_generali_anagrafica)
        self.dg_comune = builder.get_object("textbuffer_comune")
        self.dg_comune.connect("changed", self.Attiva_pulsanti_dati_generali_anagrafica)
        self.dg_provincia = builder.get_object("textbuffer_provincia")
        self.dg_provincia.connect("changed", self.Attiva_pulsanti_dati_generali_anagrafica)
        self.dg_redattore = builder.get_object("textbuffer_redattore")
        self.dg_redattore.connect("changed", self.Attiva_pulsanti_dati_generali_anagrafica)
        # imposta treeview per capitoli e categorie dei dati generali
        self.treeview_dg_supercapitoli = builder.get_object("treeview8")
        self.crea_treeview_dati_generali (self.treeview_dg_supercapitoli, "Supercapitolo")
        self.treeview_dg_capitoli = builder.get_object("treeview9")
        self.crea_treeview_dati_generali (self.treeview_dg_capitoli, "Capitolo")
        self.treeview_dg_subcapitoli = builder.get_object("treeview10")
        self.crea_treeview_dati_generali (self.treeview_dg_subcapitoli, "Subcapitolo")
        self.treeview_dg_supercategorie = builder.get_object("treeview11")
        self.crea_treeview_dati_generali (self.treeview_dg_supercategorie, "Supercategoria")
        self.treeview_dg_categorie = builder.get_object("treeview12")
        self.crea_treeview_dati_generali (self.treeview_dg_categorie, "Categoria")
        self.treeview_dg_subcategorie = builder.get_object("treeview13")
        self.crea_treeview_dati_generali (self.treeview_dg_subcategorie, "Subcategoria")

        ###########################################
        # crea i liststore per le tipologie e le categorie di appalto (x COMPUTO)
        ###########################################
        # imposta il modello per le tipologie di appalto
        self.liststore_CONTRACT_TYPE = Gtk.ListStore(str)
        # imposta il primo elemento 
        self.CNR_TYPE_NULL_STR = "-"
        self.liststore_CONTRACT_TYPE.append([self.CNR_TYPE_NULL_STR])
        # ricava glia altri elementi dalle impostazioni della lib. Preventa
        for cntr_type in preventa_settings["default_contract_type"]:
            self.liststore_CONTRACT_TYPE.append([cntr_type])
        # imposta il modello per le categorie di appalto
        self.liststore_CONTRACT_CATEGORY = Gtk.ListStore(str)
        # imposta il primo elemento 
        self.liststore_CONTRACT_CATEGORY.append([self.CNR_TYPE_NULL_STR])
        # ricava glia altri elementi dalle impostazioni della lib. Preventa
        for cntr_cat in preventa_settings["default_contract_category"]:
            self.liststore_CONTRACT_CATEGORY.append([cntr_cat])
        # configura il combobox per scelta tipologie di appalto
        self.box_tip_appalto = builder.get_object("combobox12")
        self.box_tip_appalto.pack_start(cell, True)
        self.box_tip_appalto.add_attribute(cell, 'text', 0)
        self.box_tip_appalto.set_model(self.liststore_CONTRACT_TYPE)
        self.box_tip_appalto.set_active(0)
        self.box_tip_appalto.connect("changed", self.on_combobox_appalto_changed)
        
        ###########################################
        # imposta i liststore e le treeview COMPUTO
        ###########################################
        self.liststore_computo = Gtk.ListStore(int, str, str, str, str, str,
                           str, str, str, str, str, str, str, str, str, str)
        self.treeview_computo = builder.get_object("treeview3")
        self.treeview_computo.set_model(self.liststore_computo)
        # imposta la possibilita di selezione multipla
        selection = self.treeview_computo.get_selection()
        selection.set_mode(Gtk.SelectionMode.MULTIPLE)
        # setup the signals
        self.treeview_computo.connect("cursor-changed", self.row_selected_computo)
        ###########################################
        #imposta menu popup per computo
        self.popup_computo = Gtk.Menu()
        self.popup_computo.items = [None]*18
        # utilizza elementi già esistenti nell'interfaccia grafica
        self.popup_computo.items[0] = Gtk.MenuItem("Copia Articolo")
        self.popup_computo.items[1] = Gtk.MenuItem("Incolla Articolo")
        self.popup_computo.items[2] = Gtk.MenuItem("Cancella Articolo")
        self.popup_computo.items[3] = Gtk.SeparatorMenuItem()
        self.popup_computo.items[4] = Gtk.MenuItem("Inserisci Nuovo Articolo")
        self.popup_computo.items[5] = Gtk.MenuItem("Duplica Articolo Esistente")
        self.popup_computo.items[6] = Gtk.SeparatorMenuItem()
        self.popup_computo.items[7] = Gtk.MenuItem("Modifica Blocco Articoli")
        self.popup_computo.items[8] = Gtk.SeparatorMenuItem()
        self.popup_computo.items[9] = Gtk.MenuItem("Apri Listini METEL")
        self.popup_computo.items[10] = Gtk.SeparatorMenuItem()
        self.popup_computo.items[11] = Gtk.CheckMenuItem("Nascondi Voce")
        self.popup_computo.items[12] = Gtk.CheckMenuItem("Nascondi Materiali")
        self.popup_computo.items[13] = Gtk.CheckMenuItem("Nascondi Ricarico")
        self.popup_computo.items[14] = Gtk.CheckMenuItem("Nascondi Tempo Inst.")
        self.popup_computo.items[15] = Gtk.CheckMenuItem("Nascondi Sicurezza")
        self.popup_computo.items[16] = Gtk.CheckMenuItem("Nascondi Tipologia App.")
        self.popup_computo.items[17] = Gtk.CheckMenuItem("Nascondi Categoria Lav.")
        computo_callback = {0: self.copia_articolo_computo,
                            1: self.incolla_articolo_computo,
                            2: self.cancella_articolo_computo,
                            3: None,
                            4: self.nuovo_articolo_computo,
                            5: self.duplica_articolo_computo,
                            6: None,
                            7: self.modifica_blocco_articoli,
                            8: None,
                            9: self.carica_listini_metel,
                           10: None,
                           11: self.nascondi_colonna_computo_desc_voce,
                           12: self.nascondi_colonna_computo_materiali,
                           13: self.nascondi_colonna_computo_ricarico,
                           14: self.nascondi_colonna_computo_tempo_inst,
                           15: self.nascondi_colonna_computo_sicurezza,
                           16: self.nascondi_colonna_computo_tipo_app,
                           17: self.nascondi_colonna_computo_cat_lav}
        for n in range(18):
            self.popup_computo.append(self.popup_computo.items[n])
            self.popup_computo.items[n].show()
            if n not in [3,6,8,10]:
                self.popup_computo.items[n].connect("activate", computo_callback[n])
        ###########################################
        # create the TreeViewColumns to display the data
        self.treeview_computo.columns = [None]*16
        self.treeview_computo.columns[0] = Gtk.TreeViewColumn(_('No.'))
        self.treeview_computo.columns[1] = Gtk.TreeViewColumn(_('Tariffa'))
        self.treeview_computo.columns[2] = Gtk.TreeViewColumn(_('Descrizione Codice'))
        self.treeview_computo.columns[3] = Gtk.TreeViewColumn(_('Descrizione Voce'))
        self.treeview_computo.columns[4] = Gtk.TreeViewColumn(_('u.m.'))
        self.treeview_computo.columns[5] = Gtk.TreeViewColumn(_('Quantita'))
        self.treeview_computo.columns[6] = Gtk.TreeViewColumn(_('Materiali'))
        self.treeview_computo.columns[7] = Gtk.TreeViewColumn(_('Ric.'))
        self.treeview_computo.columns[8] = Gtk.TreeViewColumn(_('Tempo'))
        self.treeview_computo.columns[9] = Gtk.TreeViewColumn(_('Sicurezza'))
        self.treeview_computo.columns[10] = Gtk.TreeViewColumn(_('Prezzo Unitario'))
        self.treeview_computo.columns[11] = Gtk.TreeViewColumn(_('Prezzo Totale'))
        self.treeview_computo.columns[12] = Gtk.TreeViewColumn(_('Note di misurazione'))
        self.treeview_computo.columns[13] = Gtk.TreeViewColumn(_('Tipologia Appalto'))
        self.treeview_computo.columns[14] = Gtk.TreeViewColumn(_('Categoria Lavorazioni'))
        self.treeview_computo.columns[15] = Gtk.TreeViewColumn(_('Data inserimento'))
        for n in range(16):
            # add columns to treeview
            self.treeview_computo.append_column(self.treeview_computo.columns[n])
            if n not in [13,14]:
                # create a CellRenderers to render the data
                self.treeview_computo.columns[n].cell = Gtk.CellRendererText()
            elif n in [13,14]:
                # create a CellRendererCombo to render data in cell 13,14
                self.treeview_computo.columns[n].cell = Gtk.CellRendererCombo()
                if n==13: model=self.liststore_CONTRACT_TYPE
                elif n==14: model=self.liststore_CONTRACT_CATEGORY
                self.treeview_computo.columns[n].cell.set_property("model", model)
                # imposta il colore della cella (grigio chiaro)
                self.treeview_computo.columns[n].cell.set_property("cell-background-rgba", self.colour_gray)
                self.treeview_computo.columns[n].cell.set_property("background-rgba", self.colour_gray)
                # imposta la colonna di testo del modello
                self.treeview_computo.columns[n].cell.set_property("text-column", 0)
                # disattiva l'immissione manuale di testo nella cella
                self.treeview_computo.columns[n].cell.set_property("has-entry", False)
            # set the cells as editable
            if n not in [15]:
                self.treeview_computo.columns[n].cell.set_property('editable', True)
                self.treeview_computo.columns[n].cell.connect('edited', self.cell_computo_edited, n)
            # add the cells to the columns
            self.treeview_computo.columns[n].pack_start(self.treeview_computo.columns[n].cell, True)
            # set the cell attributes to the appropriate liststore column
            self.treeview_computo.columns[n].add_attribute(self.treeview_computo.columns[n].cell, 'text', n)
            # make treeview searchable
            self.treeview_computo.set_search_column(n)
            # Allow sorting on the column
            self.treeview_computo.columns[n].set_sort_column_id(n)
            # set column resizable
            self.treeview_computo.columns[n].set_resizable(True)

        ###########################################
        # imposta i liststore e le treeview LISTINO
        ###########################################
        self.liststore_listino = Gtk.ListStore(int, str, str, str, str, str, 
                                               str, str, str, str, str)
        self.treeview_listino = builder.get_object("treeview4")
        self.treeview_listino.set_model(self.liststore_listino)
        # imposta la possibilita di selezione multipla
        selection = self.treeview_listino.get_selection()
        selection.set_mode(Gtk.SelectionMode.MULTIPLE)
        # setup the signals        
        self.treeview_listino.connect("cursor-changed", self.row_selected_listino)
        ###########################################
        #imposta menu popup per listino
        self.popup_listino = Gtk.Menu()
        self.popup_listino.items = [None]*5
        self.popup_listino.items[0] = Gtk.MenuItem("Nuovo")
        self.popup_listino.items[1] = Gtk.MenuItem("Copia")
        self.popup_listino.items[2] = Gtk.MenuItem("Incolla")
        self.popup_listino.items[3] = Gtk.MenuItem("Elimina")
        self.popup_listino.items[4] = Gtk.MenuItem("Calcola Totale")
        listino_callback = {0: self.nuovo_articolo_listino,
                        1: self.copia_articolo_listino,
                        2: self.incolla_articolo_listino,
                        3: self.cancella_articolo_listino,
                        4: self.calcola_prezzo_articoli_listino}
        for n in range(5):
            self.popup_listino.append(self.popup_listino.items[n])
            self.popup_listino.items[n].show()
            self.popup_listino.items[n].connect("activate", listino_callback[n])
        ###########################################
        # create the TreeViewColumns to display the data
        self.treeview_listino.columns = [None]*10
        self.treeview_listino.columns[0] = Gtk.TreeViewColumn(_('No.'))
        self.treeview_listino.columns[1] = Gtk.TreeViewColumn(_('Tariffa'))
        self.treeview_listino.columns[2] = Gtk.TreeViewColumn(_('Codice'))
        self.treeview_listino.columns[3] = Gtk.TreeViewColumn(_('Descrizione'))
        self.treeview_listino.columns[4] = Gtk.TreeViewColumn(_('u.m.'))
        self.treeview_listino.columns[5] = Gtk.TreeViewColumn(_('Quantita'))
        self.treeview_listino.columns[6] = Gtk.TreeViewColumn(_('Prezzo Unitario'))
        self.treeview_listino.columns[7] = Gtk.TreeViewColumn(_('Sconto'))
        self.treeview_listino.columns[8] = Gtk.TreeViewColumn(_('Accessori'))
        self.treeview_listino.columns[9] = Gtk.TreeViewColumn(_('Prezzo Totale'))
        for n in range(10):
            # add columns to treeview
            self.treeview_listino.append_column(self.treeview_listino.columns[n])
            # create a CellRenderers to render the data
            self.treeview_listino.columns[n].cell = Gtk.CellRendererText()
            # set the cells as editable
            self.treeview_listino.columns[n].cell.set_property('editable', True)
            self.treeview_listino.columns[n].cell.connect('edited', self.cell_listino_edited, n)
            # add the cells to the columns
            self.treeview_listino.columns[n].pack_start(self.treeview_listino.columns[n].cell, True)
            # set the cell attributes to the appropriate liststore column
            self.treeview_listino.columns[n].add_attribute(self.treeview_listino.columns[n].cell, 'text', n)
            # set column resizable
            self.treeview_listino.columns[n].set_resizable(True)

        ###########################################
        # imposta i liststore e le treeview EPU
        ###########################################
        # combobox di selezionedel campo di ricerca EPU
        self.liststore_box_ricerca_epu = Gtk.ListStore(str)
        self.box_campo_ricerca_epu = builder.get_object("combobox5")
        cell = Gtk.CellRendererText()
        self.box_campo_ricerca_epu.pack_start(cell, True)
        self.box_campo_ricerca_epu.add_attribute(cell, 'text', 0)
        self.box_campo_ricerca_epu.set_model(self.liststore_box_ricerca_epu)
        lista_campi = [_("Tariffa"), _("Descrizione codice"), _("Descrizione voce")]
        for campo in lista_campi:
            self.liststore_box_ricerca_epu.append([campo])
        self.box_campo_ricerca_epu.set_active(0)
        # ricerca nella treeview EPU
        self.entry_ricerca_epu = builder.get_object("entry16")
        self.entry_ricerca_epu.connect("activate", self.cerca_articolo_epu)
        self.entry_ricerca_epu.connect("icon-press", self.cerca_articolo_epu)
        # create the TreeView
        self.treeview_epu = builder.get_object("treeview1")
        self.treestore_epu = Gtk.TreeStore(str, str, str, str)
        self.treeview_epu.set_model(self.treestore_epu)
        # setup the signals
        self.treeview_epu.connect("cursor-changed", self.row_selected_epu)
        # setup columns
        self.treeview_epu.columns = [None]*3
        self.treeview_epu.columns[0] = Gtk.TreeViewColumn(_('Tariffa'))
        self.treeview_epu.columns[1] = Gtk.TreeViewColumn(_('Descrizione Codice'))
        self.treeview_epu.columns[2] = Gtk.TreeViewColumn(_('Descrizione Voce'))
        for n in range(3):
            # add columns to treeview
            self.treeview_epu.append_column(self.treeview_epu.columns[n])
            # create a CellRenderers to render the data
            self.treeview_epu.columns[n].cell = Gtk.CellRendererText()
            # add the cells to the columns
            self.treeview_epu.columns[n].pack_start(self.treeview_epu.columns[n].cell, True)
            # set the cell attributes to the appropriate liststore column
            self.treeview_epu.columns[n].add_attribute(self.treeview_epu.columns[n].cell, 'text', n)
            # make treeview searchable
            self.treeview_epu.set_search_column(n)
            # Allow sorting on the column
            self.treeview_epu.columns[n].set_sort_column_id(n)
            # set column resizable
            self.treeview_epu.columns[n].set_resizable(True)
        ###########################################
        #imposta menu popup per EPU
        self.popup_epu = Gtk.Menu()
        self.popup_epu.items = [None]*6
        self.popup_epu.items[0] = Gtk.MenuItem("Copia")
        self.popup_epu.items[1] = Gtk.MenuItem("Inserisci")
        self.popup_epu.items[2] = Gtk.SeparatorMenuItem()
        self.popup_epu.items[3] = Gtk.RadioMenuItem(group=None, label="Elenco Prezzi Unitari")
        self.popup_epu.items[4] = Gtk.RadioMenuItem(group=self.popup_epu.items[3], label="Archivio Principale")
        self.popup_epu.items[5] = Gtk.RadioMenuItem(group=self.popup_epu.items[3], label="Carica archivio da file...")
        epu_callback = {0: self.copia_articolo_computo,
                        1: self.incolla_articolo_archivio_principale,
                        2: None,
                        3: self.carica_archivio_epu,
                        4: self.carica_archivio_epu,
                        5: self.carica_archivio_epu}
        for n in range(6):
            self.popup_epu.append(self.popup_epu.items[n])
            self.popup_epu.items[n].show()
            if n not in [2]:
                self.popup_epu.items[n].connect("activate", epu_callback[n])

        #############################################
        # imposta i liststore e le treeview CATEGORIE
        #############################################
        self.treeview_categorie = builder.get_object("treeview2")
        self.treestore_categorie = Gtk.TreeStore(str, str, str)
        self.treeview_categorie.set_model(self.treestore_categorie)
        # setup the signals
        self.treeview_categorie.connect("cursor-changed", self.row_selected_categorie)
        # setup the columns
        self.treeview_categorie.columns = [None]*3
        self.treeview_categorie.columns[0] = Gtk.TreeViewColumn(_('No.'))
        self.treeview_categorie.columns[1] = Gtk.TreeViewColumn(_('Nome Categoria'))
        self.treeview_categorie.columns[2] = Gtk.TreeViewColumn(_('Totale Parziale'))
        for n in range(3):
            # add columns to treeview
            self.treeview_categorie.append_column(self.treeview_categorie.columns[n])
            # create a CellRenderers to render the data
            self.treeview_categorie.columns[n].cell = Gtk.CellRendererText()
            # add the cells to the columns
            self.treeview_categorie.columns[n].pack_start(self.treeview_categorie.columns[n].cell, True)
            # set the cell attributes to the appropriate liststore column
            self.treeview_categorie.columns[n].add_attribute(self.treeview_categorie.columns[n].cell, 'text', n)
            # make treeview searchable
            self.treeview_categorie.set_search_column(n)
            # Allow sorting on the column
            self.treeview_categorie.columns[n].set_sort_column_id(n)
            # set column resizable
            self.treeview_categorie.columns[n].set_resizable(True)
        self.treeview_categorie.set_expander_column(self.treeview_categorie.columns[0])
        # set only the cell 1 'Nome Categoria' as editable
        self.treeview_categorie.columns[1].cell.set_property('editable', True)
        self.treeview_categorie.columns[1].cell.connect('edited', self.cell_nome_categoria_edited, 1)
        ###########################################
        #imposta menu popup per CATEGORIE
        self.popup_categorie = Gtk.Menu()
        self.popup_categorie.items = [None]*5
        self.popup_categorie.items[0] = Gtk.MenuItem("Nuova Categoria")
        self.popup_categorie.items[1] = Gtk.MenuItem("Elimina Categoria")
        self.popup_categorie.items[2] = Gtk.MenuItem("Aggiorna Vista")
        self.popup_categorie.items[3] = Gtk.SeparatorMenuItem()
        self.popup_categorie.items[4] = Gtk.CheckMenuItem("_Visualizza categorie vuote")
        
        categorie_callback = {0: self.nuova_categoria,
                        1: self.cancella_categoria,
                        2: self.Aggiorna_treeview_categorie,
                        3: None,
                        4: self.Visualizza_categorie_vuote}
        for n in range(5):
            self.popup_categorie.append(self.popup_categorie.items[n])
            self.popup_categorie.items[n].set_use_underline(True)
            self.popup_categorie.items[n].show()
            if n not in [3]:
                self.popup_categorie.items[n].connect("activate", categorie_callback[n])

        ###########################################
        # imposta la treeview LISTA LAVORAZIONI
        ###########################################
        # imposta i combobox del capitolo
        cell = Gtk.CellRendererText()
        self.box_supercapitolo_lavorazioni = builder.get_object("combobox2")
        self.box_supercapitolo_lavorazioni.set_model(self.liststore_supercapitolo)
        self.box_supercapitolo_lavorazioni.pack_start(cell, True)
        self.box_supercapitolo_lavorazioni.add_attribute(cell, 'text', 1)
        self.box_capitolo_lavorazioni = builder.get_object("combobox3")
        self.box_capitolo_lavorazioni.pack_start(cell, True)
        self.box_capitolo_lavorazioni.add_attribute(cell, 'text', 1)
        self.box_capitolo_lavorazioni.set_model(self.liststore_capitolo)
        self.box_subcapitolo_lavorazioni = builder.get_object("combobox4")
        self.box_subcapitolo_lavorazioni.pack_start(cell, True)
        self.box_subcapitolo_lavorazioni.add_attribute(cell, 'text', 1)
        self.box_subcapitolo_lavorazioni.set_model(self.liststore_subcapitolo)
        # create the TreeView
        self.treeview_lavorazioni = builder.get_object("treeview6")
        self.liststore_lavorazioni = Gtk.ListStore(str, str, str, str,  str, 
            str, str, str, str, str,  str, str, str)
        self.treeview_lavorazioni.set_model(self.liststore_lavorazioni)
        # setup the signals        
        self.treeview_lavorazioni.connect("cursor-changed", self.row_selected_lista_lavorazioni)
        # imposta la possibilita di selezione multipla
        selection = self.treeview_lavorazioni.get_selection()
        selection.set_mode(Gtk.SelectionMode.MULTIPLE)
        cell = Gtk.CellRendererText()
        self.treeview_lavorazioni.columns = [None]*13
        self.treeview_lavorazioni.columns[0] = Gtk.TreeViewColumn('Tariffa')
        self.treeview_lavorazioni.columns[1] = Gtk.TreeViewColumn('Descrizione Codice')
        self.treeview_lavorazioni.columns[2] = Gtk.TreeViewColumn('Descrizione Voce')
        self.treeview_lavorazioni.columns[3] = Gtk.TreeViewColumn('Descrizione Estesa')
        self.treeview_lavorazioni.columns[4] = Gtk.TreeViewColumn('u.m.')
        self.treeview_lavorazioni.columns[5] = Gtk.TreeViewColumn('Materiali')
        self.treeview_lavorazioni.columns[6] = Gtk.TreeViewColumn('Ric.')
        self.treeview_lavorazioni.columns[7] = Gtk.TreeViewColumn('Temp.')
        self.treeview_lavorazioni.columns[8] = Gtk.TreeViewColumn('Sicurezza')
        self.treeview_lavorazioni.columns[9] = Gtk.TreeViewColumn('Quantita')
        self.treeview_lavorazioni.columns[10] = Gtk.TreeViewColumn('Prezzo Unitario')
        self.treeview_lavorazioni.columns[11] = Gtk.TreeViewColumn('Prezzo Totale')
        self.treeview_lavorazioni.columns[12] = Gtk.TreeViewColumn('Note')
        for n in range(13):
            # add columns to treeview
            self.treeview_lavorazioni.append_column(self.treeview_lavorazioni.columns[n])
            # create a CellRenderers to render the data
            self.treeview_lavorazioni.columns[n].cell = Gtk.CellRendererText()
            # set the cells as editable
            self.treeview_lavorazioni.columns[n].cell.set_property('editable', True)
            self.treeview_lavorazioni.columns[n].cell.connect('edited', self.cell_lista_lavorazioni_edited, n)
            # add the cells to the columns
            self.treeview_lavorazioni.columns[n].pack_start(self.treeview_lavorazioni.columns[n].cell, True)
            # set the cell attributes to the appropriate liststore column
            self.treeview_lavorazioni.columns[n].add_attribute(self.treeview_lavorazioni.columns[n].cell, 'text', n)
            # make treeview searchable
            self.treeview_lavorazioni.set_search_column(n)
            # Allow sorting on the column
            self.treeview_lavorazioni.columns[n].set_sort_column_id(n)
            # set column resizable
            self.treeview_lavorazioni.columns[n].set_resizable(True)
        ###########################################
        # imposta menu popup per LISTA LAVORAZIONI
        self.popup_lavorazioni = Gtk.Menu()
        self.popup_lavorazioni.items = [None]*14
        self.popup_lavorazioni.items[0] = Gtk.MenuItem("Copia Articolo")
        self.popup_lavorazioni.items[1] = Gtk.MenuItem("Incolla Articolo")
        self.popup_lavorazioni.items[2] = Gtk.MenuItem("Nuovo Articolo")
        self.popup_lavorazioni.items[3] = Gtk.MenuItem("Duplica Articolo")
        self.popup_lavorazioni.items[4] = Gtk.MenuItem("Cancella Articolo")
        self.popup_lavorazioni.items[5] = Gtk.SeparatorMenuItem()
        self.popup_lavorazioni.items[6] = Gtk.MenuItem("Modifica Blocco Voci")
        self.popup_lavorazioni.items[7] = Gtk.SeparatorMenuItem()
        self.popup_lavorazioni.items[8] = Gtk.CheckMenuItem("Nascondi Descr.Voce")
        self.popup_lavorazioni.items[9] = Gtk.CheckMenuItem("Nascondi Descr.Estesa")
        self.popup_lavorazioni.items[10] = Gtk.CheckMenuItem("Nascondi Materiali")
        self.popup_lavorazioni.items[11] = Gtk.CheckMenuItem("Nascondi Ricarico")
        self.popup_lavorazioni.items[12] = Gtk.CheckMenuItem("Nascondi Tempi inst.")
        self.popup_lavorazioni.items[13] = Gtk.CheckMenuItem("Nascondi Sicurezza")
        lavorazioni_callback = {0: self.copia_articolo_lista_lavorazioni,
                        1: self.incolla_articolo_lista_lavorazioni,
                        2: self.nuovo_articolo_lista_lavorazioni,
                        3: self.duplica_articolo_lista_lavorazioni,
                        4: self.cancella_articolo_lista_lavorazioni,
                        5: None,
                        6: self.modifica_blocco_articoli,
                        7: None,
                        8: self.nascondi_colonna_lavorazioni_desc_voce,
                        9: self.nascondi_colonna_lavorazioni_desc_estesa,
                       10: self.nascondi_colonna_lavorazioni_materiali,
                       11: self.nascondi_colonna_lavorazioni_ricarico,
                       12: self.nascondi_colonna_lavorazioni_tempo_inst,
                       13: self.nascondi_colonna_lavorazioni_sicurezza}
        for n in range(14):
            self.popup_lavorazioni.append(self.popup_lavorazioni.items[n])
            self.popup_lavorazioni.items[n].show()
            if n not in [5, 7]:
                self.popup_lavorazioni.items[n].connect("activate", lavorazioni_callback[n])

        #############################################
        # imposta i liststore e le treeview CAPITOLI
        #############################################
        self.treeview_capitoli = builder.get_object("treeview5")
        self.treestore_capitoli = Gtk.TreeStore(str, str, str)
        self.treeview_capitoli.set_model(self.treestore_capitoli)
        # setup the signals
        self.treeview_capitoli.connect("cursor-changed", self.row_selected_capitoli)
        # setup the columns
        self.treeview_capitoli.columns = [None]*3
        self.treeview_capitoli.columns[0] = Gtk.TreeViewColumn('No.')
        self.treeview_capitoli.columns[1] = Gtk.TreeViewColumn('Nome Capitolo')
        self.treeview_capitoli.columns[2] = Gtk.TreeViewColumn('Totale Parziale')
        for n in range(3):
            # add columns to treeview
            self.treeview_capitoli.append_column(self.treeview_capitoli.columns[n])
            # create a CellRenderers to render the data
            self.treeview_capitoli.columns[n].cell = Gtk.CellRendererText()
            # add the cells to the columns
            self.treeview_capitoli.columns[n].pack_start(self.treeview_capitoli.columns[n].cell, True)
            # set the cell attributes to the appropriate liststore column
            self.treeview_capitoli.columns[n].add_attribute(self.treeview_capitoli.columns[n].cell, 'text', n)
            # make treeview searchable
            self.treeview_capitoli.set_search_column(n)
            # Allow sorting on the column
            self.treeview_capitoli.columns[n].set_sort_column_id(n)
            # set column resizable
            self.treeview_capitoli.columns[n].set_resizable(True)
        self.treeview_capitoli.set_expander_column(self.treeview_capitoli.columns[0])
        # set only the cell 1 'Nome Capitolo' as editable
        self.treeview_capitoli.columns[1].cell.set_property('editable', True)
        self.treeview_capitoli.columns[1].cell.connect('edited', self.cell_nome_capitolo_edited, 1)
        ###########################################
        # imposta menu popup per capitoli
        self.popup_capitoli = Gtk.Menu()
        self.popup_capitoli.items = [None]*5
        self.popup_capitoli.items[0] = Gtk.MenuItem("Nuovo Capitolo")
        self.popup_capitoli.items[1] = Gtk.MenuItem("Elimina Capitolo")
        self.popup_capitoli.items[2] = Gtk.MenuItem("Aggiorna Vista")
        self.popup_capitoli.items[3] = Gtk.SeparatorMenuItem()
        self.popup_capitoli.items[4] = Gtk.CheckMenuItem("_Visualizza capitoli vuoti")
        capitoli_callback = {0: self.nuovo_capitolo,
                        1: self.cancella_capitolo,
                        2: self.Aggiorna_treeview_capitoli,
                        3: None,
                        4: self.Visualizza_capitoli_vuoti}
        for n in range(5):
            self.popup_capitoli.append(self.popup_capitoli.items[n])
            self.popup_capitoli.items[n].show()
            if n not in [3]:
                self.popup_capitoli.items[n].connect("activate", capitoli_callback[n])

        #############################################
        # Crezione della TOOLBAR
        #############################################
        hbox = builder.get_object("box3")
        # toolbar will be horizontal, with both icons and text, and
        # with 5pxl spaces between items and with an handlebox
        self.toolbar = Gtk.Toolbar()
        self.toolbar.set_orientation(Gtk.Orientation.HORIZONTAL)
        self.toolbar.set_style (Gtk.ToolbarStyle.BOTH_HORIZ)
        self.toolbar.set_show_arrow(False)
        self.toolbar.set_border_width(5)
        hbox.pack_start(self.toolbar, True, True, 5)
        #creazione di pulsanti e altro
        self.toolbar.buttons = [None] * 12
        self.toolbar.buttons[0] = Gtk.ToolButton('preventa-new-articolo')
        self.toolbar.buttons[1] = Gtk.ToolButton(Gtk.STOCK_CUT)
        self.toolbar.buttons[2] = Gtk.ToolButton(Gtk.STOCK_COPY)
        self.toolbar.buttons[3] = Gtk.ToolButton(Gtk.STOCK_PASTE)
        self.toolbar.buttons[4] = Gtk.ToolButton('preventa-erase-articolo')
        self.toolbar.buttons[5] = Gtk.ToolButton('preventa-insert-articolo')
        self.toolbar.buttons[6] = Gtk.SeparatorToolItem()
        self.toolbar.buttons[7] = Gtk.ToolButton(Gtk.STOCK_FULLSCREEN)
        self.toolbar.buttons[8] = Gtk.ToolButton(Gtk.STOCK_LEAVE_FULLSCREEN)
        self.toolbar.buttons[9] = Gtk.SeparatorToolItem()
        self.toolbar.buttons[10] = Gtk.ToolButton('preventa-reload-computo')
        self.toolbar.buttons[11] = Gtk.ToolButton('preventa-listino')
        # dizionario delle callback dei pulsanti
        toolbar_callback = {0: self.menu_nuovo,
                            1: self.menu_taglia,
                            2: self.menu_copia,
                            3: self.menu_incolla,
                            4: self.menu_cancella,
                            5: self.menu_duplica,
                            6: None,
                            7: self.Fullscreen,
                            8: self.Unfullscreen,
                            9: None,
                           10: self.Update,
                           11: self.carica_listini_metel}
        toolbar_tooltip =  {0: "Inserisci un nuovo articolo di Epu",
                            1: "Taglia l'articolo selezionato",
                            2: "Copia l'articolo selezionato",
                            3: "Incolla l'articolo selezionato",
                            4: "Cancella l'articolo selezionato",
                            5: "Duplica l'articolo selezionato e inserisce un nuovo articolo di Epu",
                            6: "",
                            7: "Schermo intero",
                            8: "Schermo normale",
                            9: "",
                           10: "Aggiorna tutto",
                           11: "Carica listini METEL"}
        for n in range(12):
            self.toolbar.insert(self.toolbar.buttons[n], n)
            if n not in [6,9]: #per escludere i separatori delle seguenti istruzioni
                self.toolbar.buttons[n].set_tooltip_text(toolbar_tooltip[n])
                self.toolbar.buttons[n].connect_object("clicked", toolbar_callback[n], "")
            self.toolbar.buttons[n].show()
        if settings["show_toolbar"]:
            self.toolbar.show()

        ####################################################################
        # Supporto Drag'n Drop per le treeview COMPUTO, LAVORAZIONI, LISTINO
        ####################################################################
        # Costanti per il supporto Drag n'Drop
        self.TIPO_ART_COMPUTO = 111
        self.TIPO_ART_EPU = 112
        self.TIPO_ART_LISTINO = 113
        self.TIPO_TEXT_PLAIN = 0
        self.ARTICOLI_EPU = "preventa/epu"
        self.ARTICOLI_COMPUTO = "preventa/computo"
        self.ARTICOLI_LISTINO = "preventa/listino"
        self.TEXT_PLAIN = "text/plain"
        START_BUTTON = Gdk.ModifierType.BUTTON1_MASK
        DRAG_ACTION = Gdk.DragAction.COPY
        DEST_DEFAULT = Gtk.DestDefaults.ALL
        FLAG_APP = Gtk.TargetFlags.SAME_APP
        FLAG_WIDGET = Gtk.TargetFlags.SAME_WIDGET
        # target per il Drag'n Drop
        self.target_epu = Gdk.Atom.intern(self.ARTICOLI_EPU, True)
        self.target_cmp = Gdk.Atom.intern(self.ARTICOLI_COMPUTO, True)
        self.target_lst = Gdk.Atom.intern(self.ARTICOLI_LISTINO, True)
        # setup EPU the Gtk.TargetList for drag dest and source
        targetlist_epu = Gtk.TargetList.new([])
        targetlist_epu.add(self.target_epu, FLAG_APP, self.TIPO_ART_EPU)
        # setup COMPUTO the Gtk.TargetList for drag destination
        targetlist_cmpd = Gtk.TargetList.new([])
        targetlist_cmpd.add(self.target_cmp, 0, self.TIPO_ART_COMPUTO)
        targetlist_cmpd.add(self.target_epu, FLAG_APP, self.TIPO_ART_EPU)
        # setup COMPUTO the Gtk.TargetList for drag source
        targetlist_cmps = Gtk.TargetList.new([])
        targetlist_cmps.add(self.target_cmp, FLAG_WIDGET, self.TIPO_ART_COMPUTO)
        # setup LISTINO the Gtk.TargetList for drag destination
        targetlist_lst = Gtk.TargetList.new([])
        targetlist_lst.add(self.target_lst, 0, self.TIPO_ART_LISTINO)
        # setup LAVORAZIONI the Gtk.TargetList for drop destination
        targetlist_lavd = Gtk.TargetList.new([])
        targetlist_lavd.add(self.target_epu, 0, self.TIPO_ART_EPU)
        # setup LAVORAZIONI the Gtk.TargetList for drag source
        targetlist_lavs = Gtk.TargetList.new([])
        targetlist_lavs.add(self.target_epu, FLAG_WIDGET, self.TIPO_ART_EPU)
        # EPU Treeview impostazioni Drag source
        self.treeview_epu.enable_model_drag_source(START_BUTTON,[], DRAG_ACTION)
        self.treeview_epu.connect("drag_data_get", self.treeview_drag_data_get)
        self.treeview_epu.drag_source_set_target_list(targetlist_epu)
        # COMPUTO Treeview Drag destination
        self.treeview_computo.enable_model_drag_dest([], DRAG_ACTION)
        self.treeview_computo.connect("drag_drop", self.cb_drag_drop)
        self.treeview_computo.connect("drag_data_received", self.cb_drag_data_received)
        self.treeview_computo.drag_dest_set_target_list(targetlist_cmpd)
        self.treeview_computo.drag_dest_add_text_targets()
        # COMPUTO Treeview Drag source
        self.treeview_computo.enable_model_drag_source(START_BUTTON,[], DRAG_ACTION)
        self.treeview_computo.connect("drag_data_get", self.treeview_drag_data_get)
        self.treeview_computo.connect("drag_begin", self.treeview_drag_begin)
        self.treeview_computo.drag_source_set_target_list(targetlist_cmps)
        # LAVORAZIONI Treeview Drag destination
        self.treeview_lavorazioni.connect("drag_data_received", self.cb_drag_data_received)
        self.treeview_lavorazioni.enable_model_drag_dest([], DRAG_ACTION)
        self.treeview_lavorazioni.drag_dest_set_target_list(targetlist_lavd)
        self.treeview_lavorazioni.drag_dest_add_text_targets()
        # LAVORAZIONI Treeview Drag Source
        self.treeview_lavorazioni.enable_model_drag_source(START_BUTTON, [], DRAG_ACTION)
        self.treeview_lavorazioni.drag_source_set_target_list(targetlist_lavs)
        # LISTINO Treeview Drag destination
        self.treeview_listino.connect("drag_drop", self.cb_drag_drop)
        self.treeview_listino.connect("drag_data_received", self.cb_drag_data_received)
        self.treeview_listino.enable_model_drag_dest([], DRAG_ACTION)
        self.treeview_listino.drag_dest_set_target_list(targetlist_lst)
        self.treeview_listino.drag_dest_add_text_targets()
        
        ####################################################################
        # Supporto Drag'n Drop per la finestra principale
        ####################################################################
        self.drag_dest_set(DEST_DEFAULT, [], DRAG_ACTION)
        self.connect("drag_data_received", self.window_drag_data_received)
        self.drag_dest_add_uri_targets()
        
        #############################################
        # Inizializzazione della Clipboard Preventa
        #############################################
        # Crea una clipboard per importare sezioni di computo dall'esterno
        CLIPBOARD_NAME = Gdk.Atom.intern("PREVENTARES_CLIPBOARD", True)
        self.clipboard = Gtk.Clipboard.get(CLIPBOARD_NAME)
        # la clipboard mantiene in memoria ogni tipo di dati
        self.clipboard.set_can_store(None)

        #############################################
        # Aggiorna e popola l'interfaccia grafica
        #############################################
        # imposta i parametri dell'applicazione
        self.setup_settings()
        # aggiorna la maschera dei dati generali
        self.Aggiorna_maschera_dati_generali(self)
        # aggiorna le treeview capitoli e categorie
        self.Aggiorna_liststore_capitoli(self)
        self.Aggiorna_liststore_categorie(self)
        # Aggiorna la Barra di Stato con il totale del computo
        self.Aggiorna_BarraStato_default()
        # nascondi la progressbar
        self.builder.get_object("progressbar1").hide()

################################################################################
###  IMPOSTAZIONI DELL'INTERFACCIA GRAFICA #####################################
################################################################################
    def setup_settings(self):
        """imposta l'applicazione secondo i parametri iniziali"""
        self.popup_categorie.items[4].set_active(settings["show_empty_category"])
        self.popup_capitoli.items[4].set_active(settings["show_empty_chapter"])
        self.popup_lavorazioni.items[8].set_active(
            not settings["show_column_desc_voce_lavorazioni"])
        self.popup_lavorazioni.items[9].set_active(
            not settings["show_column_desc_estesa_lavorazioni"])
        self.popup_lavorazioni.items[10].set_active(
            not settings["show_column_materiali_lavorazioni"])
        self.popup_lavorazioni.items[11].set_active(
            not settings["show_column_ricarico_lavorazioni"])
        self.popup_lavorazioni.items[12].set_active(
            not settings["show_column_tempo_inst_lavorazioni"])
        self.popup_lavorazioni.items[13].set_active(
            not settings["show_column_sicurezza_lavorazioni"])
        self.popup_computo.items[11].set_active(not settings["show_column_desc_voce"])
        self.popup_computo.items[12].set_active(not settings["show_column_materiali"])
        self.popup_computo.items[13].set_active(not settings["show_column_ricarico"])
        self.popup_computo.items[14].set_active(not settings["show_column_tempo_inst"])
        self.popup_computo.items[15].set_active(not settings["show_column_sicurezza"])
        self.popup_computo.items[16].set_active(not settings["show_column_contract"])
        self.popup_computo.items[17].set_active(not settings["show_column_contract"])
        # se è già attivato il calcolo automatico nasconde l'opzione del menù perchè superflua
        preventa_settings = self.preventivo.get_settings()
        if preventa_settings["ricalcolo_automatico_prezzi"]:
            self.popup_listino.items[4].hide()
        # Finestra all'avvio massimizzata
        if settings["maximize_window_at_startup"]:
            self.maximize()
        # Toolbar attiva
        if settings["show_toolbar"]:
            self.toolbar.show()
        else:
            self.toolbar.hide()
        # caricare l'archivio principale
        if settings["load_main_archive"]:
            self.Aggiorna_treeview_epu(self)
        # riferimento ai frame (cornici) visibili
        if not settings["show_mask_article_full"]:
            self.frame_dati_articolo.hide()
            self.frame_elementi_analisi.hide()
            self.frame_descrizioni_voce_codice.hide()
        elif settings["show_mask_article_full"]:
            self.frame_dati_articolo.show()
            self.frame_elementi_analisi.show()
            self.frame_descrizioni_voce_codice.show()
        # disattiva gli elementi capitoli epu e capitoli computo 
        self.frame_capitoli.hide() #capitoli epu
        self.frame_categorie.hide() #capitoli computo
        # setup the notebook position (orizontal/vertical)
        if settings["archive_notebook_orizontal"]: # default vertical
           pos = Gtk.PositionType.LEFT
           ang = 90.0
           orientation = Gtk.Orientation.VERTICAL
        else: 
           pos = Gtk.PositionType.TOP
           ang = 0.0
           orientation = Gtk.Orientation.HORIZONTAL
        # setup the notebooks and labels orientation
        self.notebook_3.set_tab_pos(pos)
        self.notebook_4.set_tab_pos(pos)
        list_labels = [self.label_cat, self.label_cap, 
                       self.label_arc1, self.label_arc2]
        for label in list_labels:
            label.set_angle(ang)
            label.hide() #TODO hide label???
            parent = label.get_parent()
            parent.set_orientation(orientation)
        return

################################################################################
###  CLIPBOARD DI PREVENTARES - GESTIONE DEI DATI ##############################
################################################################################
#TODO possono essere eliminate, non più utilizzate (?)
    def clipboard_get_art(self, clipboard, selectiondata, info, data):
        """codifica in una stringa la lista di articoli e la copia nella clipboard"""
        # converte data in una stringa leggibile dalla funzione eval()
        lista = list()
        if data is None:
            return
        for articolo in data:
            lista.append(str(articolo))
        text = repr(lista)
        target = selectiondata.get_target()
        if target.name() == self.ARTICOLI_COMPUTO:
            selectiondata.set(target, info, text)
        elif target.name() == self.ARTICOLI_EPU:
            selectiondata.set(target, info, text)
        elif target.name() == self.ARTICOLI_LISTINO:
            selectiondata.set(target, info, text)
        else:
            clipboard.set_text(text)
        return

    def clipboard_clear_art(self, clipboard, data):
        """funzione chimata quando si sovrascrive un dato nella clipboard"""
        del data
        return

    def clipboard_cb_articoli(self, clipboard, selectiondata, data):
        """funzione chiamata quando si 'incolla' dalla clipboard, converte una
           stringa in una lista di articoli con la funzione eval()"""
        text = selectiondata.get_text()
        target = selectiondata.get_target().name()
        lista_articoli = clipboard.get_data(target)
        logger.debug(_("Cliboard destination callback: clipboard: {0}; text: {1}; target: {2}; data: {3}".format(clipboard, text, target, data)))
        if text is None:
            lista_articoli = None
        else:
            # converto il testo in una lista di stringhe
            lista = eval(text)
            for stringa in lista:
                if target == self.ARTICOLI_COMPUTO or target == self.ARTICOLI_EPU:
                    articolo = Preventa.convert_string_to_articolo(stringa, self.db)
                elif target == self.ARTICOLI_LISTINO:
                    articolo = Preventa.convert_string_to_listino(stringa, self.db)
                # aggiungo l'articolo decodificato alla lista di articoli del computo
                lista_articoli.append(articolo)
        if target == self.ARTICOLI_COMPUTO:
            self.lista_articoli_computo = lista_articoli
        elif target == self.ARTICOLI_EPU:
            self.lista_articoli_epu = lista_articoli
        elif target == self.ARTICOLI_LISTINO:
            self.lista_articoli_listino = lista_articoli

    def clipboard_callback_txt(self, clipboard, text, data):
        """funzione chiamata quando si 'incolla' del testo dalla clipboard"""
        pass # do nothing for now
        return

################################################################################
########## DRAG'N DROP FUNCTIONS ###############################################
################################################################################

    def treeview_drag_begin (self, widget, drag_context):
        """"""
        # setup a different icon if multiple rows are selected
        action = drag_context.get_actions()
        selection = widget.get_selection()
        stock_id = Gtk.STOCK_DND_MULTIPLE
        hot_x, hot_y = (0,0)
        if selection.count_selected_rows() > 1:
            pass#TODO Gtk.drag_set_icon_stock(drag_context, stock_id, hot_x, hot_y)
            print Gtk.drag_set_icon_stock(drag_context, stock_id, hot_x, hot_y)
        # debug output
        logger.debug(_("Drag source begin: widget: {0}; action:'{1}';".format(widget, action)))
    
    def treeview_drag_data_get(self, widget, drag_context, selectiondata, info, time):
        """"""
        # gets drag targets
        data = None
        target = selectiondata.get_target()
        target_name = target.name()
        source_window = drag_context.get_source_window()
        dest_window =  drag_context.get_dest_window()
        # select the data to store
        if info == self.TIPO_ART_COMPUTO and self.articolo_computo is not None:
            data = self.articolo_computo
        #elif info == self.TIPO_ART_COMPUTO and ... is not None:
        #    data = ...
        elif info == self.TIPO_ART_EPU and self.articolo_epu is not None:
            data = self.articolo_epu
        #elif info == self.TIPO_ART_EPU and self.lista_articoli_epu is not None:
        #    data = self.lista_articoli_epu
        elif info == self.TIPO_ART_LISTINO and self.articolo_listino is not None:
            data = self.articolo_listino
        # set the selectiondata
        if data is not None:
            selectiondata.set(target, info, str(data))
            # debug info for selectiondata
            sel_type = selectiondata.get_data_type().name()
            # if it's an external source copy in the clipboard
            if source_window != dest_window:
                self.clipboard.set_data(target_name, data)
        else: sel_type = selectiondata.get_data_type() 
        # debug selectiondata
        if selectiondata.targets_include_text():
            sel_data = selectiondata.get_text()
        else:
            sel_data = selectiondata.get_data()
        # debug output
        logger.debug(_("Drag source data get: widget: {0}; target:'{1}'; info:{2}; time:{3}; selection_data_type:'{4}'; selection_data:'{5}'".format(widget, target_name, info, time, sel_type, sel_data)))

    def cb_drag_drop (self, widget, drag_context, x, y, time):
        """If it is not in a drop zone, it returns False and no further processing is necessary. 
           Otherwise, the handler returns True"""
        drag_action = drag_context.get_actions()
        list_targets = drag_context.list_targets()
        source_widget = Gtk.drag_get_source_widget(drag_context)
        source_window = drag_context.get_source_window()
        dest_window =  drag_context.get_dest_window()
        # debug output
        logger.debug(_("Drag destination data dropped: widget: {0}; drag_action: {1}; (x,y): ({2},{3}),  time: {4}".format(widget, drag_action, x, y,  time)))
        # debug: list target type to drop
        for target in list_targets:
            logger.debug(_("Drag destination data target: target: {0}; target_type: {1}; target_name: {2}".format(target, type(target), target.name())))
        # error message if dropping in the 'listino' treeview without an article
        if widget == self.treeview_listino and self.target_lst not in list_targets:
            self.message(self, "errore", _("L'elemento selezionato non è una voce di listino"))
            return True
        return False

    def cb_drag_data_received (self, widget, drag_context, x, y, selection, info, time):
        """Esegue il 'drop' sul computo inserendo un nuovo articolo, dopo aver ricevuto il testo da applicativi esterni"""
        # aggiungi informazioni di logger
        data = selection.get_data()
        logger.debug(_("Drag destination data received: widget: {0}; (x,y): ({1},{2}), info: {3}; time: {4}; selection_data: '{5}'".format(widget, x, y, info, time, data)))
        # ricava la lista dei targets
        list_targets = drag_context.list_targets()
        for target in drag_context.list_targets():
            logger.debug(_("Drag destination target: {0}".format(target.name())))
        # se gli articoli sono della stessa treeview chiedi se spostare l'articolo
        # (modificare la sua primary key) o se copiarlo (duplicarlo)
        source_widget = Gtk.drag_get_source_widget(drag_context)
        if source_widget == widget and info == self.TIPO_ART_COMPUTO: 
            if widget.get_path_at_pos(x, y) is not None:
                paths, column, cell_x, cell_y = widget.get_path_at_pos(x, y) 
                model = widget.get_model()
                iter = model.get_iter(paths)
                prymary_key = int(model.get_value(iter, 0))
                if self.message(self, "domanda", _("Spostare l'articolo selezionato nella posizione n.{0}".format(prymary_key))):
                    # effettuare lo spostamento dell'articolo selezionato
                    if self.articolo_computo is not None:
                        self.preventivo.update_articolo_computo (self.prymary_key, 
                                  self.articolo_computo, new_id= prymary_key)
                        # Aggiorna tutta l'interfaccia grafica del tab 'Computo'
                        self.prymary_key = prymary_key
                        self.Aggiorna_finestra_dialogo(self)
                    else:
                        self.message(self, "errore", _("Nessun articolo selezionato"))
                # dopo lo spostamento concludi l'azione
                return True
        # se la sorgente è la treeview lavorazioni e sono art di computo recupera i dati
        elif type(source_widget) == type(self.treeview_lavorazioni) and \
             info == self.TIPO_ART_EPU:
            print self.articolo_epu
            # copy from the clipboard
            data = self.clipboard.get_data(target.name())
            if type(data) is list:
                self.lista_articoli_epu = data
            else: 
                self.articolo_epu = data
        # se la sorgente è la treeview computo e sono art di computo recupera i dati
        elif type(source_widget) == type(self.treeview_computo) and \
             info == self.TIPO_ART_COMPUTO:
            # copy from the clipboard
            data = self.clipboard.get_data(target.name())
            if type(data) is list:
               self.lista_articoli_computo = data
            else:
               self.articolo_computo = data
        # handle other cases
        else: pass
        # se gli articoli sono articoli di epu inserirli preservando le categorie
        if info == self.TIPO_ART_EPU:
            self.incolla_articolo_archivio_principale()
            # azzera gli elementi in memoria per l'EPU
            self.prymary_key = None
            self.articolo_epu = None
            self.lista_articoli_epu = None
        # se gli articoli sono articoli di computo incollali
        elif info == self.TIPO_ART_COMPUTO:
            self.incolla_articolo_computo()
            # azzera gli elementi in memoria per il computo
            self.prymary_key = None
            self.articolo_computo = None
            self.lista_articoli_computo = None
        # se gli articoli sono articoli di listino incollali
        elif info == self.TIPO_ART_LISTINO:
            if self.metel is not None:
                self.articolo_metel = self.metel.rigo_copiato
                self.incolla_articolo_listino(self)
                # azzera gli articoli in memoria
                self.listino_key = None
                self.articolo_listino = None
                self.lista_articoli_listino = None
        elif info == self.TIPO_TEXT_PLAIN:
            text = selection.get_text()
            if self.message(self, "domanda", _("Vuoi inserire un nuovo articolo con descrizione: '%s'" % text)):
                if widget == self.treeview_computo:
                    self.nuovo_articolo_computo(self, data=text)
                elif widget== self.treeview_lavorazioni:
                    self.nuovo_articolo_lista_lavorazioni(self, data=text)
                elif widget== self.treeview_listino:
                    self.nuovo_articolo_listino(self, data=text)
                else:
                    logger.error(_("Nessun widget può ricevere questo testo: {0}".format(text)))
        return True

################################################################################
### APERTURA FILE COMPUTO CON DRAG'N DROP  #####################################
################################################################################
    def window_drag_data_received(self, widget, drag_context, x, y, selection, info, timestamp):
        uri = selection.get_uris()
        i = len(uri)
        # crea la progressbar
        self.progressbar = ProgressBar(self.builder, len(uri), testo = None)
        for uri in uri:
            path = self.get_file_path_from_dnd_dropped_uri(uri)
            if os.path.isfile(path): # is it file?
                # controlla se il computo è già aperto
                if path == self.__filename:
                    self.message(self, "errore", _("Computo già aperto. Impossibile da aprire"))
                    i -= 1
                # apri una finestra con un nuovo preventivo
                else:
                    try:
                        computo_aperto = ComputoWindow(filename=path, 
                              filearchivio=self.__filearchivio, 
                              filelistino=self.__filelistino,
                              preventa_settings=self.__preventa_settings, 
                              computo_settings=self.__computo_settings)
                        computo_aperto.show()
                    except:
                        self.message(self, "errore", _("Il file {0} non è un archivio PreventARES valido".format(path)))
                        i -= 1
                # avanza la progressbar
                self.progressbar.avanza()
        # distruggi la progressbar
        self.progressbar.destroy()
        # aumenta i livelli di Gtk.main()
        while i > 0:
            i -= 1
            Gtk.main()
        return True

    def get_file_path_from_dnd_dropped_uri(self, uri):
        #get the path to file
        path = ""
        if uri.startswith('file:\\\\\\'): # windows
            path = uri[8:] # 8 is len('file:///')
        elif uri.startswith('file://'): # nautilus, rox
            path = uri[7:] # 7 is len('file://')
        elif uri.startswith('file:'): # xffm
            path = uri[5:] # 5 is len('file:')
        path = urllib.url2pathname(path) # escape special chars
        path = path.strip('\r\n\x00') # remove \r\n and NULL
        return path

################################################################################
###  CODICE GENERICO VALIDO PER TUTTI ##########################################
################################################################################

    def __index_combobox (self, widget, combobox_obj, data=None):
        """dizionario in cui i valori della colonna'0' del combobox sono la chiave per accedere all'indice del combobox"""
        dictionary = {-1:-1}
        model = combobox_obj.get_model()
        iter = model.get_iter_first()
        index = 0
        while iter is not None:
            key = int(model.get_value(iter,0))
            dictionary[key] = index
            iter = model.iter_next(iter)
            index += 1
        return dictionary

    def __index_combobox_reverse (self, widget, combobox_obj, data=None):
        """dizionario in cui l'indice del combobox è la chiave per accedere ai valori della colonna'0' del combobox"""
        dictionary = {-1:-1}
        model = combobox_obj.get_model()
        iter = model.get_iter_first()
        index = 0
        while iter is not None:
            key = int(model.get_value(iter,0))
            dictionary[index] = key
            iter = model.iter_next(iter)
            index += 1
        return dictionary

    def __verifica_dato_immesso (self, widget, obj, data=None):
        """Verifica se il dato immesso è valido o avvisa l'utente tramite un messaggio di errore
           questo metodo è valido solamente per oggetti che supportano il metodo .get_text() (es. Gtk.Entry)"""
        text = obj.get_text()
        text = text.replace(',', '.')
        try:
            dato = float(text)
            obj.set_text(text)
        except ValueError:
            if not self.message(self, "errore", "Valore non corretto. Immettere una numero decimale o intero"):
                elenco_caratteri_ok = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '.']
                for carattere in list(text):
                    if carattere not in elenco_caratteri_ok:
                        text = text.replace(carattere, '')
                widget.grab_focus()
                obj.set_text(text)
                try:
                    dato = float(text)
                except ValueError:
                    dato = 0.0
                    obj.set_text('0.0')
        return dato

    def __verifica_testo_immesso (self, text):
        """Verifica se il dato immesso è valido o avvisa l'utente tramite un messaggio di errore
           questo metodo è valido solamente per stringhe di testo"""
        try:
            text = float(eval(text.replace(',', '.')))
        except SyntaxError, NameError:
            if not self.message(self, "errore", "Valore non corretto. Immettere una numero decimale o intero"):
                elenco_caratteri_ok = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '.']
                for carattere in list(text):
                    if carattere not in elenco_caratteri_ok:
                        text = text.replace(carattere, '')
                try:
                    text = float(text)
                except ValueError:
                    text = 0.0
        return text

    def __verifica_intero_immesso (self, text):
        """Verifica se il dato immesso è un numero intero valido o avvisa l'utente 
           tramite un messaggio di errore. metodo è valido solo per stringhe"""
        try:
            text = int(text)
        except ValueError:
            if not self.message(self, "errore", "Valore non corretto. Immettere una numero intero positivo"):
                return None
        # se l'intero è negativo lo rende positivo
        return abs(text)

    def __cancella_articoli_memoria(self):
        """Funzione per cancellare tutti gli articoli o le liste di articoli in memoria"""
        self.prymary_key = 0 #valore univoco di accesso alle righe del database computo
        self.articolo_computo = None
        self.listino_key = 0 #valore univoco di accesso alle righe del database listino
        self.articolo_listino = None
        self.articolo_epu = None #articolo di EPU in memoria
        self.lista_articoli_computo = None
        self.lista_articoli_epu = None
        self.lista_articoli_listino = None
################################################################################
######### AGGIORNAMENTO INTERFACCIA GRAFICA ####################################
################################################################################

    def Seleziona_articolo_corrente(self, widget, data=None):
        """posiziona il selettore sul rigo corrente"""
        if self.prymary_key >=0:
            selection = self.treeview_computo.get_selection()
            model = self.treeview_computo.get_model()
            iter = model.get_iter_first()
            if iter is None:
               return None
            value = int(model.get_value(iter, 0))
            while value != self.prymary_key:
                iter = model.iter_next(iter)
                if iter is None:
                    iter = model.get_iter_first()
                    break
                value = int(model.get_value(iter, 0))
            # seleziona il rigo corrispondente
            selection.select_iter(iter)
            path = model.get_path(iter)
            # effettua lo scrolling della treeview fino all'articolo corrente
            self.treeview_computo.scroll_to_cell(path, column=None, 
                use_align=False, row_align=0.0, col_align=0.0)
            return path
        else:
            return None

    def Aggiorna_maschera_articolo(self, widget, data=None):
        """Aggiorna maschera con i dati dell'articolo di computo/epu"""
        # verifica in quale pagina si trova il notebook (0= dati generali, 1= computo, 2= epu)
        notebook_page = self.notebook.get_current_page()
        if self.articolo_computo is None and notebook_page == 1:
            return False
        elif self.articolo_epu is None and notebook_page == 2:
            return False
        elif notebook_page == 1:
            articolo = self.articolo_computo
        elif notebook_page == 2:
            articolo = self.articolo_epu
        # imposta i decimali dei campi numerici
        dec = 2
        # Aggiornamento dei campi equivalenti per 'Computo' ed 'Epu'
        self.tariffa.set_text(articolo.tariffa)
        self.descrizione_codice.set_text(articolo.descrizione_codice)
        self.descrizione_voce.set_text(articolo.descrizione_voce)
        self.descrizione_estesa.set_text(articolo.descrizione_estesa)
        self.ricarico.set_value(articolo.ricarico)
        self.tempo_inst.set_value(articolo.tempo_inst)
        self.sicurezza.set_value(articolo.sicurezza)
        self.costo_materiali.set_text("{0:.{1}f}".format(articolo.costo_materiali, dec))
        self.prezzo_unitario.set_text("{0:.{1}f}".format(articolo.prezzo_unitario, dec))
        # quantità
        self.quantita.set_text(str(articolo.quantita))
        # per unita_misura 
        self.unita_misura.get_child().set_text(articolo.unita_misura)
        if articolo.unita_misura not in self.liststore_um.elenco_um:
            self.liststore_um.elenco_um.append(articolo.unita_misura)
            self.liststore_um.append((articolo.unita_misura,))
        # capitoli
        self.Aggiorna_liststore_capitoli(self)
        self.box_supercapitolo.set_active(self.__index_combobox(self, self.box_supercapitolo)[articolo.supercapitolo])
        self.box_capitolo.set_active(self.__index_combobox(self, self.box_capitolo)[articolo.capitolo])
        self.box_subcapitolo.set_active(self.__index_combobox(self, self.box_subcapitolo)[articolo.subcapitolo])
        # fasce di costo aggiuntive (1-4)
        self.costo_mat_1.set_text("{0:.{1}f}".format(articolo.costo_mat1, dec))
        self.costo_mat_2.set_text("{0:.{1}f}".format(articolo.costo_mat2, dec))
        self.costo_mat_3.set_text("{0:.{1}f}".format(articolo.costo_mat3, dec))
        self.costo_mat_4.set_text(str(articolo.costo_mat4))
        # categorie
        self.Aggiorna_liststore_categorie(self)
        self.box_supercategoria.set_active(self.__index_combobox(self, self.box_supercategoria)[articolo.supercategoria])
        self.box_categoria.set_active(self.__index_combobox(self, self.box_categoria)[articolo.categoria])
        self.box_subcategoria.set_active(self.__index_combobox(self, self.box_subcategoria)[articolo.subcategoria])
        # attiva e disattiva gli elementi non editabili nella Tab 'Epu'
        if notebook_page == 1:
            self.quantita.set_sensitive(True)
            self.box_supercategoria.set_sensitive(True)
            self.box_categoria.set_sensitive(True)
            self.box_subcategoria.set_sensitive(True)
            self.box_supercapitolo.set_sensitive(False)
            self.box_capitolo.set_sensitive(False)
            self.box_subcapitolo.set_sensitive(False)
            self.frame_capitoli.hide() #capitoli epu
            self.frame_categorie.show()
        elif notebook_page == 2:
            self.quantita.set_sensitive(False)
            self.box_supercategoria.set_sensitive(False)
            self.box_categoria.set_sensitive(False)
            self.box_subcategoria.set_sensitive(False)
            self.box_supercapitolo.set_sensitive(True)
            self.box_capitolo.set_sensitive(True)
            self.box_subcapitolo.set_sensitive(True)
            self.frame_categorie.hide()
            self.frame_capitoli.show() #capitoli epu
        # Disattiva il pulsante di aggiornamento maschera articolo 
        # (verrà riattivato da una chiamata degli elementi della maschera)
        self.Disattiva_pulsante_aggiornamento(self)
        return True

    def Aggiorna_liststore_capitoli(self, widget, data=None):
        """Aggiorna il liststore dei capitoli"""
        self.lista_cap = self.preventivo.capitoli_rows_list()
        self.liststore_supercapitolo.clear()
        self.liststore_capitolo.clear()
        self.liststore_subcapitolo.clear()
        for cap in self.lista_cap[0]:
            elem = [str(cap[0]), str(cap[1]), str(cap[2]), str(cap[3])]
            self.liststore_supercapitolo.append(elem)
        for cap in self.lista_cap[1]:
            elem = [str(cap[0]), str(cap[1]), str(cap[2]), str(cap[3])]
            self.liststore_capitolo.append(elem)
        for cap in self.lista_cap[2]:
            elem = [str(cap[0]), str(cap[1]), str(cap[2]), str(cap[3])]
            self.liststore_subcapitolo.append(elem)
        return self.lista_cap

    def Aggiorna_liststore_categorie(self, widget, data=None):
        """Aggiorna il liststore delle categorie"""
        self.lista_cat = self.preventivo.categorie_rows_list()
        self.liststore_supercategoria.clear()
        self.liststore_categoria.clear()
        self.liststore_subcategoria.clear()
        for cat in self.lista_cat[0]:
            elem = [str(cat[0]), str(cat[1]), str(cat[2]), str(cat[3])]
            self.liststore_supercategoria.append(elem)
        for cat in self.lista_cat[1]:
            elem = [str(cat[0]), str(cat[1]), str(cat[2]), str(cat[3])]
            self.liststore_categoria.append(elem)
        for cat in self.lista_cat[2]:
            elem = [str(cat[0]), str(cat[1]), str(cat[2]), str(cat[3])]
            self.liststore_subcategoria.append(elem)
        return self.lista_cat

    def Aggiorna_finestra_dialogo(self, widget, aggiorna_epu=False, 
                                  aggiorna_tutto=False, aggiorna_cap=True):
        """Aggiorna tutte le treeview i liststore e le maschere di dialogo"""
        # verifica in quale pagina si trova il notebook (0= dati generali, 1= computo, 2= epu)
        notebook_page = self.notebook.get_current_page()
        # crea la progressbar
        if aggiorna_tutto: n_agg = 8
        else: n_agg = 5
        self.progressbar = ProgressBar(self.builder, n_agg, testo = None)
        # Inizia l'aggiornamento
        self.Aggiorna_liststore_capitoli(self)
        self.progressbar.avanza()
        self.Aggiorna_liststore_categorie(self)
        self.progressbar.avanza()
        # aggiornamento è escluso per certi tab (x velocizzare l'aggiornamento)
        if notebook_page == 1 or aggiorna_tutto:
            self.Aggiorna_treeview_computo(self)
            self.progressbar.avanza()
            self.Aggiorna_treeview_listino(self)
            self.progressbar.avanza()
            if aggiorna_cap or aggiorna_tutto:
                self.Aggiorna_treeview_categorie(self)
            self.progressbar.avanza()
            # seleziona l'articolo corrente
            self.Seleziona_articolo_corrente(self)
        if notebook_page == 2 or aggiorna_tutto:
            if aggiorna_cap or aggiorna_tutto:
                self.Aggiorna_treeview_capitoli(self)
            self.progressbar.avanza()
            self.Aggiorna_lista_lavorazioni(self)
            self.progressbar.avanza()
            self.Aggiorna_treeview_listino(self)
            self.progressbar.avanza()
            # seleziona l'articolo corrente
            self.Seleziona_articolo_lista_lavorazioni(self)
        # distruggi la progressbar
        self.progressbar.destroy()
        # Aggiorna la Barra di Stato con il totale del computo
        self.Aggiorna_BarraStato_default()
        # Disattiva il pulsante di aggiornamento maschera articolo (è riattivato da una chiamata degli elementi della maschera)
        self.Disattiva_pulsante_aggiornamento(self)
        # Verifica se ricaricare l'epu o se non farlo
        if aggiorna_epu or aggiorna_tutto:
            self.Aggiorna_treeview_epu(self)
            return True
        else:
            return False

    def Aggiorna_treeview_computo(self, widget, data=None):
        """Aggiorna la treeview del computo"""
        self.liststore_computo.clear()
        for articolo in self.preventivo.computo_rows_list(
                                        supercategoria= self.categorie[0], 
                                        categoria=self.categorie[1], 
                                        subcategoria=self.categorie[2],
                                        convert_to_string=True, 
                                        contract_type=self.contract_type[0]):
            # aggiuge al liststore l'articolo
            self.liststore_computo.append(articolo)
        self.treeview_computo.columns_autosize()
######### AGGIORNAMENTO DELLA TREEVIEW CATEGORIE ###############################
    def Aggiorna_treeview_categorie(self, widget, visualizza_tutti=None, data=None):
        """Aggiorna la treeview delle categorie"""
        if visualizza_tutti is None:
            visualizza_tutti = settings["show_empty_category"]
        self.treestore_categorie.clear()
        self.lista_cat = self.preventivo.categorie_rows_list()
        c_t = self.contract_type[0] # tipologia di contratto (es. Lavori a corpo)
        # popola la treeview
        for sup in self.lista_cat[0]:
            if self.preventivo.computo_count_rows(supercategoria= sup[0], contract_type=c_t) > 0 or visualizza_tutti:
                item = (str(sup[0]), sup[1], self.preventivo.calcolo_prezzi(supercategoria=sup[0], contract_type=c_t))
                iter = self.treestore_categorie.append(None, item)
                for cat in self.lista_cat[1]:
                    if self.preventivo.computo_count_rows(supercategoria= sup[0], categoria=cat[0], contract_type=c_t) > 0 or visualizza_tutti:
                        item = (str(cat[0]), cat[1], self.preventivo.calcolo_prezzi(supercategoria=sup[0], categoria=cat[0], contract_type=c_t))
                        iter1 = self.treestore_categorie.append(iter, item)
                        for sub in self.lista_cat[2]:
                            if self.preventivo.computo_count_rows(supercategoria=sup[0], categoria=cat[0], subcategoria=sub[0], contract_type=c_t) > 0 or visualizza_tutti:
                                item = (str(sub[0]), sub[1], self.preventivo.calcolo_prezzi(supercategoria=sup[0], categoria=cat[0], subcategoria=sub[0], contract_type=c_t))
                                iter2 = self.treestore_categorie.append(iter1, item)
        self.treeview_categorie.columns_autosize()
        self.Seleziona_categoria_corrente(self)

######### AGGIORNAMENTO DELLA TREEVIEW CAPITOLI ################################
    def Aggiorna_treeview_capitoli(self, widget, visualizza_tutti=None, data=None):
        """Aggiorna la treeview dei capitoli"""
        if visualizza_tutti is None:
            visualizza_tutti = settings["show_empty_chapter"]
        self.treestore_capitoli.clear()
        self.lista_cap = self.preventivo.capitoli_rows_list()
        # popola la treeview
        for sup in self.lista_cap[0]:
            if len(self.preventivo.epu_rows_list(supercapitolo= sup[0])) > 0 or visualizza_tutti:
                item = (str(sup[0]), sup[1], self.preventivo.calcolo_prezzi_lavorazioni(supercapitolo=sup[0], capitolo=None, subcapitolo=None))
                iter = self.treestore_capitoli.append(None, item)
                for cap in self.lista_cap[1]:
                    if len(self.preventivo.epu_rows_list(supercapitolo= sup[0], capitolo=cap[0])) > 0 or visualizza_tutti:
                        item = (str(cap[0]), cap[1], self.preventivo.calcolo_prezzi_lavorazioni(supercapitolo=sup[0], capitolo=cap[0], subcapitolo=None))
                        iter1 = self.treestore_capitoli.append(iter, item)
                        for sub in self.lista_cap[2]:
                            if len(self.preventivo.epu_rows_list(supercapitolo=sup[0], capitolo=cap[0], subcapitolo=sub[0])) > 0 or visualizza_tutti:
                                item = (str(sub[0]), sub[1], self.preventivo.calcolo_prezzi_lavorazioni(supercapitolo=sup[0], capitolo=cap[0], subcapitolo=sub[0]))
                                iter2 = self.treestore_capitoli.append(iter1, item)
        self.treeview_capitoli.columns_autosize()
        self.Seleziona_capitolo_corrente(self)

######### AGGIORNAMENTO DELLA TREEVIEW DELL'EPU ################################
    def Aggiorna_treeview_epu(self, widget, archivio_esterno=None):
        """Aggiorna la treeview dell'elenco prezzi"""
        if settings["load_main_archive"]:
            archivio = self.archivio
        elif archivio_esterno is not None:
            archivio = archivio_esterno
        else:
            archivio = self.preventivo
        # se non esiste alcun archivio disponibile esci dalla routine
        if archivio is None:
            logger.info(_("Nessun archivio articoli disponibile"))
            return
        # 'scollega' la treeview dal suo modello x velocizzare le operazioni
        self.treeview_epu.set_model(None)
        # Set the TreeView "fixed-height-mode" property to TRUE making all rows have the same height and avoiding the individual calculation of the height of each row
        self.treeview_epu.set_property('fixed-height-mode', True)
        # elimina tutte le righe dal treestore
        self.treestore_epu.clear()
        # a seconda delle impostazioni carica la vista con capitoli e subcapitoli o la semplice lista degli articoli
        if settings["load_epu_chapter"]:
            self.__create_treeview_epu_chapter_(archivio)
        else: 
            self.__create_treeview_epu_short_(archivio)
        # 'ricollega' la treeview al suo modello
        self.treeview_epu.set_model(self.treestore_epu)
        # ridimensiona le colonne della treeview
        self.treeview_epu.columns_autosize()
        # posiziona la treeview sull'ultimo articolo di epu selezionato
        self.Seleziona_articolo_treeview_epu(self)

    def __create_treeview_epu_chapter_(self, archivio):
        """"""
        # messaggio di default per la progressbar
        msg = _("Caricamento articoli archivio {0:.0%}")
        # ricava il numero di articoli da inserire
        self.lista_cap = archivio.capitoli_rows_list()
        lista_articoli = archivio.epu_rows_list(supercapitolo= None, capitolo= None, subcapitolo=self.lista_cap[2][0])
        n_articoli = len(lista_articoli)
        # crea la progressbar
        self.progressbar = ProgressBar(self.builder, n_articoli//100)
        n = 0
        # popola la treeview
        for sup in self.lista_cap[0]: 
            if len(archivio.epu_rows_list(supercapitolo= sup[0])) > 0:
                lista = [str(sup[0]), str(sup[1]), str(sup[2]), str(sup[2])]
                iter = self.treestore_epu.append(None, lista)
                n+=1
                for cap in self.lista_cap[1]:
                    if len(archivio.epu_rows_list(supercapitolo= sup[0], capitolo= cap[0])) > 0:
                        lista = [str(cap[0]), str(cap[1]), str(cap[2]), str(cap[2])]
                        iter1 = self.treestore_epu.append(iter, lista)
                        n+=1
                        for sub in self.lista_cap[2]:
                            lista_epu = archivio.epu_rows_list(supercapitolo= sup[0], capitolo= cap[0], subcapitolo=sub[0])
                            if len (lista_epu) > 0:
                               lista = [str(sub[0]), str(sub[1]), str(sub[2]), str(sub[2])]
                               iter2 = self.treestore_epu.append(iter1, lista)
                               n+=1
                               for art in lista_epu:
                                   lista = [art[0], art[4], art[5], '']
                                   iter3 = self.treestore_epu.append(iter2, lista)
                                   n+=1
                                   # aggiorna il contatore righe mostrate nella maschera
                                   nr,rest = divmod(n, 100)
                                   if rest == 0:
                                       self.progressbar.avanza(new_text = msg.format(float(nr*100)/n_articoli))
        # distruggi o nascondi la progressbar
        self.progressbar.destroy()

    def __create_treeview_epu_short_(self, archivio):
        """"""
        # ricava il numero di articoli da inserire
        self.lista_cap = archivio.capitoli_rows_list()
        lista_articoli = archivio.epu_rows_list(supercapitolo= None, capitolo= None, subcapitolo=self.lista_cap[2][0])
        n_articoli = len(lista_articoli)
        # crea la progressbar
        self.progressbar = ProgressBar(self.builder, n_articoli//100)
        n = 1
        # popola la treeview con gli articoli
        lista_epu = archivio.epu_rows_list()
        for art in lista_epu:
            lista = [art[0], art[4], art[5], '']
            iter = self.treestore_epu.append(None, lista)
            n+=1
            # aggiorna il contatore righe mostrate nella maschera
            nr,rest = divmod(n, 100)
            if rest == 0:
                # aggiorna la barra d'avanzamento
                self.progressbar.avanza(new_text = _("Caricamento articoli archivio {0:.0%}").format(float(nr*100)/n_articoli))
        # distruggi o nascondi la progressbar
        self.progressbar.destroy()
######### FINE AGGIORNAMENTO DELLA TREEVIEW DELL'EPU ###########################

    def Aggiorna_lista_lavorazioni(self, widget, data=None):
        """Aggiorna la treeview dell'elenco prezzi nella 'linguetta' Elenco Prezzi"""
        self.liststore_lavorazioni.clear()
        lista_lavorazioni = self.preventivo.lavorazioni_rows_list(
                                            supercapitolo= self.capitoli[0],
                                            capitolo= self.capitoli[1], 
                                            subcapitolo=self.capitoli[2],
                                            convert_to_string=True)
        # popola la treeview
        for art in lista_lavorazioni:
            self.liststore_lavorazioni.append(art)

    def Aggiorna_maschera_dati_generali(self, widget, data=None):
        """Aggiorna i dati generali nella 'linguetta' Dati Generali"""
        dati = self.preventivo.dati_generali_list()
        if len(dati) == 0:
            dati = [0, 0, 0, '', '', '', '', '', '']
        # imposta treeview per capitoli e categorie dei dati generali
        self.dg_ricarico.set_value(float(dati[0]))
        self.dg_manodopera.set_text(str(dati[1]))
        self.dg_sicurezza.set_value(float(dati[2]))
        self.dg_valuta.set_text(dati[3])
        self.dg_nome_lavoro.set_text(dati[4])
        self.dg_indirizzo.set_text(dati[5])
        self.dg_comune.set_text(dati[6])
        self.dg_provincia.set_text(dati[7])
        self.dg_cliente.set_text(dati[8])
        self.dg_redattore.set_text(dati[9])
        pulsante_annulla = self.builder.get_object("button18")
        pulsante_applica = self.builder.get_object("button19")
        pulsante_annulla.set_sensitive(False)
        pulsante_applica.set_sensitive(False)

################################################################################
###### NOTEBOOK (per Dati Generali - Computo - Epu) ############################
################################################################################
    def notebook_page_changed(self, widget, page, page_num, data=None):
        """operazioni da effettuare quando viene cambiata la pagina del notebook (0= dati generali, 1= computo, 2= epu)"""
        if page_num == 0:
            self.Aggiorna_maschera_dati_generali(self)
        elif page_num == 1:
            self.progressbar = ProgressBar(self.builder, 5, testo = None)
            self.Aggiorna_liststore_capitoli(self)
            self.progressbar.avanza()
            self.Aggiorna_liststore_categorie(self)
            self.progressbar.avanza()
            self.Aggiorna_treeview_computo(self)
            self.progressbar.avanza()
            self.Aggiorna_treeview_listino(self)
            self.progressbar.avanza()
            self.Aggiorna_treeview_categorie(self)
            self.progressbar.avanza()
            self.progressbar.destroy()
            # Sposta la visualizzazione del listino nel tab 'Computo'
            self.treeview_listino.reparent(self.computo_scrolledwindow_listino)
            # Minimizza l'espansore della maschera articoli
            self.computo_expander.set_expanded(False)
            # Sposta la visualizzazione della maschera di modifica articono nel tab 'Computo'
            self.mask_art.hide()
            self.mask_art.reparent(self.computo_expander)
            # Sposta la visualizzazione della maschera Epu nel tab 'Computo'
            self.box_treeview_epu.reparent(self.vbox_laterale_sx_tab_computo)
            self.box_treeview_epu.resize_children()
        elif page_num == 2:
            self.progressbar = ProgressBar(self.builder, 5, testo = None)
            self.progressbar.avanza()
            self.Aggiorna_liststore_capitoli(self)
            self.progressbar.avanza()
            self.Aggiorna_treeview_capitoli(self)
            self.progressbar.avanza()
            self.Aggiorna_lista_lavorazioni(self)
            self.progressbar.avanza()
            self.Aggiorna_treeview_listino(self)
            self.progressbar.avanza()
            self.progressbar.destroy()
            # Sposta la visualizzazione del listino nel tab 'Epu'
            self.treeview_listino.reparent(self.lavorazioni_scrolledwindow_listino)
            # Minimizza l'espansore della maschera articoli
            self.lav_expander.set_expanded(False)
            # Sposta la visualizzazione della maschera di modifica articolo nel tab 'Epu'
            self.mask_art.hide()
            self.mask_art.reparent(self.lav_expander)
            # Sposta la visualizzazione della maschera Epu nel tab 'Epu'
            self.box_treeview_epu.reparent(self.vbox_laterale_sx_tab_epu)
            self.box_treeview_epu.resize_children()
        return

    def on_expander_activated(self, widget, box=None):
        """quando gli espansori sono attivati ridimensiona i widget che contengono"""
        #widget.hide()
        widget.resize_children()
        #widget.show_all()
        if box is not None and not widget.get_expanded():
            box.show_all()
        elif box is not None and widget.get_expanded():
            box.hide()
        
################################################################################
###### PULSANTI e MENU PRINCIPALE ##############################################
################################################################################
    def apri_computo(self, widget, data=None):
        """apre una nuova finesta con un computo esistente"""
        dialog = FileChooserDialog(testo= _("Apri un computo esistente"), mode="open", 
                                   filetype="db", default_path=settings["default_working_directory"])
        fileComputo = dialog.run()
        if fileComputo == self.__filename:
            self.message(self, "errore", _("Computo già aperto. Impossibile da aprire"))
            return None
        elif fileComputo is not None:
            # crea la progressbar
            self.progressbar = ProgressBar(self.builder, None, testo = None)
            # apri una finestra con un nuovo preventivo
            computo_aperto = ComputoWindow(filename=fileComputo, filearchivio=self.__filearchivio, 
                              filelistino=self.__filelistino, preventa_settings=self.__preventa_settings, 
                              computo_settings=self.__computo_settings)
            computo_aperto.show()
            # distruggi la progressbar
            self.progressbar.destroy()
            Gtk.main()
            return computo_aperto
        elif fileComputo is None:
            self.message(self, "errore", _("Impossibile aprire un nuovo computo, file inesistente"))
            return None
        else:
            return None

    def nuovo_computo(self, widget, data=None):
        """apre una nuova finesta con un nuovo computo"""
        dialog = FileChooserDialog(testo= _("Salva un nuovo preventivo"), mode="new", 
                                   filetype="db", default_path=settings["default_working_directory"])
        fileComputo = dialog.run()
        if fileComputo == self.__filename:
            self.message(self, "errore", _("Computo già aperto. Impossibile sovrascrivere"))
            return None
        elif fileComputo is not None:
            root, ext = os.path.splitext(fileComputo)
            if ext != ".db": fileComputo =  root + ".db"
            # crea la progressbar
            self.progressbar = ProgressBar(self.builder, None, testo = None)
            # apri una finestra con un nuovo preventivo
            computo_nuovo = ComputoWindow(filename=fileComputo, 
                                          filearchivio=self.__filearchivio, 
                                          filelistino=self.__filelistino, 
                                          preventa_settings=self.__preventa_settings, 
                                          computo_settings=self.__computo_settings)
            computo_nuovo.show()
            # distruggi la progressbar
            self.progressbar.destroy()
            Gtk.main()
            return computo_nuovo
        elif fileComputo is None:
            self.message(self, "errore", _("Impossibile aprire un nuovo computo, file inesistente"))
            return None
        else:
            return None

    def about(self, widget, data=None):
        """Display the about box for preventares."""
        about = AboutPreventaresDialog.AboutPreventaresDialog()
        response = about.run()
        about.destroy()

    def printing(self, widget, data=None):
        """Display the printing window for preventares."""
        # ottieni l'elenco dei capitoli e delle categorie da passare al dialog
        cat_list = self.preventivo.categorie_rows_list()
        cap_list = self.preventivo.capitoli_rows_list()
        # imposta il file per il salvataggio delle impostazioni
        settings_file_name = get_print_settings_file() 
        # carica le impostazioni di stampa da file (se salvate)
        try:
            pr_set = Gtk.print_settings_new_from_file(settings_file_name)
        except:
            pr_set = None
        # apri l'interfaccia delle opzioni di stampa
        print_opt = PrintoptionDialog.PrintoptionDialog(print_settings=pr_set,
                                                        list_capitoli=cap_list, 
                                                        list_categorie=cat_list)
        response = print_opt.run()
        if response == Gtk.ResponseType.OK:
            # Recupera le impostazioni di stampa da usare x la procedura di stampa
            print_settings = print_opt.print_settings
            tipo_stampa =  print_settings.get("print_type")
            # recupera le categorie e i capitoli da stampare ('None' stampa tutto)
            cat = print_opt.cat_list
            cap = print_opt.cap_list
            # salva le impostazioni di stampa (salva le impostazioni scelte dall'utente)
            print_settings.to_file(settings_file_name)
            # nasconde la finestra delle opzioni di stampa
            print_opt.hide()
            # Richiama il modulo per la stampa e stampa il lavoro
            printing = preventa_print.PrintPreventa(self.preventivo, tipo_stampa, 
                  print_settings=print_settings, categorie=cat, capitoli=cap)
            # Recupera le impostazioni di stampa da usare x il salvataggio
            print_settings_last = printing.print_settings
            # salva le impostazioni di stampa finali
            print_settings_last.to_file(settings_file_name)
        print_opt.destroy()
            
    def preferences(self, widget, data=None):
        """Display the preferences window for preventares."""
        set_= self.__computo_settings
        dialog = OptionChooserDialog.Dialog("Impostazioni del Database", 
                                     self.__computo_settings, 
                                     op_keys_names=SETTINGS_TRANSLATION)
        global settings
        settings = dialog.run()
        self.setup_settings()

    def quit(self, widget, data=None):
        """Signal handler for closing the ComputoWindow."""
        if self.message(self, "domanda", _("Vuoi salvare le modifiche prima di uscire?")):
            self.preventivo.save_database()
        else: 
            self.preventivo.rollback_database()
        self.destroy()

    def on_destroy(self, widget, data=None):
        """Called when the ComputoWindow is closed."""
        # Clean up code for saving application state should be added here.
        self.preventivo.connection_shutdown()
        ###################################
        # clean up code
        ###################################
        if self.__archivio_principale is not None:
            try:
                self.__archivio_principale.connection_shutdown()
            except:
                pass #non dare errore se l'archivio non si chiude correttamente
        del self.__archivio_principale
        del self.preventivo
        # variabili da finestra listini METEL
        #del self.metel
        ###################################
        Gtk.main_quit()

    def Update (self, widget, data=None):
        """pulsante per aggiore tutta l'interfaccia grafica"""
        self.Aggiorna_finestra_dialogo(self, aggiorna_epu=True, 
                                       aggiorna_tutto=True, 
                                       aggiorna_cap=True)

    def Fullscreen(self, widget, data=None):
        """pulsante per lavorare a schermo intero"""
        self.fullscreen()

    def Unfullscreen(self, widget, data=None):
        """pulsante togliere lo schermo intero"""
        self.unfullscreen()

    def View_report(self, widget, data=None):
        """apre una finestra di dialogo per visualizzare i report manodopara e incidenza"""
        window = self.builder.get_object("computo_window")
        report = ReportWindow("Report Incidenze", self.preventivo, 
                              parent_window = window)

    def export_to_dumpfile(self, widget, data=None):
        """Called when the ComputoWindow is closed."""
        # Attiva una finestra per la scelta del file
        dialog = FileChooserDialog(testo=_("File di testo da esportare"), mode="save", 
                                   filetype="txt", default_path=settings["default_working_directory"])
        fileDump = dialog.run()
        if fileDump is not None:
            # crea la progressbar
            self.progressbar = ProgressBar(self.builder, None, testo = None)
            self.preventivo.export_to_dumpfile(fileDump)
            # distruggi la progressbar
            self.progressbar.destroy()
            self.message(self, "info", _("File %s esportato.") % fileDump)
            return True
        else:
            self.message(self, "avvisio", _("File non selezionato. Impossibile esportare il database."))
            return False

    def export_to_csv_file(self, widget, data=None):
        """Esporta il file in formato csv"""
        # ottieni l'elenco dei capitoli e delle categorie da passare al dialog
        cat_list = self.preventivo.categorie_rows_list()
        cap_list = self.preventivo.capitoli_rows_list()
        # ottieni le impostazioni di stampa
        settings_file_name = get_print_settings_file() 
        # carica le impostazioni di stampa da file (se salvate)
        try:
            pr_set = Gtk.print_settings_new_from_file(settings_file_name)
        except:
            pr_set = None
        # apri l'interfaccia delle opzioni di stampa
        print_opt = PrintoptionDialog.PrintoptionDialog(print_settings=pr_set, 
                                                        list_capitoli=cap_list, 
                                                        list_categorie=cat_list)
        # semplifica l'interfaccia 'spegnendo' le impostazioni che non servono
        print_opt.modify_ui()
        response = print_opt.run()
        if response == Gtk.ResponseType.OK:
            # Recupera le impostazioni di stampa da usare x la procedura di stampa
            csv_settings = print_opt.print_settings
            tipo_stampa =  csv_settings.get("print_type")
            # recupera le categorie e i capitoli da stampare ('None' stampa tutto)
            cat = print_opt.cat_list
            cap = print_opt.cap_list
            # nasconde la finestra delle opzioni di stampa
            print_opt.hide()
            # Attiva una finestra per la scelta del file
            dialog = FileChooserDialog(testo=_("File CSV da esportare"), 
                                       mode="save", filetype="csv", 
                                       default_path=settings["default_working_directory"])
            FileName = dialog.run()
            if FileName is not None:
                root, ext = os.path.splitext(FileName)
                if ext != ".csv":
                    FileName =  root + ".csv"
                # crea la progressbar
                self.progressbar = ProgressBar(self.builder, None, testo = None)
                # Richiama il modulo per la stampa e stampa il lavoro
                export_csv = preventa_print.CsvPreventa(self.preventivo, FileName,
                             tipo_stampa, csv_settings, categorie=cat, capitoli=cap)
                # distruggi la progressbar
                self.progressbar.destroy()
                self.message(self, "info", _("File %s esportato in formato CSV.") % FileName)
                return True
            else:
                self.message(self, "errore", _("File non selezionato. Impossibile esportare il computo in CSV."))
                return False
        else:
            pass
        print_opt.destroy()
        return True

    def export_to_pwe_file(self, widget, data=None):
        """Esporta il file in formato PWE per Primus"""
        # creo una lista di nomi delle opzioni x facilitare l'internazionalizzazione
        opt_name = (_("Esporta Elementi di Listino"),)
        # creo il dizionario delle opzioni
        options = {opt_name[0]: False}
        # apro la finestra di dialogo delle opzioni
        dialog = OptionChooserDialog.Dialog(_("Opzioni esportazione formato PWE"), options, buttons="apply")
        options = dialog.run()
        # Attiva una finestra per la scelta del file
        dialog = FileChooserDialog(testo=_("File PWE da esportare"), mode="save", 
                                   filetype="pwe", default_path=settings["default_working_directory"])
        FileName = dialog.run()
        if FileName is not None:
            root, ext = os.path.splitext(FileName)
            if ext != ".pwe":
                FileName =  root + ".pwe"
            # crea la progressbar
            self.progressbar = ProgressBar(self.builder, None, testo = None)
            self.preventivo.export_to_primus_pwe (FileName, esporta_elementi_listino=options[opt_name[0]])
            # distruggi la progressbar
            self.progressbar.destroy()
            self.message(self, "info", _("File %s esportato in formato PWE.") % FileName)
            return True
        else:
            self.message(self, "errore", _("File non selezionato. Impossibile esportare il computo in formato PWE."))
            return False

    def import_from_file_dump(self, widget, data=None):
        """Called when the ComputoWindow is closed."""
        # Attiva una finestra per la scelta del file
        dialog = FileChooserDialog(testo=_("File DUMP da importare"), mode="open", 
                                   filetype="txt", default_path=settings["default_working_directory"])
        fileDump = dialog.run()
        if fileDump is not None:
            # crea la progressbar
            self.progressbar = ProgressBar(self.builder, None, testo = None)
            self.preventivo.import_from_dumpfile(fileDump)
            # distruggi la progressbar
            self.progressbar.destroy()
            self.message(self, "info", _("File database importato correttamente."))
            return True
        else:
            self.message(self, "errore", _("File non esistente. Impossibile eseguire l'importazione."))
            return False

    def import_from_pwe(self, widget, data=None):
        """Import Primus PWE file."""
        # Attiva una finestra per la scelta del file
        dialog = FileChooserDialog(testo=_("File PWE da importare"), 
                                   mode="open", filetype="pwe", 
                                   default_path=settings["default_working_directory"])
        filePWE = dialog.run()
        if filePWE is not None:
            # crea la progressbar
            progressbar = ProgressBar(self.builder, None, testo=_("Importazione PWE. Attendere."), 
                          window_mode = True, modal=True)
            GObject.timeout_add(10, progress_timeout, progressbar.progressbar)
            while Gtk.events_pending():
                Gtk.main_iteration()
            #GObject.idle_add(self.preventivo.import_from_primus_pwe, filePWE)
            self.preventivo.import_from_primus_pwe(filePWE)
            # distruggi la progressbar
            progressbar.destroy()
            # aggiorna l'interfaccia grafica
            self.Aggiorna_finestra_dialogo(self)
            self.Aggiorna_maschera_dati_generali(self)
            return True
        else:
            self.message(self, "errore", _("File non esistente. Impossibile eseguire l'importazione."))
            return False

    def import_from_six(self, widget, data=None):
        """Import Primus PWE file."""
        # Attiva una finestra per la scelta del file
        dialog = FileChooserDialog(testo=_("File XML da importare"), 
                                   mode="open", filetype="xml", 
                                   default_path=settings["default_working_directory"])
        fileSIX = dialog.run()
        if fileSIX is not None:
            # crea la progressbar
            self.progressbar = ProgressBar(self.builder, None, testo = None)
            self.preventivo.import_XML(fileSIX)
            # distruggi la progressbar
            self.progressbar.destroy()
            # aggiorna l'interfaccia grafica
            self.Aggiorna_finestra_dialogo(self)
            self.Aggiorna_maschera_dati_generali(self)
            return True
        else:
            self.message(self, "errore", _("File non esistente. Impossibile eseguire l'importazione."))
            return False


    def salva_modifiche_computo(self, widget, data=None):
        """Esegue il commit sul database."""
        self.preventivo.save_database()

    def save_as(self, widget, data=None):
        """Permette di salvare il computo corrente in un nuovo file."""
        # Attiva una finestra per la scelta del nuovo file
        dialog = FileChooserDialog(testo= _("Salva un nuovo preventivo"), 
                                   mode="new", filetype="db", 
                                   default_path=settings["default_working_directory"])
        fileComputo = dialog.run()
        if fileComputo == self.__filename:
            self.message(self, "errore", _("Computo già aperto. Impossibile sovrascrivere"))
            return None
        elif fileComputo is not None:
            root, ext = os.path.splitext(fileComputo)
            if ext != ".db":
                fileComputo =  root + ".db"
            # crea la progressbar
            self.progressbar = ProgressBar(self.builder, None, testo = None)
            # effettua il DUMP del computo correntemente aperto in memoria
            connection = self.preventivo.get_database()
            dump = connection.iterdump()
            # crea un nuovo database con il nuovo nome di file
            self.__filename = fileComputo
            nuovo_preventivo = Preventa.Preventivo(self.__filename, self.__preventa_settings)
            # cancella le tavole dal database
            nuovo_preventivo.delete_database()
            # esegue l'importazione delle istruzioni SQL sul nuovo database appena creato
            connection = nuovo_preventivo.get_database()
            cur = connection.cursor()
            for line in dump:
                self.progressbar.avanza()
                try:
                    cur.execute(line)
                except:
                    logger.error(_("Errore istruzione SQL non importata: %s") % line)
            # chiude la connessione del computo corrente
            self.preventivo.rollback_database()
            self.preventivo.connection_shutdown()
            # effettua il salvataggio del nuovo computo
            self.preventivo = nuovo_preventivo
            self.preventivo.save_database()
            # Aggiorna l'interfaccia grafica corrente
            self.Aggiorna_finestra_dialogo(self)
            self.Aggiorna_maschera_dati_generali(self)
            # Cambia il nome della finestra 
            window = self.builder.get_object("computo_window")
            window.set_title(_("Computo: %s" % self.preventivo.get_filename()))
            # distruggi la progressbar
            self.progressbar.destroy()
            return
        elif fileComputo is None:
            self.message(self, "errore", _("Impossibile salvare un nuovo computo, file inesistente"))
            return None
        else:
            return None

    def annulla_modifiche_computo(self, widget, data=None):
        """Esegue il rollback sul database."""
        if self.message(self, "domanda", _("Vuoi annullare tutte le modifiche effettuate finora?")):
            self.preventivo.rollback_database()
            self.Aggiorna_finestra_dialogo(self)

    def ricerca_preventivo(self, widget, icon_pos=None, event=None, data=None):
        """Funzione di ricerca nella maschera computo o lavorazioni.""" 
        if icon_pos is not None:
            if icon_pos.real == 1:
               self.entry_chiave_ricerca.set_text('')
               return
        notebook_page = self.notebook.get_current_page()
        model = self.box_campo_ricerca.get_model()
        iter = self.box_campo_ricerca.get_active_iter()
        campo = model.get_value(iter, 0)
        chiave = self.entry_chiave_ricerca.get_text()
        lista_computo = list()
        #TODO this will break if columns' names are translated!
        if campo == "Tariffa":
            column = "Tariffa"
        elif campo == "Descrizione Codice":
            column = "Descrizione_codice"
        elif campo == "Descrizione Voce":
            column = "Descrizione_voce"
        elif campo == "Descrizione Estesa":
            column = "Descrizione_estesa"
        elif campo == "Prezzo Unitario":
            column = "Prezzo_unitario"
        else:
            column = None
        # ricerca nel tab 'Computo'
        if notebook_page == 1:
            if column == "Descrizione_codice" or column == "Descrizione_voce" or column == "Descrizione_estesa":
                lista_epu = self.preventivo.search_tables ("Epu", column, chiave, "Tariffa")
                for art in lista_epu:
                    lista_computo.extend(self.preventivo.computo_rows_list(
                                              supercategoria= self.categorie[0], 
                                              categoria=self.categorie[1],
                                              subcategoria=self.categorie[2], 
                                              column= "Tariffa", 
                                              search_key = art[0],
                                              convert_to_string=True,
                                              contract_type=self.contract_type[0]))
            else:
                lista_computo = self.preventivo.computo_rows_list(
                                                supercategoria= self.categorie[0], 
                                                categoria=self.categorie[1],
                                                subcategoria=self.categorie[2], 
                                                column= column, 
                                                search_key = chiave,
                                                convert_to_string=True,
                                                contract_type=self.contract_type[0])
            self.liststore_computo.clear()
            for art in lista_computo:
                self.liststore_computo.append(art)
        # ricerca nel tab 'Epu'
        elif notebook_page == 2:
            tabella = "Epu"
            self.liststore_lavorazioni.clear()
            lista_lavorazioni = self.preventivo.lavorazioni_rows_list(
                                                supercapitolo= None, 
                                                capitolo=None, 
                                                subcapitolo=None, 
                                                column= column, 
                                                search_key = chiave,
                                                convert_to_string=True)
            for art in lista_lavorazioni:
                self.liststore_lavorazioni.append(art)
        # se l'articolo selezionato non è nullo selezionalo
        if self.articolo_computo is not None:
            self.Seleziona_articolo_corrente(self)

    def export_to_odt(self, widget, data=None):
        """Esporta il file in formato ODT per LibreOffice"""
        # ottieni l'elenco dei capitoli e delle categorie da passare al dialog
        cat_list = self.preventivo.categorie_rows_list()
        cap_list = self.preventivo.capitoli_rows_list()
        # ottieni le impostazioni di stampa
        settings_file_name = get_print_settings_file() 
        # carica le impostazioni di stampa da file (se salvate)
        try:
            pr_set = Gtk.print_settings_new_from_file(settings_file_name)
        except:
            pr_set = None
        # apri l'interfaccia delle opzioni di stampa
        print_opt = PrintoptionDialog.PrintoptionDialog(print_settings=pr_set, 
                                                        list_capitoli=cap_list, 
                                                        list_categorie=cat_list)
        # semplifica l'interfaccia 'spegnendo' le impostazioni che non servono
        print_opt.modify_oo_ui()
        response = print_opt.run()
        if response == Gtk.ResponseType.OK:
            # Recupera le impostazioni di stampa da usare x la conversione
            odt_settings = print_opt.print_settings
            tipo_stampa =  odt_settings.get("print_type")
            # get the font description and extract the font size and type
            font_type = odt_settings.get("font_type")
            fd = Pango.FontDescription.from_string(font_type)
            font = fd.get_family()
            size = fd.get_size()/Pango.SCALE
            #TODO remove ott model hardcoding
            odt_model = os.path.join(get_data_path(), "media",
                                     "preventares-cm.ott")
            #title = odt_settings.get("main_title")
            # categorie e i capitoli da stampare ('None' stampa tutto)
            cat = print_opt.cat_list
            cap = print_opt.cap_list
            # nasconde la finestra delle opzioni di stampa
            print_opt.hide()
            #TODO import ontop???
            a = PreventaConvertToWriter(self.preventivo,
                font=font,
                size=size,
                output_type=odt_settings.get("print_type"),
                descr_breve=odt_settings.get("computo_print_descr_voce"), 
                descr_estesa=odt_settings.get("computo_print_descr_estesa"), 
                note=odt_settings.get("computo_print_note"), 
                no_chapter=odt_settings.get("print_without_chapter"), 
                analisi=False, 
                imageURL="file:///" + odt_settings.get("owner_image"),
                odt_model=odt_model)
        else:
            pass #does the operation ends here?
        print_opt.destroy()
        return True

###### MENU COMPUTO ####################################################

    def ricalcolo_generale(self, widget, data=None):
        """Funzione di ricalcolo generale del preventivo.""" 
        total = self.preventivo.ricalcolo_generale()
        self.Aggiorna_finestra_dialogo(self)

    def ricalcolo_totale_preventivo(self, widget, data=None):
        """Funzione di ricalcolo del totale del preventivo.""" 
        self.Aggiorna_BarraStato_default()

    def aggiorna_articoli_listino_da_archivio(self, widget, data=None):
        """Funzione di ricalcolo del totale del preventivo."""
        if self.message(self, "avviso", _("Questa opzione modificherà tutti gli articoli di listino del computo")):
            if self.__filelistino is None:
                self.message(self, "errore", _("Impossibile aprire il database listino, file inesistente"))
                return False
            # aprire il database metel
            archivio_metel = metel_lib.MetelClass(filename=self.__filelistino)
            # ottenere la lista di tutti i listini
            art_listino_archivio = archivio_metel.extract_metel_rows()
            # ottenere la lista di tutti i listini del computo
            art_listino_cmp = self.preventivo.listino_rows_list()
            # crea la progressbar
            n_articoli = len(art_listino_archivio)
            self.progressbar = ProgressBar(self.builder, n_articoli/100, 
                               testo = "Lettura archivio articoli")
            n = 0
            # crea un dizionario con chiave il 'codice metel' e valori tutte le prymary key con quel codice
            diz_codici_archivio = dict() # {codice: (prezzo, data,sconto)}
            for art in art_listino_archivio:
                marca = art[1]
                codice = marca + art[2] + art[12]
                prezzo = art[10]
                #mul = art[11]
                data = art[16]
                sconto = archivio_metel.get_sconto_marca (marca)
                diz_codici_archivio[codice] = (prezzo,data,sconto)
                n+=1
                # avanzamento barra di stato solo ogni 100 articoli (x rendere più veloce l'operazione)
                if divmod(n,100)[1] == 0:
                    self.progressbar.avanza(new_text = "Lettura archivio articoli {0:d}/{1:d}".format(
                                            n, n_articoli))
            # distruggi la progressbar
            self.progressbar.destroy()
            #while Gtk.events_pending(): Gtk.main_iteration()
            # inizializzazione elenco codici non aggiornati e art da aggiornare
            lista_art_non_agg = list()
            lista_art_aggiornare = list()
            # crea la progressbar
            n_articoli = len(art_listino_cmp)
            self.progressbar = ProgressBar(self.builder, n_articoli, 
                               testo = "Aggiornamento articoli computo")
            n = 0
            # effettua il parsing delle voci di listino del computo
            # se ha dei codici uguali all'archivio metel ne aggiorna il prezzo
            for art in art_listino_cmp:
               # ottieni l'articolo da aggiornare
               prymary_key = art[0]
               articolo = self.preventivo.copia_articolo_listino(prymary_key)
               # verifica se è da aggiornare
               if articolo.codice in diz_codici_archivio:
                   # aggiorna il prezzo dell'articolo e la data
                   articolo.prezzo_unitario = diz_codici_archivio[articolo.codice][0]
                   articolo.note = diz_codici_archivio[articolo.codice][1] #data articolo
                   articolo.sconto = diz_codici_archivio[articolo.codice][2] #sconto articolo
                   # aggiorna il prezzo dell'articolo e la data 
                   #self.preventivo.update_articolo_listino (prymary_key, articolo) #FIXME
                   lista_art_aggiornare.append(articolo) 
                   # effettua il log dell'articolo da aggiornare
                   logger.debug("Articolo di listino n.'{0.nr}' e codice '{0.codice}' da aggiornare.".format(articolo))
               else:
                   #lista_art_non_agg.append((articolo.tariffa, articolo.codice))
                   logger.info("Articolo non aggiornato: Tariffa:'{0.tariffa}', Articolo:'{0.codice}'".format(articolo))
               # avanzamento barra di stato
               n+=1
               self.progressbar.avanza(new_text = "Aggiornamento articoli {0:d}/{1:d}".format(n, n_articoli))
            # distruggi la progressbar
            self.progressbar.destroy()
            del art_listino_cmp
            # aggiorna il prezzo dell'articolo e la data
            if len(lista_art_aggiornare) > 0:
               logger.debug("Avvio aggiornamento codici di listino.")
               self.preventivo.update_lista_art_listino (None, None, None, None, None, *lista_art_aggiornare) #FIXME
            #if len(lista_art_non_agg) > 0:
            #    for tariffa, codice in lista_art_non_agg:
            #        logger.info("Articolo non aggiornato: Tariffa:'{0}', Articolo:'{1}'".format(
            #                     tariffa, codice))
            #TODO aprire anche una finestra con l'elenco degli articoli non aggiornati
            pass 
            # elimina i dati non più utili
            del lista_art_aggiornare
            del diz_codici_archivio
        else:
            return False

    def modifica_tab_manodopera(self, widget, data=None):
        """Funzione per modificare la tabella della manodopera e calcolarne la media"""
        # avvia la finestra di dialogo
        modify = ManodoperaDialog.ManodoperaDialog(self.preventivo)
        response = modify.run()
        if response == Gtk.ResponseType.OK:
            # aggiorna la finestra di dialogo.
            logger.debug("Modifiche apportate alla tabella 'Manodopera'")
            self.Aggiorna_finestra_dialogo(self, aggiorna_tutto=False)
        else:
            pass
        modify.destroy()
        return True

###### MENU EDIT ####################################################

    def menu_nuovo(self, widget, data=None):
        """Funzione di copia nella maschera computo o epu.""" 
        notebook_page = self.notebook.get_current_page()
        # copia nel tab 'Computo'
        if notebook_page == 1:
            self.nuovo_articolo_computo(self)
        # copia nel tab 'Epu'
        elif notebook_page == 2:
            self.nuovo_articolo_lista_lavorazioni(self)

    def menu_copia(self, widget, data=None):
        """Funzione di copia nella maschera computo o epu.""" 
        notebook_page = self.notebook.get_current_page()
        # copia nel tab 'Computo'
        if notebook_page == 1:
            articolo = self.copia_articolo_computo()
        # copia nel tab 'Epu'
        elif notebook_page == 2:
            articolo = self.copia_articolo_lista_lavorazioni(self)

    def menu_incolla(self, widget, data=None):
        """Funzione di incolla nella maschera computo o epu.""" 
        notebook_page = self.notebook.get_current_page()
        # incolla nel tab 'Computo'
        if notebook_page == 1:
            self.incolla_articolo_computo()
        # incolla nel tab 'Epu'
        elif notebook_page == 2:
            self.incolla_articolo_lista_lavorazioni(self)

    def menu_cancella(self, widget, data=None):
        """Funzione di cancella nella maschera computo o epu.""" 
        notebook_page = self.notebook.get_current_page()
        # cancella nel tab 'Computo'
        if notebook_page == 1:
            self.cancella_articolo_computo(self)
        # cancella nel tab 'Epu'
        elif notebook_page == 2:
            self.cancella_articolo_lista_lavorazioni(self)

    def menu_taglia(self, widget, data=None):
        """Funzione di taglia nella maschera computo o epu.""" 
        notebook_page = self.notebook.get_current_page()
        # taglia nel tab 'Computo'
        if notebook_page == 1:
            articoli = self.copia_articolo_computo()
            self.cancella_articolo_computo(self)
            if len(articoli) == 1:
                self.articolo_computo = articoli[0]
            elif len(articoli) > 1:
                self.lista_articoli_computo = articoli
        # taglia nel tab 'Epu'
        elif notebook_page == 2:
            articoli = self.copia_articolo_lista_lavorazioni(self)
            self.cancella_articolo_lista_lavorazioni(self)
            if len(articoli) == 1:
                self.articolo_epu = articoli[0]
            elif len(articoli) > 1:
                self.lista_articoli_epu = articoli

    def menu_duplica(self, widget, data=None):
        """Funzione di incolla nella maschera computo o epu.""" 
        notebook_page = self.notebook.get_current_page()
        # incolla nel tab 'Computo'
        if notebook_page == 1:
            self.duplica_articolo_computo(self)
        # incolla nel tab 'Epu'
        elif notebook_page == 2:
            self.duplica_articolo_lista_lavorazioni(self)

    def menu_seleziona_tutto(self, widget, data=None):
        """Funzione per selezionare tutte le righe nella maschera computo o epu.""" 
        notebook_page = self.notebook.get_current_page()
        # seleziona nel tab 'Computo'
        if notebook_page == 1:
            selection = self.treeview_computo.get_selection()
            selection.select_all()
            self.row_selected_computo(self)
        # seleziona nel tab 'Epu'
        elif notebook_page == 2:
            selection = self.treeview_lavorazioni.get_selection()
            selection.select_all()
            self.row_selected_lista_lavorazioni(self)

    def menu_deseleziona_tutto(self, widget, data=None):
        """Funzione per deselezionare tutte le righe nella maschera computo o epu.""" 
        notebook_page = self.notebook.get_current_page()
        # deseleziona nel tab 'Computo'
        if notebook_page == 1:
            selection = self.treeview_computo.get_selection()
            selection.unselect_all()
            self.articolo_computo = None
            self.lista_articoli_computo = None
        # deseleziona nel tab 'Epu'
        elif notebook_page == 2:
            selection = self.treeview_lavorazioni.get_selection()
            selection.unselect_all()
            self.articolo_epu = None
            self.lista_articoli_epu = None

################################################################################
########## PULSANTI TOOLBAR  ###################################################
################################################################################

    def nuovo_articolo_computo(self, widget, data=None):
        """inserisce un nuovo articolo in computo."""
        index = settings["character_nr_for_descr_cod_from_descr_estesa"]
        articolo_name = settings["new_articolo_name"]
        if data is not None and type(data) == str:
            articolo_name = data[:index]
        self.articolo_computo = self.preventivo.nuovo_articolo_computo ("0001", articolo_name,
                                 descrizione_voce=None, descrizione_estesa=data, 
                                 supercategoria = self.categorie[0], categoria = self.categorie[1], 
                                 subcategoria = self.categorie[2])
        self.Aggiorna_finestra_dialogo(self)

    def cancella_articolo_computo(self, widget, data=None):
        """inserisce un nuovo articolo in computo."""
        if self.articolo_computo is not None:
            if self.message(self, "domanda", _("Vuoi cancellare l'articolo N.%d?") % self.prymary_key):
                self.preventivo.delete_articoli_computo(self.articolo_computo)
        elif self.lista_articoli_computo is not None:
            if self.message(self, "domanda", _("Vuoi cancellare tutti gli articoli selezionati?")):
                self.preventivo.delete_articoli_computo(*self.lista_articoli_computo)
        else:
            if self.message(self, "avviso", _("Nessun articolo selezionato da cancellare")):
                return False
        self.Aggiorna_finestra_dialogo(self)

    def copia_articolo_computo(self, data=None):
        """copia l'articolo corrente del computo."""
        selection = self.treeview_computo.get_selection()
        model, paths = selection.get_selected_rows()
        if selection.count_selected_rows() == 1:
            self.lista_articoli_computo = None
            data = [self.articolo_computo]
        elif selection.count_selected_rows() > 1:
            self.prymary_key = None
            self.articolo_computo = None
            data = self.lista_articoli_computo
        else: return
        # save the current data in the clipboard
        self.clipboard.set_data(self.ARTICOLI_COMPUTO, data)
        return data

    def incolla_articolo_computo(self, data=None):
        """incolla articolo in computo nella posizione (categorie) correnti."""
        # esegue l'istruzione per incollare un articolo in computo
        if self.articolo_computo is not None:
            self.preventivo.incolla_articolo_computo(self.categorie[0], 
                                                     self.categorie[1], 
                                                     self.categorie[2], 
                                                     self.articolo_computo)
        elif self.lista_articoli_computo is not None:
            self.preventivo.incolla_articolo_computo(self.categorie[0], 
                                                     self.categorie[1], 
                                                     self.categorie[2],
                                                     *self.lista_articoli_computo)
        else:
            # se entrambe le liste sono nulle allora incolla dalla clipboard
            self.lista_articoli_computo = self.clipboard.get_data(self.ARTICOLI_COMPUTO)
            #self.clipboard.request_contents(self.ARTICOLI_COMPUTO, self.clipboard_cb_articoli, None)
            # se la lista non è nulla o vuota incolla gli articoli della lista in computo 
            if self.lista_articoli_computo is not None and len(self.lista_articoli_computo) > 0:
                self.preventivo.insert_articoli_from_archivio_computo(*self.lista_articoli_computo)
            # in tutti gli altri casi dai un messaggio di errore
            else:
                if self.message(self, "avviso", _("Nessun articolo copiato, copia o seleziona uno o più articoli che vuoi incollare")):
                    return False
        # dopo aver incollato pulisce la memoria dagli art memorizzati
        self.__cancella_articoli_memoria()
        # aggiorna l'interfaccia grafica
        self.Aggiorna_finestra_dialogo(self)
        return True

    def duplica_articolo_computo(self, widget, data=None):
        """duplica l'articolo di computo selezionato"""
        if self.articolo_computo is not None:
            lista_articoli = self.preventivo.insert_articoli_epu(True, self.articolo_computo)
        elif self.lista_articoli_computo is not None:
            lista_articoli = self.preventivo.insert_articoli_epu(True, *self.lista_articoli_computo)
            self.preventivo.insert_articoli_computo(*lista_articoli)
        elif self.articolo_computo is None:
            self.message(self, "errore", _("Nessun articolo selezionato. Impossibile duplicare l'articolo"))
            return False
        if len(lista_articoli) > 0:
            self.preventivo.insert_articoli_computo(*lista_articoli)
        self.Aggiorna_finestra_dialogo(self)

    def carica_listini_metel(self, widget, data=None):
        """carica la finestra dei listini metel"""
        if self.__filelistino is None:
            self.message(self, "errore", _("Impossibile aprire il database listino, file inesistente"))
            return False
        else:
            self.metel = MetelWindow.MetelWindow(filename= self.__filelistino)
            self.metel.show()
            Gtk.main()
            return True

    def modifica_blocco_articoli(self, widget, data=None):
        """Funzione per modificare in blocco le caratteristiche di articoli di epu"""
        liststore_capitoli = (self.liststore_supercapitolo, self.liststore_capitolo, self.liststore_subcapitolo)
        liststore_categorie = (self.liststore_supercategoria, self.liststore_categoria, self.liststore_subcategoria)
        liststore_appalto = (self.liststore_CONTRACT_TYPE, self.liststore_CONTRACT_CATEGORY)
        lista_articoli = None
        # verifica in quale pagina si trova il notebook (0= dati generali, 
        #1= computo, 2= epu) per ricavare la lista degli articoli
        notebook_page = self.notebook.get_current_page()
        if notebook_page == 1:
            if self.articolo_computo is not None:
                lista_articoli = [self.articolo_computo]
            elif self.lista_articoli_computo is not None:
                lista_articoli = self.lista_articoli_computo
            else:
                lista_articoli = None
        elif notebook_page == 2:
            if self.articolo_epu is not None:
                lista_articoli = [self.articolo_epu]
            elif self.lista_articoli_epu is not None:
                lista_articoli = self.lista_articoli_epu
            else:
                lista_articoli = None
        if lista_articoli is None:
            self.message(self, "errore", _("Nessun articolo selezionato da modificare."))
            return None
        # avvia la finestra di modifica
        modify = ModifyDialog.ModifyDialog(liststore_capitoli, 
                                           liststore_categorie,
                                           liststore_appalto,
                                           self.liststore_um)
        response = modify.run()
        if response == Gtk.ResponseType.OK:
            # Make any updates based on changed attribute here.
            p = modify.parametri
            scambio_costi = modify.scambio_costi
            modifica_computo = modify.modifica_computo
            variazione_prezzi = modify.variazione_prezzi
            if self.message(self, "domanda", _("Applicare le opzioni selezionate a tutti gli articoli della selezione?")):
                # avviare una progressbar
                text, step = (_("Operazione"), 5)
                progressbar = ProgressBar(self.builder, step, window_mode = True, modal=True)
                progressbar.avanza(new_text="{0} {1}/{2}".format(text, 1, step))
                while Gtk.events_pending():
                    Gtk.main_iteration()
                # recupera le impostazioni
                result = self.preventivo.update_articoli_epu (p[0], p[1], p[2], 
                   p[3], p[4], p[5], p[6], p[7], p[8], p[9], p[10], p[11], 
                   *lista_articoli)
                # avanza la progressbar
                progressbar.avanza(new_text="{0} {1}/{2}".format(text, 2, step))
                while Gtk.events_pending():
                    Gtk.main_iteration()
                # funzione per modificare di parti di computo.
                if modifica_computo:
                    self.preventivo.update_articoli_computo (p[12], p[13], p[14], 
                                    None, None, p[18], p[19], None, *lista_articoli)
                # avanza la progressbar
                progressbar.avanza(new_text="{0} {1}/{2}".format(text, 3, step))
                while Gtk.events_pending():
                    Gtk.main_iteration()
                # funzione per scambio costi.
                if scambio_costi is not None:
                    self.preventivo.scambia_prezzi_articoli_epu (scambio_costi[0], scambio_costi[1], *lista_articoli)
                # avanza la progressbar
                progressbar.avanza(new_text="{0} {1}/{2}".format(text, 4, step))
                while Gtk.events_pending():
                    Gtk.main_iteration()
                # funzione per gli aumenti e diminuzioni dei prezzi.
                if variazione_prezzi:
                    self.preventivo.modifica_prezzi (p[15], p[16], p[17], *lista_articoli)
                # avanza la progressbar
                progressbar.avanza(new_text="{0} {1}/{2}".format(text, 5, step))
                while Gtk.events_pending():
                    Gtk.main_iteration()
                # distruggi la progressbar
                progressbar.destroy()
            # aggiorna la finestra di dialogo
            self.Aggiorna_finestra_dialogo(self, aggiorna_tutto=False)
        modify.destroy()
        return True

################################################################################
########## COMBOBOX - CATEGORIE PER APPALTO ####################################
################################################################################
    def on_combobox_appalto_changed(self, widget, data=None):
        """callback per la selezione degli articoli di computo secondo la 
        tipologia di contratto di appalto (lavori a corpo, ecc.)"""
        a, b = self.contract_type
        a = widget.get_active()-1 #il primo elem. del modello serve solo a visualizzare tutte le tipologie
        if a > -1:
            self.contract_type = (a,b)
        else:
            self.contract_type = (None,b)
        # aggiorna la treeview computo
        self.Aggiorna_treeview_computo(self)
        # aggiorna la treeview categorie
        self.Aggiorna_treeview_categorie(self)
################################################################################
########## TREEVIEW - CATEGORIE ################################################
################################################################################
    def Seleziona_categoria_corrente(self, widget, data=None):
        """posiziona il selettore sul rigo corrente"""
        path = Gtk.TreePath()
        if self.categorie is not None:
            a, b, c = self.categorie
            if a is None: a = 0
            if b is None: b = 0
            if c is None: c = 0
            model = self.treeview_categorie.get_model()
            nome_sup, nome_cat, nome_sub = self.preventivo.get_categorie_name_from_id (a, b, c)
            iter = model.get_iter_first()
            # cerca le supercategorie
            while iter is not None:
                nr = int(model.get_value(iter, 0))
                nome = model.get_value(iter, 1)
                if nr == a and nome == nome_sup:
                    path = model.get_path(iter)
                    break
                iter = model.iter_next(iter)
            # cerca le categorie
            iter = model.iter_children(iter)
            while iter is not None:
                nr = int(model.get_value(iter, 0))
                nome = model.get_value(iter, 1)
                if nr == b and nome == nome_cat:
                    path = model.get_path(iter)
                    break
                iter = model.iter_next(iter)
            # cerca le subcategorie
            iter = model.iter_children(iter)
            while iter is not None:
                nr = int(model.get_value(iter, 0))
                nome = model.get_value(iter, 1)
                if nr == c and nome == nome_sub:
                    path = model.get_path(iter)
                    break
                iter = model.iter_next(iter)
        # converti la path e selezionala
        self.treeview_categorie.expand_to_path(path)
        selection = self.treeview_categorie.get_selection()
        selection.select_path(path)
        return path

    def on_treeview_categorie_button_press_event(self, widget, event, data=None):
        """Funzione per richiamare un menu popup alla pressione di un tasto del mouse"""
        if event.button == 3:
            x = int(event.x)
            y = int(event.y)
            time = event.time
            pthinfo = self.treeview_categorie.get_path_at_pos(x, y)
            if pthinfo is not None:
                path, col, cellx, celly = pthinfo
                self.treeview_categorie.grab_focus()
                self.treeview_categorie.set_cursor( path, col, 0)
                for n in range(4):
                    self.popup_categorie.items[n].show()
            else:
                # mostra solo i pulsanti 'visualizza tutte le categorie' e 'Aggiorna tutto'
                for n in range(4):
                    if n not in [2,4]:
                        self.popup_categorie.items[n].hide()
            # Crea e predispone un menu interattivo popup da utilizzare contestualmente
            self.popup_categorie.popup(None, None, None, None, event.button, time)
            return True
        else:
            return False

    def espandi_treeview_categorie(self, widget, data=None):
        """Espande la treeview"""
        self.treeview_categorie.expand_all()

    def collassa_treeview_categorie(self, widget, data=None):
        """Collassa la treeview"""
        self.treeview_categorie.collapse_all()

    def row_selected_categorie(self, widget, data=None):
        """Funzione chiamata quando la riga viene selezionata, mette in memoria la categoria corrente"""
        selection = self.treeview_categorie.get_selection()
        if selection is None: return
        model, iter = selection.get_selected()
        if iter is None: # se non c'è nessun rigo selezionato
            #iter = model.get_iter_first()
            path = Gtk.TreePath.new()
        else:
            path = model.get_path(iter)
        if path.get_depth() >= 1:
            new_path = path.new_from_string(str(path).split(":")[0])
            a = int(model.get_value(model.get_iter(new_path),0))
        else: a = None
        if path.get_depth() >= 2:
            stringa = "{0}:{1}".format(path.get_indices()[0], path.get_indices()[1])
            new_path = path.new_from_string(str(stringa))
            b = int(model.get_value(model.get_iter(new_path), 0))
        else: b = None
        if path.get_depth() == 3:
            c = int(model.get_value(iter,0))
        else: c = None
        self.categorie = (a, b, c)
        # imposta la categoria 0 per la visualizzazione di tutto il computo
        if self.categorie == (0, None, None):
           self.categorie = (None, None, None)
        #visualizza solo gli articoli di computo della categoria inerente
        self.liststore_computo.clear()      
        for articolo in self.preventivo.computo_rows_list(
                                        supercategoria= self.categorie[0], 
                                        categoria=self.categorie[1], 
                                        subcategoria=self.categorie[2],
                                        convert_to_string=True,
                                        contract_type=self.contract_type[0]): 
            self.liststore_computo.append(articolo)
        return

    def nuova_categoria(self, widget, data=None):
        """Inserisce una nuova categoria"""
        if self.categorie is None:
            if self.message(self, "avviso", _("Impossibile creare categorie, seleziona prima un ramo delle categorie")):
                return False
        if self.categorie[1] is None:
            self.preventivo.insert_capitoli_categorie("Supercategoria", "Nuova Supercategoria")
        elif self.categorie[2] is None:
            self.preventivo.insert_capitoli_categorie("Categoria", "Nuova Categoria")
        else:
            self.preventivo.insert_capitoli_categorie("Subcategoria", "Nuova Supercategoria")
        # se disabilitata attivare la visualizzazione delle categorie vuote
        if not settings["show_empty_category"]:
            self.popup_categorie.items[4].set_active(True)
        else:
            self.Aggiorna_treeview_categorie(self)


    def cell_nome_categoria_edited(self, cell, path, new_text, col_nr):
        """Called when a text cell is edited.  It puts the new text in the model so that it is displayed properly."""
        model = self.treeview_categorie.get_model()
        model[path][col_nr] = new_text
        sup, cat, sub = self.categorie
        if cat is None and sub is None:
            result = self.preventivo.update_capitoli_categorie (sup, "Supercategoria", new_text)
        elif sub is None:
            result = self.preventivo.update_capitoli_categorie (cat, "Categoria", new_text)
        elif sup is not None and cat is not None and sub is not None:
            result = self.preventivo.update_capitoli_categorie (sub, "Subcategoria", new_text)
        self.Aggiorna_liststore_categorie(self)
        self.Seleziona_categoria_corrente(self)
        return

    def cancella_categoria(self, widget, data=None):
        """Cancella una categoria esistente"""
        if self.categorie is None:
            if self.message(self, "avviso", _("Impossibile eliminare una categoria, nessuna categoria è selezionata")):
                return False
        if self.categorie[1] is None:
            self.preventivo.delete_capitoli_categorie("Supercategoria", self.categorie[0], column = None)
        elif self.categorie[2] is None:
            self.preventivo.delete_capitoli_categorie("Categoria", self.categorie[1], column = None)
        else:
            self.preventivo.delete_capitoli_categorie("Subcategoria", self.categorie[2], column = None)
        self.Aggiorna_treeview_categorie(self)

    def Visualizza_categorie_vuote(self, widget, data=None):
        """Funzione per visualizzare nella treeview anche le categorie vuote"""
        if self.popup_categorie.items[4].get_active():
            settings["show_empty_category"] = True
        elif not self.popup_categorie.items[4].get_active():
            settings["show_empty_category"] = False
        self.Aggiorna_treeview_categorie(self, visualizza_tutti=None)
        self.Seleziona_categoria_corrente(self)
################################################################################
########## TREEVIEW - EPU ######################################################
################################################################################

    def Seleziona_articolo_treeview_epu(self, widget, data=None):
        """posiziona il selettore sull'articolo epu in memoria"""
        iter3 = None
        if self.articolo_epu is not None:
            selection = self.treeview_epu.get_selection()
            model = self.treeview_epu.get_model()
            iter = model.get_iter_first()
            tariffa = model.get_value(iter, 0)
            while iter is not None:
                iter1 = model.iter_children(iter)
                while iter1 is not None:
                    iter2 = model.iter_children(iter1)
                    while iter2 is not None:
                        iter3 = model.iter_children(iter2)
                        while iter3 is not None:
                            tariffa = model.get_value(iter3, 0)
                            if tariffa == self.articolo_epu.tariffa:
                                break
                            iter3 = model.iter_next(iter3)
                        iter2 = model.iter_next(iter2)
                    iter1 = model.iter_next(iter1)
                iter = model.iter_next(iter)
            if iter3 is not None:
                # seleziona il rigo corrispondente
                path = model.get_path(iter3)
            else: path = Gtk.TreePath()#(0,)
            self.treeview_epu.expand_to_path(path)
            selection.select_path(path)
            # effettua lo scrolling della treeview fino all'articolo corrente
            self.treeview_epu.scroll_to_cell(path, column=None, 
                use_align=False, row_align=0.0, col_align=0.0)
            return path
        else:
            return None

    def on_treeview_epu_button_press_event(self, widget, event, data=None):
        """Funzione per richiamare un menu popup alla pressione di un tasto del mouse"""
        if event.button == 3:
            x = int(event.x)
            y = int(event.y)
            time = event.time
            pthinfo = self.treeview_epu.get_path_at_pos(x, y)
            if pthinfo is not None:
                path, col, cellx, celly = pthinfo
                self.treeview_epu.grab_focus()
                self.treeview_epu.set_cursor( path, col, 0)
                # Crea e predispone un menu interattivo popup da utilizzare contestualmente
                self.popup_epu.popup(None, None, None, None, event.button, time)
            else:
                self.popup_epu.popup(None, None, None, None, event.button, time)
            return True
        elif event.button == 2:
            pass
            return True 
        else:
            return False

    def espandi_treeview_epu(self, widget, data=None):
        """Espande la treeview"""
        self.treeview_epu.expand_all()

    def collassa_treeview_epu(self, widget, data=None):
        """Collassa la treeview"""
        self.treeview_epu.collapse_all()

    def row_selected_epu(self, widget, data=None):
        """Funzione chiamata quando la riga viene selezionata, mette in memoria l'articolo"""
        selection = widget.get_selection()
        if selection is None: return
        model, iter = selection.get_selected()
        self.lista_articoli_epu = None
        if iter is None:
            self.articolo_epu = None
        else:
            tariffa = model.get_value(iter, 0)
            if self.archivio is not None:
                archivio = self.archivio
            else:
                archivio = self.preventivo
            self.articolo_epu = archivio.copia_articolo_epu (tariffa, 
                                         supercategoria = self.categorie[0], 
                                         categoria = self.categorie[1], 
                                         subcategoria = self.categorie[2], 
                                         copia_nomi_capitoli= True)

    def incolla_articolo_archivio_principale(self, data=None):
        """Funzione per incollare l'articolo dall'archivio nel computo"""
        # se notebook si trova nella pag. epu inserisci gli articoli solo lì e 
        # non nella tab. db 'Computo' (0= dati generali, 1= computo, 2= epu)
        notebook_page = self.notebook.get_current_page()
        if notebook_page == 2:
            insert_cmp=False
        else:
            insert_cmp=True
        if self.articolo_epu is not None:
            self.preventivo.insert_articoli_from_archivio_epu (insert_cmp, 
                self.articolo_epu)
        elif self.lista_articoli_epu is not None:
            self.preventivo.insert_articoli_from_archivio_epu (insert_cmp,
                *self.lista_articoli_epu)
        else:
            # se entrambe le liste sono 'None' allora incolla dalla clipboard
            self.lista_articoli_epu = self.clipboard.get_data(self.ARTICOLI_EPU)
            #self.clipboard.request_contents(self.target_epu, self.clipboard_cb_articoli, None)
            # se la lista è ancora nulla dai un messaggio di errore
            if self.lista_articoli_epu is None:
                if self.message(self, "avviso", _("Nessun articolo copiato, copia o seleziona uno o più articoli che vuoi incollare")):
                    return False
            # se la lista non è vuota incolla gli articoli della lista in computo 
            elif len(self.lista_articoli_epu) > 0:
                self.preventivo.insert_articoli_from_archivio_epu (
                                insert_cmp, *self.lista_articoli_epu)
        self.Aggiorna_finestra_dialogo(self)
        return True

    def cerca_articolo_epu(self, widget, icon_pos=None, event=None, data=None):
        """Funzione per ricercare e filtrare gli articoli nella treeview dell'Epu"""
        if icon_pos is not None:
            if icon_pos.real == 0:
               self.treeview_epu.set_model(self.treestore_epu)
            elif icon_pos.real == 1:
               self.entry_ricerca_epu.set_text('')
               self.treeview_epu.set_model(self.treestore_epu)
               return
        # se la chiave è nulla imposto 'treestore_epu' come treestore dell'epu
        chiave = self.entry_ricerca_epu.get_text()
        if chiave == str():
            self.treeview_epu.set_model(self.treestore_epu)
            return
        else:
            # creo un filtro di ricerca x il treestore dell'epu
            modelfilter = self.treestore_epu.filter_new()
            modelfilter.set_visible_func(self.__filter_epu_by_value_, chiave)
            # imposto il filtro di ricerca come treestore dell'epu
            self.treeview_epu.set_model(modelfilter)
            # espande la treeview
            self.treeview_epu.expand_all()

    def __filter_epu_by_value_(self, model, iter, data):
        """imposta la visibilita dell'elemento in base alla chiave di ricerca presente nella colonna n.2"""
        if model.iter_has_child(iter):
            return True
        column = self.box_campo_ricerca_epu.get_active()
        value = model.get_value(iter, column)
        if data is None: data = str()
        if value is None: value = str()
        return (data.upper() in value.upper())

    def carica_archivio_epu(self, widget, data=None):
        """Funzione per caricare l'archivio principale, un archivio esterno oppure l'Epu"""
        if not widget.get_active():
            return None
        if self.popup_epu.items[3].get_active():
            settings["load_main_archive"] = False
            self.archivio = None
        elif self.popup_epu.items[4].get_active():
            settings["load_main_archive"] = True
            self.archivio = self.__archivio_principale
        elif self.popup_epu.items[5].get_active():
            settings["load_main_archive"] = False
            # apre la finestra di dialogo per la scelta dell'archivio esterno
            dialog = FileChooserDialog(testo="Apri un computo esistente", mode="open",
                                   filetype="db", default_path=settings["default_working_directory"])
            fileComputo = dialog.run()
            if fileComputo == self.__filename:
                self.message(self, "errore", _("Computo già aperto. Impossibile aprirlo"))
                return None
            elif fileComputo is None:
                self.message(self, "errore", _("Impossibile aprire un nuovo computo, file inesistente"))
                return None
            # se il file esiste carica l'archivio esterno
            elif fileComputo is not None:
                self.archivio = Preventa.Preventivo(fileComputo, self.__preventa_settings)
        self.articolo_epu = None
        self.lista_articoli_epu = None
        self.Aggiorna_treeview_epu(self, archivio_esterno = self.archivio)

################################################################################
########## TREEVIEW - CAPITOLI #################################################
################################################################################

    def Seleziona_capitolo_corrente(self, widget, data=None):
        """posiziona il selettore sul rigo corrente"""
        path = Gtk.TreePath()
        if self.capitoli is not None:
            a, b, c = self.capitoli
            if a is None: a = 0
            if b is None: b = 0
            if c is None: c = 0
            model = self.treeview_capitoli.get_model()
            nome_sup, nome_cat, nome_sub = self.preventivo.get_capitoli_name_from_id (a, b, c)
            iter = model.get_iter_first()
            # cerca le supercapitoli
            while iter is not None:
                nr = int(model.get_value(iter, 0))
                nome = model.get_value(iter, 1)
                if nr == a and nome == nome_sup:
                    path = model.get_path(iter)
                    break
                iter = model.iter_next(iter)
            # cerca le capitoli
            iter = model.iter_children(iter)
            while iter is not None:
                nr = int(model.get_value(iter, 0))
                nome = model.get_value(iter, 1)
                if nr == b and nome == nome_cat:
                    path = model.get_path(iter)
                    break
                iter = model.iter_next(iter)
            # cerca le subcapitoli
            iter = model.iter_children(iter)
            while iter is not None:
                nr = int(model.get_value(iter, 0))
                nome = model.get_value(iter, 1)
                if nr == c and nome == nome_sub:
                    path = model.get_path(iter)
                    break
                iter = model.iter_next(iter)
        # converti la path e selazionala
        self.treeview_capitoli.expand_to_path(path)
        selection = self.treeview_capitoli.get_selection()
        selection.select_path(path)
        return path

    def on_treeview_capitoli_button_press_event(self, widget, event, data=None):
        """Funzione per richiamare un menu popup alla pressione di un tasto del mouse"""
        if event.button == 3:
            x = int(event.x)
            y = int(event.y)
            time = event.time
            pthinfo = self.treeview_capitoli.get_path_at_pos(x, y)
            if pthinfo is not None:
                path, col, cellx, celly = pthinfo
                self.treeview_capitoli.grab_focus()
                self.treeview_capitoli.set_cursor( path, col, 0)
                # mostra le opzioni, 'elimina' capitolo e 'Aggiorna tutto'
                for n in [0,1,2,3]:
                    self.popup_capitoli.items[n].show()
            else:
                # nasconde le opzioni, 'elimina' capitolo e 'Aggiorna tutto'
                for n in [0,1,2,3]:
                    self.popup_capitoli.items[n].hide()
            # Crea e predispone un menu interattivo popup da utilizzare contestualmente
            self.popup_capitoli.popup(None, None, None, None, event.button, time)
            return True
        else:
            return False

    def row_selected_capitoli(self, widget, data=None):
        """Funzione chiamata quando la riga viene selezionata, mette in memoria il capitolo"""
        selection = self.treeview_capitoli.get_selection()
        if selection is None: return
        model, iter = selection.get_selected()
        if iter is None: # se non c'è nessun rigo selezionato
            #iter = model.get_iter_first()
            path = Gtk.TreePath.new()
        else:
            path = model.get_path(iter)
        #print path
        if path.get_depth() >= 1:
            new_path = path.new_from_string(str(path).split(":")[0])
            a = int(model.get_value(model.get_iter(new_path),0))
        else: a = None
        if path.get_depth() >= 2:
            stringa = "{0}:{1}".format(path.get_indices()[0], path.get_indices()[1])
            new_path = path.new_from_string(str(stringa))
            b = int(model.get_value(model.get_iter(new_path), 0))
        else: b = None
        if path.get_depth() == 3:
            c = int(model.get_value(iter,0))
        else: c = None
        self.capitoli = (a, b, c)
        if self.capitoli == (0, None, None):
           self.capitoli = (None, None, None)
        #visualizza solo gli articoli del capitolo selezionato
        self.liststore_lavorazioni.clear()
        for articolo in self.preventivo.lavorazioni_rows_list(
                                        supercapitolo= self.capitoli[0], 
                                        capitolo=self.capitoli[1], 
                                        subcapitolo=self.capitoli[2],
                                        convert_to_string=True): 
            self.liststore_lavorazioni.append(articolo)
        return


    def espandi_treeview_capitoli(self, widget, data=None):
        """Espande la treeview"""
        self.treeview_capitoli.expand_all()

    def collassa_treeview_capitoli(self, widget, data=None):
        """Collassa la treeview"""
        self.treeview_capitoli.collapse_all()

    def nuovo_capitolo(self, widget, data=None):
        """Inserisce un nuovo capitolo"""
        if self.capitoli is None:
            if self.message(self, "avviso", _("Impossibile creare un nuovo capitolo, seleziona prima un ramo dei capitoli")):
                return False
        if self.capitoli[1] is None:
            self.preventivo.insert_capitoli_categorie("Supercapitolo", "Nuovo Supercapitolo")
        elif self.capitoli[2] is None:
            self.preventivo.insert_capitoli_categorie("Capitolo", "Nuovo Capitolo")
        else:
            self.preventivo.insert_capitoli_categorie("Subcapitolo", "Nuovo Subcapitolo")
        # se disabilitata attivare la visualizzazione dei capitoli vuoti
        if not settings["show_empty_chapter"]:
            self.popup_capitoli.items[4].set_active(True)
        else:
            self.Aggiorna_treeview_capitoli(self)
        self.Aggiorna_treeview_capitoli(self)
        #aggiorna i combobox dei capitoli degli articoli
        self.Aggiorna_liststore_capitoli(self)

    def cell_nome_capitolo_edited( self, cell, path, new_text, col_nr):
        """Called when a text cell is edited.  It puts the new text in the model so that it is displayed properly."""
        model = self.treeview_capitoli.get_model()
        model[path][col_nr] = new_text
        sup, cap, sub = self.capitoli
        if cap == None and sub == None:
            result = self.preventivo.update_capitoli_categorie (sup, "Supercapitolo", new_text)
        elif sub == None:
            result = self.preventivo.update_capitoli_categorie (cap, "Capitolo", new_text)
        elif sup is not None and cap is not None and sub is not None:
            result = self.preventivo.update_capitoli_categorie (sub, "Subcapitolo", new_text)
        self.Aggiorna_liststore_capitoli(self)
        self.Seleziona_articolo_lista_lavorazioni(self)
        return

    def cancella_capitolo(self, widget, data=None):
        """Cancella un capitolo esistente"""
        if self.capitoli == None:
            if self.message(self, "avviso", _("Impossibile eliminare il capitolo, nessuna capitolo è selezionata")):
                return False
        if self.capitoli[1] == None:
            self.preventivo.delete_capitoli_categorie("Supercapitolo", self.capitoli[0], column = None)
        elif self.capitoli[2] == None:
            self.preventivo.delete_capitoli_categorie("Capitolo", self.capitoli[1], column = None)
        else:
            self.preventivo.delete_capitoli_categorie("Subcapitolo", self.capitoli[2], column = None)
        self.Aggiorna_treeview_capitoli(self)


    def Visualizza_capitoli_vuoti(self, widget, data=None):
        """Funzione per visualizzare nella treeview anche i capitoli vuoti"""
        if self.popup_capitoli.items[4].get_active():
            settings["show_empty_chapter"] = True
        elif not self.popup_capitoli.items[4].get_active():
            settings["show_empty_chapter"] = False
        self.Aggiorna_treeview_capitoli(self)

################################################################################
########## TREEVIEW - COMPUTO ##################################################
################################################################################

    def on_treeview_computo_button_press_event(self, widget, event, data=None):
        """Funzione per richiamare un menu popup alla pressione di un tasto del mouse"""
        x = int(event.x)
        y = int(event.y)
        time = event.time
        pthinfo = widget.get_path_at_pos(x, y)
        selection = widget.get_selection()
        # richiama il menu popup con il pulsante destro
        if event.button == 3:
            if selection.count_selected_rows() > 1 and (pthinfo is not None):
                widget.grab_focus()
                # mostra i pulsanti 'copia', 'incolla', 'cancella' articolo e il loro separatore
                for n in [0,1,2,3,5,6,7,8]:
                    self.popup_computo.items[n].show()
            elif pthinfo is not None:
                path, col, cellx, celly = pthinfo
                widget.grab_focus()
                widget.set_cursor(path, col, 0)
                # mostra i pulsanti 'copia', 'incolla', 'cancella' articolo e il loro separatore
                for n in [0,1,2,3,5,6,7,8]:
                    self.popup_computo.items[n].show()
            else:
                # nasconde i pulsanti 'copia', 'incolla', 'cancella' articolo e il loro separatore
                if self.articolo_computo is None and self.lista_articoli_computo is None:
                    # nasconde pulsante incolla
                    self.popup_computo.items[1].hide()
                for n in [0,2,3,5,6,7,8]:
                    self.popup_computo.items[n].hide()
            # Fa apparire il menu interattivo popup da utilizzare contestualmente
            self.popup_computo.popup(None, None, None, None, event.button, time)
            return True
        else:
            if pthinfo is None:
                self.prymary_key = None
                self.articolo_computo = None
                self.lista_articoli_computo = None
                selection.unselect_all()
            return False

    def row_selected_computo(self, widget, data=None):
        """Funzione chiamata quando la riga viene selezionata, mette in memoria l'articolo"""
        selection = widget.get_selection()
        if selection is None: return
        model, paths = selection.get_selected_rows()
        if selection.count_selected_rows() == 1:
            self.lista_articoli_computo = None
            # workaround x BUG: ValueError: invalid tree path
            # quando si seleziona una path nel momento in cui la treeview contiene 1 sola riga
            if paths == [Gtk.TreePath()]: iter = model.get_iter_first()
            else: iter = model.get_iter(paths[0])
            # recupera la chiave primaria dell'articolo
            if iter is None: #FIX TypeError: iter must be a GtkTreeIter
                self.prymary_key = None
            else:
                self.prymary_key = int(model.get_value(iter, 0))
            self.articolo_computo = self.preventivo.copia_articolo_computo (
                                  self.prymary_key, copia_nomi_categorie= True)
            # aggiorna la maschera dell'articolo e del listino
            self.Aggiorna_maschera_articolo(self)
            self.Aggiorna_treeview_listino(self)
        elif selection.count_selected_rows() > 1:
            self.prymary_key = None
            self.articolo_computo = None
            self.lista_articoli_computo = list()
            for n in range(len(paths)):
                iter = model.get_iter(paths[n])
                prymary_key = int(model.get_value(iter, 0))
                self.lista_articoli_computo.append(self.preventivo.copia_articolo_computo(
                                                   prymary_key, copia_nomi_categorie= True))
        # Imposta l'etichetta Tariffa
        if self.articolo_computo is not None:
            self.entry_tariffa_computo.set_text(self.articolo_computo.tariffa)
            data = [self.articolo_computo]
        else: 
            self.entry_tariffa_computo.set_text('')
            data = self.lista_articoli_computo
        # Copia gli articoli nella clipboard 
        if data is not None:
            self.clipboard.set_data(self.ARTICOLI_COMPUTO, data)
            #self.clipboard.set_with_data(self.clipboard.targets,
            #                             self.clipboard_get_art,
            #                             self.clipboard_clear_art, data)            

    def nascondi_colonna_computo_desc_voce (self, widget, data=None):
        """nasconde le colonne della treeview computo"""
        checkmenuitem = self.popup_computo.items[11]
        self.__inset_remove_computo_column(checkmenuitem, "show_column_desc_voce", 3)
    def nascondi_colonna_computo_materiali (self, widget, data=None):
        """nasconde le colonne della treeview computo"""
        checkmenuitem = self.popup_computo.items[12]
        self.__inset_remove_computo_column(checkmenuitem, "show_column_materiali", 6)
    def nascondi_colonna_computo_ricarico (self, widget, data=None):
        """nasconde le colonne della treeview computo"""
        checkmenuitem = self.popup_computo.items[13]
        self.__inset_remove_computo_column(checkmenuitem, "show_column_ricarico", 7)
    def nascondi_colonna_computo_tempo_inst (self, widget, data=None):
        """nasconde le colonne della treeview computo"""
        checkmenuitem = self.popup_computo.items[14]
        self.__inset_remove_computo_column(checkmenuitem, "show_column_tempo_inst", 8)
    def nascondi_colonna_computo_sicurezza (self, widget, data=None):
        """nasconde le colonne della treeview computo"""
        checkmenuitem = self.popup_computo.items[15]
        self.__inset_remove_computo_column(checkmenuitem, "show_column_sicurezza", 9)
    def nascondi_colonna_computo_tipo_app (self, widget, data=None):
        """nasconde le colonne della treeview computo"""
        checkmenuitem = self.popup_computo.items[16]
        self.__inset_remove_computo_column(checkmenuitem, "show_column_contract", 13)
    def nascondi_colonna_computo_cat_lav (self, widget, data=None):
        """nasconde le colonne della treeview computo"""
        checkmenuitem = self.popup_computo.items[17]
        self.__inset_remove_computo_column(checkmenuitem, "show_column_contract", 14)

    def __inset_remove_computo_column (self, checkmenuitem, setting_name, col_nr):
        # ottiene e modifica le impostazioni del programma
        if checkmenuitem.get_active():
            settings[setting_name] = False
        elif not checkmenuitem.get_active():
            settings[setting_name] = True
        # imposta la visibilità delle colonne
        if not settings[setting_name]:
            self.treeview_computo.columns[col_nr].set_visible(False)
        else:
            self.treeview_computo.columns[col_nr].set_visible(True)
        return True

    def cell_computo_edited(self, cell, path, new_text, col_nr):
        """Called when a text cell is edited.  It puts the new text in the model so that it is displayed properly."""
        if self.articolo_computo is None:
            return False
        model = self.treeview_computo.get_model()
        if col_nr == 0:
            new_id = self.__verifica_intero_immesso(new_text)
            model[path][col_nr] = new_id
        elif col_nr in [5, 6, 7, 8, 9, 10]:
            new_text = self.__verifica_testo_immesso(new_text)
            model[path][col_nr] = str(new_text)
        elif col_nr not in [11]:
            model[path][col_nr] = new_text
        tariffa = self.articolo_computo.tariffa
        if col_nr == 1:
            self.articolo_computo.tariffa = new_text
        elif col_nr == 2:
            self.articolo_computo.descrizione_codice = new_text
        elif col_nr == 3:
            self.articolo_computo.descrizione_voce = new_text
        elif col_nr == 4:
            self.articolo_computo.unita_misura = new_text
        elif col_nr == 5:
            self.articolo_computo.quantita = new_text
        elif col_nr == 6:
            self.articolo_computo.costo_materiali = new_text
        elif col_nr == 7:
            self.articolo_computo.ricarico = new_text
        elif col_nr == 8:
            self.articolo_computo.tempo_inst = new_text
        elif col_nr == 9:
            self.articolo_computo.sicurezza = new_text
        elif col_nr == 10:
            self.articolo_computo.set_unit_price(float(new_text))
        elif col_nr == 11:
            self.articolo_computo.prezzo_totale = new_text
        elif col_nr == 12:
            self.articolo_computo.note = new_text
        elif col_nr == 13:
            # se il testo è = alla costante per cat. nulla new_tex è una str vuota
            if new_text == self.CNR_TYPE_NULL_STR: new_text = str()
            # cambio della tipologia di appalto
            self.articolo_computo.tipo_lavori = new_text
        elif col_nr == 14:
            # se il testo è = alla costante per cat. nulla new_tex è una str vuota
            if new_text == self.CNR_TYPE_NULL_STR: new_text = str()
            # cambio della categoria di lavorazione (x appalto)
            self.articolo_computo.cat_appalto = new_text
        # effettua l'aggiornamento/modifica dell'articolo
        if col_nr in [1, 2, 3, 4, 6, 7, 8, 9, 10]:
            result = self.preventivo.update_articolo_epu (tariffa, self.articolo_computo)
        # se è stata moificata la primary_key dopo l'aggiornamento cambia anche la variabile
        elif col_nr == 0:
            result = self.preventivo.update_articolo_computo (self.prymary_key, 
                                          self.articolo_computo, new_id= new_id)
            self.prymary_key = new_id
        else:
            result = self.preventivo.update_articolo_computo (self.prymary_key, self.articolo_computo)
        # aggiornamento della UI
        if col_nr == 0:
            # Aggiorna tutta l'interfaccia grafica del tab 'Computo'
            self.Aggiorna_finestra_dialogo(self)
            self.Aggiorna_maschera_dati_generali(self)
            return True
        else:
            # aggiorna il rigo della treeview
            iter = model.get_iter(path)
            art = self.preventivo.computo_rows_list (column='id', 
                                  search_key=self.prymary_key, 
                                  convert_to_string=True)
            self.liststore_computo.insert_after(iter, art[0])
            self.liststore_computo.remove(iter)
            # seleziona il nuovo rigo (dopo la rimozione l'iter è il successivo, ovvero il nuovo rigo inserito)
            selection = self.treeview_computo.get_selection()
            selection.select_iter(iter)
            # aggiorna la maschera dell'articolo e del listino
            self.Aggiorna_maschera_articolo(self)
            self.Aggiorna_treeview_listino(self)
            self.Aggiorna_BarraStato_default()
            return True

################################################################################
########## MASCHERA DI MODIFICA ARTICOLO #######################################
################################################################################

    def modifica_articolo_epu (self, widget, data=None):
        """Modifica l'articolo di computo correntemente in memoria"""
        # verifica in quale pagina si trova il notebook (0= dati generali, 1= computo, 2= epu)
        notebook_page = self.notebook.get_current_page()
        if self.articolo_computo is None and notebook_page == 1:
            return False
        elif self.articolo_epu is None and notebook_page == 2:
            return False
        elif notebook_page == 1:
            articolo = self.articolo_computo
            tariffa = self.articolo_computo.tariffa
        elif notebook_page == 2:
            articolo = self.articolo_epu
            tariffa = self.articolo_epu.tariffa
        else:
            self.message(self, "errore", "Nessun articolo selezionato da modificare!")
            return False
        articolo.tariffa = self.tariffa.get_text()
        articolo.descrizione_codice = self.descrizione_codice.get_text()
        articolo.descrizione_voce = self.descrizione_voce.get_text()
        start = self.descrizione_estesa.get_start_iter()
        end = self.descrizione_estesa.get_end_iter()
        articolo.descrizione_estesa = self.descrizione_estesa.get_text(start, end, False)
        # ricarico
        articolo.ricarico = self.ricarico.get_value()
        # sicurezza (percentuale)
        articolo.sicurezza = self.sicurezza.get_value()
        # modifica il tempo di installazione e costo_materiali o il prezzo unitario
        if not self.prezzo_unitario.get_editable():
            # tempo di installazione
            articolo.tempo_inst = self.tempo_inst.get_value_as_int()
            # costo materiali
            articolo.costo_materiali = self.__verifica_dato_immesso(self, self.costo_materiali)
        elif self.prezzo_unitario.get_editable():
            # prezzo unitario
            prezzo_unitario = self.__verifica_dato_immesso(self, self.prezzo_unitario)
            # ricalcola del costo e tempo dell'articolo in base al totale
            articolo.set_unit_price(prezzo_unitario)
        # unita misura
        articolo.unita_misura = self.unita_misura.get_child().get_text()
        if articolo.unita_misura not in self.liststore_um.elenco_um:
            self.liststore_um.elenco_um.append(articolo.unita_misura)
            self.liststore_um.append((articolo.unita_misura,))
        # capitoli epu
        if (self.box_supercapitolo.get_sensitive() and 
            self.box_capitolo.get_sensitive() and
            self.box_subcapitolo.get_sensitive()):
            sup = self.__index_combobox_reverse(self, 
                    self.box_supercapitolo)[self.box_supercapitolo.get_active()]
            cap = self.__index_combobox_reverse(self, 
                    self.box_capitolo)[self.box_capitolo.get_active()]
            sub = self.__index_combobox_reverse(self, 
                    self.box_subcapitolo)[self.box_subcapitolo.get_active()]
            if sup != -1 and cap != -1 and sub != -1:
                articolo.supercapitolo = sup
                articolo.capitolo = cap
                articolo.subcapitolo = sub
        # fasce di costo aggiuntive (1-4)
        articolo.costo_mat1 = self.__verifica_dato_immesso(self, self.costo_mat_1)
        articolo.costo_mat2 = self.__verifica_dato_immesso(self, self.costo_mat_2)
        articolo.costo_mat3 = self.__verifica_dato_immesso(self, self.costo_mat_3)
        articolo.costo_mat4 = self.__verifica_dato_immesso(self, self.costo_mat_4)
        # utilizza la funzione di aggiornamento del database Epu
        result = self.preventivo.update_articolo_epu (tariffa, articolo)
        return True

    def modifica_articolo_computo (self, widget, data=None):
        """Aggiorna i valori di computo dell'articolo corrente"""
        if self.articolo_computo is None:
            return False
        # quantita
        self.articolo_computo.quantita = self.__verifica_dato_immesso(self, self.quantita)
        # categorie Computo
        sup = self.__index_combobox_reverse(self, self.box_supercategoria)[self.box_supercategoria.get_active()]
        cat = self.__index_combobox_reverse(self, self.box_categoria)[self.box_categoria.get_active()]
        sub = self.__index_combobox_reverse(self, self.box_subcategoria)[self.box_subcategoria.get_active()]
        if sup != -1 and cat != -1 and sub != -1:
            self.articolo_computo.supercategoria = sup
            self.articolo_computo.categoria = cat
            self.articolo_computo.subcategoria = sub
        # utilizza la funzione di aggiornamento del database Computo
        result = self.preventivo.update_articolo_computo (self.prymary_key, self.articolo_computo)
        return True

    def pulsante_aggiorna_articolo (self, widget, data=None):
        """Aggiorna l'articolo modificato nell'apposita maschera"""
        if self.notebook.get_current_page() == 1:
            self.modifica_articolo_computo(self)
        self.modifica_articolo_epu(self)
        # aggiorna l'interfaccia grafica (solo le 3 treeview)
        self.Aggiorna_maschera_articolo(self)
        self.Aggiorna_finestra_dialogo(self)
        
    def Attiva_pulsante_aggiornamento(self, widget, *args):
        """Callback dei widget della aggiorna articolo - tab 'Computo',
           alla digitazione di dati (segnale "focus") il widget attiva il pulsante
           aggiorna articolo"""
        pulsante_aggiornamento = self.builder.get_object("button_aggiorna_articolo")
        pulsante_aggiornamento.set_sensitive(True)

    def Disattiva_pulsante_aggiornamento(self, widget, *args):
        """Callback dei widget della aggiorna articolo - tab 'Computo',
           alla digitazione di dati (segnale "focus") il widget attiva il pulsante
           aggiorna articolo"""
        pulsante_aggiornamento = self.builder.get_object("button_aggiorna_articolo")
        pulsante_aggiornamento.set_sensitive(False)
        # ripristina lo stato della cella prezzo_unitario
        if self.prezzo_unitario.get_editable():
            self.prezzo_unitario.set_property('editable', False)
            self.prezzo_unitario.override_background_color(Gtk.StateType.NORMAL,
                self.colour_lred)
            self.prezzo_unitario.set_property('primary-icon-stock', Gtk.STOCK_STOP)
            self.prezzo_unitario.set_sensitive(False)
            # rende editabili i pulsanti tempo e costo materiali
            self.costo_materiali.set_property('editable', True)
            self.costo_materiali.override_background_color(Gtk.StateType.NORMAL,
                self.colour_gray)
            self.costo_materiali.set_property('primary-icon-stock', Gtk.STOCK_EDIT)
            self.costo_materiali.set_sensitive(True)
            self.tempo_inst.set_sensitive(True)

    def callback_icon_press_entry(self, widget, icon_pos, event, data=None):
        """callback utilizzata quando si preme l'ICONA di una Gtk.Entry"""
        editable = widget.get_property('editable')
        if editable:
            colour_t = self.colour_gray
            colour_f = self.colour_lred
            icon_t = Gtk.STOCK_EDIT
            icon_f = Gtk.STOCK_STOP
        else:
            colour_t = self.colour_lred
            colour_f = self.colour_gray
            icon_f = Gtk.STOCK_EDIT
            icon_t = Gtk.STOCK_STOP
        # modifica i widget
        if icon_pos.real == 0:
            widget.set_property('editable', not editable)
            widget.override_background_color(Gtk.StateType.NORMAL, colour_f)
            widget.set_property('primary-icon-stock', icon_f)
            widget.set_sensitive(not editable)
            if widget is self.prezzo_unitario:
                wid = self.costo_materiali
                self.tempo_inst.set_sensitive(False)
            elif widget is self.costo_materiali:
                wid = self.prezzo_unitario
                self.tempo_inst.set_sensitive(True)
            else:
                wid = widget
            # rende non editabili i pulsanti 'opposti'
            wid.set_property('editable', editable)
            wid.override_background_color(Gtk.StateType.NORMAL, colour_t)
            wid.set_property('primary-icon-stock', icon_t)
            wid.set_sensitive(editable)
        return
            

    def callback_activate_entry(self, widget, data=None):
        """callback utilizzata quando si preme ENTER in una Gtk.Entry"""
        pass

################################################################################
########## TREEVIEW - LISTINO ##################################################
################################################################################

    def Aggiorna_treeview_listino(self, widget, data=None):
        """Aggiorna la visualizzazione della treeview listino"""
        # Pagine del notebook: 0= dati generali, 1= Computo, 2= Epu
        notebook_page = self.notebook.get_current_page()
        self.liststore_listino.clear()
        if self.articolo_computo is not None and notebook_page == 1:
            for listino in self.preventivo.listino_rows_list (
                                           tariffa = self.articolo_computo.tariffa,
                                           convert_to_string=True):
                self.liststore_listino.append(list(listino))
        elif self.articolo_epu is not None and notebook_page == 2:
            for listino in self.preventivo.listino_rows_list (
                                           tariffa = self.articolo_epu.tariffa,
                                           convert_to_string=True):
                self.liststore_listino.append(list(listino))

    def on_treeview_listino_button_press_event(self, widget, event, data=None):
        """Funzione per richiamare un menu popup alla pressione di un tasto del mouse"""
        # richiama il menu popup con il pulsante destro
        if event.button == 3:
            x = int(event.x)
            y = int(event.y)
            time = event.time
            pthinfo = self.treeview_listino.get_path_at_pos(x, y)
            selection = self.treeview_listino.get_selection()
            if selection.count_selected_rows() > 1 and (pthinfo is not None):
                self.treeview_listino.grab_focus()
                self.popup_listino.items[1].show()
                self.popup_listino.items[3].show()
            elif pthinfo is not None:
                path, col, cellx, celly = pthinfo
                self.treeview_listino.grab_focus()
                self.treeview_listino.set_cursor(path, col, 0)
                self.popup_listino.items[1].show()
                self.popup_listino.items[3].show()
            else:
                # nasconde le opzioni, 'copia' e 'cancella' articolo
                self.popup_listino.items[1].hide()
                self.popup_listino.items[3].hide()
            # Crea e predispone un menu interattivo popup da utilizzare contestualmente
            self.popup_listino.popup(None, None, None, None, event.button, time)
            return True
        else:
            return False

    def row_selected_listino(self, widget, data=None):
        """Funzione chiamata quando una o più righe di listino riga vengono selezionate"""
        selection = widget.get_selection()
        if selection is None: return
        model, paths = selection.get_selected_rows()
        if selection.count_selected_rows() == 1:
            self.lista_articoli_listino = None
            iter = model.get_iter(paths[0])
            self.listino_key = int(model.get_value(iter, 0))
            self.articolo_listino = self.preventivo.copia_articolo_listino (int(self.listino_key))
            # se la finestra listino METEL è aperta azzera il listino copiato
            self.articolo_metel = None
            if self.metel is not None:
                self.metel.rigo_copiato = None
        elif selection.count_selected_rows() > 1:
            self.listino_key = None
            self.articolo_listino = None
            self.lista_articoli_listino = list()
            for n in range(len(paths)):
                iter = model.get_iter(paths[n])
                listino_key = int(model.get_value(iter, 0))
                self.lista_articoli_listino.append(self.preventivo.copia_articolo_listino (int(listino_key)))

    def __select_tariffa_for_listino(self):
        """Determina la tariffa su cui si inserirà/cancellerà, ecc. l'articolo di listino 
           a seconda se stiamo lavorando sul Tab 'Computo' o 'Epu"""
        # Pagine del notebook: 0= dati generali, 1= Computo, 2= Epu
        notebook_page = self.notebook.get_current_page()
        if  notebook_page == 1:
            if self.articolo_computo is not None:
                tariffa = self.articolo_computo.tariffa
            else: tariffa = None
        elif  notebook_page == 2:
            if self.articolo_epu is not None:
                tariffa = self.articolo_epu.tariffa
            else: tariffa = None
        else: tariffa = None
        return tariffa

    def nuovo_articolo_listino(self, widget, data=None):
        """inserisce un nuovo articolo in computo."""
        # Pagine del notebook: 0= dati generali, 1= Computo, 2= Epu
        notebook_page = self.notebook.get_current_page()
        if self.articolo_computo is None and notebook_page == 1:
            self.message(self, "errore", _("Nessun articolo di computo selezionato. Seleziona un articolo di computo in cui inserire il nuovo articolo di listino"))
            return False
        elif self.articolo_epu is None and notebook_page == 2:
            self.message(self, "errore", _("Nessun articolo di elenco prezzi selezionato. Seleziona un articolo in cui inserire l'articolo di listino"))
            return False
        # determina la tariffa in cui inserire l'articolo a seconda se stiamo lavorando su 'Computo' o 'Epu'
        tariffa = self.__select_tariffa_for_listino()
        if tariffa is None:
            self.message(self, "errore", _("Nessun articolo selezionato. Seleziona un articolo prima di inserire un articolo di listino"))
            return False
        # inseriamo l'articolo
        if data is None: data = _("Nuova voce di listino")
        self.articolo_listino = self.preventivo.nuovo_articolo_listino (
                                      tariffa, "LIS000001", data)
        self.Aggiorna_maschera_articolo(self)
        self.Aggiorna_finestra_dialogo(self)
        return True

    def cancella_articolo_listino(self, widget, data=None):
        """cancella uno o più articoli di listino."""
        if self.articolo_listino is not None:
            if self.message(self, "domanda", _("Vuoi cancellare l'articolo di listino N.%d" % self.listino_key)):
                self.preventivo.delete_articoli_listino (self.articolo_listino)
        elif self.lista_articoli_listino is not None:
            if self.message(self, "domanda", _("Vuoi cancellare gli articoli di listino selezionati?")):
                self.preventivo.delete_articoli_listino (*self.lista_articoli_listino)
        else:
            if self.message(self, "avviso", _("Nessun articolo di listino selezionato")):
                return False
        self.Aggiorna_maschera_articolo(self)
        self.Aggiorna_finestra_dialogo(self)
        return True

    def copia_articolo_listino(self, widget, data=None):
        """copia l'articolo corrente del computo."""
        self.row_selected_listino(self.treeview_listino)

    def incolla_articolo_listino(self, widget, data=None):
        """incolla articolo in computo nella posizione (categorie) correnti."""
        # Pagine del notebook: 0 = dati generali, 1 = Computo, 2 = Epu
        notebook_page = self.notebook.get_current_page()
        if self.articolo_computo is None and notebook_page == 1:
            self.message(self, "errore", _("Nessun articolo di computo selezionato. Seleziona un articolo in cui incollare il nuovo articolo di listino"))
            return False
        elif self.articolo_epu is None and notebook_page == 2:
            self.message(self, "errore", _("Nessun articolo di elenco prezzi selezionato. Seleziona un articolo in cui incollare l'articolo di listino"))
            return False
        # determina la tariffa in cui inserire l'articolo a seconda se stiamo lavorando su 'Computo' o 'Epu'
        tariffa = self.__select_tariffa_for_listino()
        if tariffa is None:
            self.message(self, "errore", _("Nessun articolo selezionato. Seleziona un articolo prima di inserire un articolo di listino"))
            return False
        # se la finestra listino METEL è aperta 'recupera' il listino copiato
        if self.metel is not None:
            self.articolo_metel = self.metel.rigo_copiato
        if self.articolo_metel is not None: 
            result = self.preventivo.nuovo_articolo_listino (tariffa, self.articolo_metel[0], 
                                self.articolo_metel[1], unita_misura = self.articolo_metel[2], 
                                prezzo_unitario = self.articolo_metel[3], sconto = self.articolo_metel[4])
        elif self.articolo_listino is not None:
            result = self.preventivo.insert_articoli_listino(tariffa, self.articolo_listino)
        elif self.lista_articoli_listino is not None:
            result = self.preventivo.insert_articoli_listino(tariffa, *self.lista_articoli_listino)
        else:
            self.message(self, "errore", _("Nessun articolo di listino copiato, copia o seleziona un articolo"))
            return False
        # se la libreria ha riscontrato che un articolo è duplicato e non è stato inserito avvisa l'utente
        if not result:
            self.message(self, "errore", _("Una voce di listino è già esistente per l'articolo %s, e non è stato aggiunto" % tariffa))
        self.Aggiorna_finestra_dialogo(self)
        return True

    def calcola_prezzo_articoli_listino(self, widget, data=None):
        """calcola il prezzo di tutti gli articoli di listino e trasverisce il prezzo all'epu."""
        # determina la tariffa in cui inserire l'articolo a seconda se stiamo lavorando su 'Computo' o 'Epu'
        tariffa = self.__select_tariffa_for_listino()
        # calcola il prezzo e aggiorna l'epu
        if tariffa is not None:
            self.preventivo.ricalcolo_prezzo_tariffa(tariffa)
            # aggiorna l'interfaccia grafica
            self.Aggiorna_maschera_articolo(self)
            self.Aggiorna_finestra_dialogo(self)
        else:
            self.message(self, "errore", _("Nessun articolo selezionato. Seleziona un articolo prima di effettuare il ricalcolo"))
            return False
        return True

    def cell_listino_edited( self, cell, path, new_text, col_nr):
        """Called when a text cell is edited.  It puts the new text in the model so that it is displayed properly."""
        model = self.treeview_listino.get_model()
        aggiorna_cap=True
        if col_nr in [5, 6, 7, 8, 9]:
            new_text = self.__verifica_testo_immesso(new_text)
        if col_nr not in [0, 1, 9]:
            model[path][col_nr] = new_text
        # aggiorna i valori dell'articolo di listino
        if col_nr == 2:
            self.articolo_listino.codice = new_text
            aggiorna_cap=False
        elif col_nr == 3:
            self.articolo_listino.descrizione_codice = new_text
            aggiorna_cap=False
        elif col_nr == 4:
            self.articolo_listino.unita_misura = new_text
            aggiorna_cap=False
        elif col_nr == 5:
            self.articolo_listino.quantita = new_text
        elif col_nr == 6:
            self.articolo_listino.prezzo_unitario = new_text
        elif col_nr == 7:
            self.articolo_listino.sconto = new_text
        elif col_nr == 8:
            self.articolo_listino.accessori = new_text
        result = self.preventivo.update_articolo_listino (self.listino_key, self.articolo_listino)
        # aggiorna l'interfaccia grafica
        self.Aggiorna_maschera_articolo(self)
        self.Aggiorna_finestra_dialogo(self, aggiorna_cap=aggiorna_cap)
        return

################################################################################
########## TREEVIEW - LISTA LAVORAZIONI ########################################
################################################################################

    def Seleziona_articolo_lista_lavorazioni(self, widget, data=None):
        """posiziona il selettore della lista lavorazioni sull'articolo di elenco prezzi in memoria"""
        if self.articolo_epu is not None:
            selection = self.treeview_lavorazioni.get_selection()
            model = self.treeview_lavorazioni.get_model()
            iter = model.get_iter_first()
            if iter is None:
               return None
            tariffa = model.get_value(iter, 0)
            while tariffa != self.articolo_epu.tariffa:
                iter = model.iter_next(iter)
                if iter is None:
                    iter = model.get_iter_first()
                    break
                tariffa = model.get_value(iter, 0)
            # seleziona il rigo corrispondente
            selection.select_iter(iter)
            path = model.get_path(iter)
            # effettua lo scrolling della treeview fino all'articolo corrente
            self.treeview_lavorazioni.scroll_to_cell(path, column=None, use_align=False, row_align=0.0, col_align=0.0)
            return path
        else:
            return None

    def on_treeview_lavorazioni_button_press_event(self, widget, event, data=None):
        """Funzione per richiamare un menu popup alla pressione di un tasto del mouse"""
        x = int(event.x)
        y = int(event.y)
        time = event.time
        pthinfo = self.treeview_lavorazioni.get_path_at_pos(x, y)
        selection = self.treeview_lavorazioni.get_selection()
        if event.button == 3:
            if selection.count_selected_rows() > 1 and (pthinfo is not None):
                self.treeview_lavorazioni.grab_focus()
                # mostra i pulsanti 'copia', 'incolla', 'cancella', 'duplica' e i loro separatori
                for n in [0,1,3,4,5,6]:
                    self.popup_lavorazioni.items[n].show()
            elif pthinfo is not None:
                path, col, cellx, celly = pthinfo
                self.treeview_lavorazioni.grab_focus()
                self.treeview_lavorazioni.set_cursor(path, col, 0)
                # mostra i pulsanti 'copia', 'incolla', 'cancella', 'duplica' e i loro separatori
                for n in [0,1,3,4,5,6]:
                    self.popup_lavorazioni.items[n].show()
            else:
                # nasconde i pulsanti 'copia', 'incolla', 'cancella', 'duplica' e i loro separatori
                for n in [0,1,3,4,5,6]:
                    self.popup_lavorazioni.items[n].hide()
            # Crea e predispone un menu interattivo popup da utilizzare contestualmente
            self.popup_lavorazioni.popup(None, None, None, None, event.button, time)
            return True
        else:
            if pthinfo is None:
                 # azzera gli articoli in memoria e deseleziona tutto
                 self.prymary_key = None
                 self.articolo_epu = None
                 self.lista_articoli_epu = None
                 selection.unselect_all()
                 # Imposta l'etichetta Tariffa vuota
                 self.entry_tariffa_lavorazioni.set_text('')
            return False

    def row_selected_lista_lavorazioni(self, widget, data=None):
        """Funzione chiamata quando la riga o le righe della lista lavorazioni vengono selezionate"""
        self.prymary_key = None
        selection = self.treeview_lavorazioni.get_selection()
        if selection is None: return
        model, paths = selection.get_selected_rows()
        if selection.count_selected_rows() == 1:
            self.lista_articoli_epu = None
            iter = model.get_iter(paths[0])
            tariffa = model.get_value(iter, 0)
            self.articolo_epu = self.preventivo.copia_articolo_epu(tariffa, copia_nomi_capitoli= True)
            # aggiorna la maschera dell'articolo e del listino
            self.Aggiorna_maschera_articolo(self)
            self.Aggiorna_treeview_listino(self)
        elif selection.count_selected_rows() > 1:
            self.articolo_epu = None
            self.lista_articoli_epu = list()
            for n in range(len(paths)):
                iter = model.get_iter(paths[n])
                tariffa = model.get_value(iter, 0)
                self.lista_articoli_epu.append(self.preventivo.copia_articolo_epu(tariffa, copia_nomi_capitoli= True))
        if self.articolo_epu is not None:
            index1, index2, index3 = self.articolo_epu.supercapitolo, self.articolo_epu.capitolo, self.articolo_epu.subcapitolo
            # Imposta l'etichetta Tariffa
            self.entry_tariffa_lavorazioni.set_text(self.articolo_epu.tariffa)
            data = [self.articolo_epu]
        else: 
            index1, index2, index3 = -1, -1, -1
            # Imposta l'etichetta Tariffa vuota
            self.entry_tariffa_lavorazioni.set_text('')
            data = self.lista_articoli_epu
        # Imposta i combobox dei capitoli in base all'articolo selezionato
        if self.box_supercapitolo_lavorazioni.get_active() != index1:
            self.box_supercapitolo_lavorazioni.set_active(
                     self.__index_combobox(self, self.box_supercapitolo_lavorazioni)[index1])
        if self.box_capitolo_lavorazioni.get_active() != index2:
            self.box_capitolo_lavorazioni.set_active(
                     self.__index_combobox(self, self.box_capitolo_lavorazioni)[index2])
        if self.box_subcapitolo_lavorazioni.get_active() != index3:
            self.box_subcapitolo_lavorazioni.set_active(
                     self.__index_combobox(self, self.box_subcapitolo_lavorazioni)[index3])
        # Copia gli articoli nella clipboard
        if data is not None:
            self.clipboard.set_data(self.ARTICOLI_EPU, data)
            
            

    def copia_articolo_lista_lavorazioni(self, widget, data=None):
        """copia in memoria l'articolo corrente di elenco prezzi"""
        self.articolo_computo = None
        self.lista_articoli_computo = None
        selection = self.treeview_lavorazioni.get_selection()
        if selection is None: return
        model, paths = selection.get_selected_rows()
        if selection.count_selected_rows() == 1:
            self.lista_articoli_epu = None
            data = [self.articolo_epu]
        elif selection.count_selected_rows() > 1:
            self.prymary_key = None
            self.articolo_epu = None
            data = self.lista_articoli_epu
        else: return
        # save data in the clipboard
        self.clipboard.set_data(self.ARTICOLI_EPU, data)
        return data

    def incolla_articolo_lista_lavorazioni(self, widget, data=None):
        """inserisce nell'elenco prezzi una lista di articoli"""
        if self.articolo_epu is not None:
            self.preventivo.incolla_articolo_epu (self.capitoli[0], self.capitoli[1], 
                                                  self.capitoli[2], self.articolo_epu)
        elif self.lista_articoli_epu is not None:
            self.preventivo.incolla_articolo_epu (self.capitoli[0], self.capitoli[1], 
                                                  self.capitoli[2], *self.lista_articoli_epu)
        else:
            # se entrambe le liste sono 'None' allora incolla dalla clipboard
            self.lista_articoli_epu = self.clipboard.get_data(self.ARTICOLI_EPU)
            #self.clipboard.request_contents(self.target_epu, self.clipboard_cb_articoli, None)
            # se la lista non è nulla o vuota incolla gli articoli della clipboard in computo (ruotine inerimento articoli esterni)
            if self.lista_articoli_epu is not None and len(self.lista_articoli_epu) > 0:
                self.preventivo.insert_articoli_from_archivio_epu (True, *self.lista_articoli_epu)
            # in tutti gli altri casi dai un messaggio di errore
            else:
                if self.message(self, "avviso", _("Nessun articolo copiato, copia o seleziona uno o più articoli che vuoi incollare")):
                    return False
        # dopo aver incollato pulisce la memoria dagli art memorizzati
        self.__cancella_articoli_memoria()
        # Aggiorna l'interfaccia grafica
        self.Aggiorna_finestra_dialogo(self)

    def nuovo_articolo_lista_lavorazioni(self, widget, data=None):
        """inserisce un nuovo articolo nella lista lavorazioni"""
        index = settings["character_nr_for_descr_cod_from_descr_estesa"]
        articolo_name = settings["new_articolo_name"]
        if data is not None and type(data) == str:
            articolo_name = data[:index]
        self.articolo_epu = self.preventivo.nuovo_articolo_epu ("0001", articolo_name,
                                    descrizione_voce=None, descrizione_estesa=data, 
                                    supercapitolo = self.capitoli[0], 
                                    capitolo = self.capitoli[1], 
                                    subcapitolo = self.capitoli[2])
        self.Aggiorna_finestra_dialogo(self)

    def duplica_articolo_lista_lavorazioni(self, widget, data=None):
        """duplica l'articolo della lista lavorazioni selezionato"""
        if self.articolo_epu is not None:
            self.preventivo.insert_articoli_epu(True, self.articolo_epu)
        elif self.lista_articoli_epu is not None:
            self.preventivo.insert_articoli_epu(True, *self.lista_articoli_epu)
        else:
            if self.message(self, "avviso", _("Nessun articolo copiato. Copia o seleziona uno o più articoli che vuoi incollare")):
                return False
        self.Aggiorna_finestra_dialogo(self)

    def cancella_articolo_lista_lavorazioni(self, widget, data=None):
        """cancella l'articolo della lista lavorazioni selezionato"""
        if self.articolo_epu is not None:
            if self.message(self, "domanda", _("Vuoi cancellare dall'elenco prezzi e dal computo l'articolo con tariffa '%s'?") % self.articolo_epu.tariffa):
                self.preventivo.delete_articoli_epu(self.articolo_epu)
        elif self.lista_articoli_epu is not None:
            if self.message(self, "domanda", _("Vuoi cancellare dall'elenco prezzi e dal computo tutti gli articoli selezionati?")):
                self.preventivo.delete_articoli_epu(*self.lista_articoli_epu)
        else:
            if self.message(self, "avviso", _("Nessun articolo selezionato da cancellare")):
                return False
        self.Aggiorna_finestra_dialogo(self)

    def modifica_capitoli_lavorazioni (self, widget, data=None):
        """Modifica il capitolo dell'articolo di elenco prezzi attualmente selezionato"""
        # la 'chiave' la posizione atturale del box di scelta del capitolo, il 'valore' è il capitolo stesso
        sup = self.__index_combobox_reverse(self, self.box_supercapitolo_lavorazioni)[self.box_supercapitolo_lavorazioni.get_active()]
        cap = self.__index_combobox_reverse(self, self.box_capitolo_lavorazioni)[self.box_capitolo_lavorazioni.get_active()]
        sub = self.__index_combobox_reverse(self, self.box_subcapitolo_lavorazioni)[self.box_subcapitolo_lavorazioni.get_active()]
        if sup == -1 or cap == -1 or sub == -1:
            return False
        if self.articolo_epu is not None:
            # sostituisci i capitoli dell'articolo di epu
            tariffa = self.articolo_epu.tariffa
            if self.message(self, "domanda", "Vuoi spostare l'articolo con tariffa '%s' nel capitolo '%d,%d,%d'?" % (tariffa, sup, cap, sub)):
                self.articolo_epu.supercapitolo = sup
                self.articolo_epu.capitolo = cap
                self.articolo_epu.subcapitolo = sub
                # utilizza la funzione di aggiornamento del database Epu
                result = self.preventivo.update_articolo_epu (tariffa, self.articolo_epu)
                self.Aggiorna_finestra_dialogo(self)
                return True
            else:
                # aggiornamento annullato
                return False

    def nascondi_colonna_lavorazioni_desc_voce (self, widget, data=None):
        """nasconde le colonne della treeview lavorazioni"""
        self.__inset_remove_lavorazioni_column(8, "show_column_desc_voce_lavorazioni", 2)
    def nascondi_colonna_lavorazioni_desc_estesa (self, widget, data=None):
        """nasconde le colonne della treeview lavorazioni"""
        self.__inset_remove_lavorazioni_column(9, "show_column_desc_estesa_lavorazioni", 3)
    def nascondi_colonna_lavorazioni_materiali (self, widget, data=None):
        """nasconde le colonne della treeview lavorazioni"""
        self.__inset_remove_lavorazioni_column(10, "show_column_materiali_lavorazioni", 5)
    def nascondi_colonna_lavorazioni_ricarico (self, widget, data=None):
        """nasconde le colonne della treeview lavorazioni"""
        self.__inset_remove_lavorazioni_column(11, "show_column_ricarico_lavorazioni", 6)
    def nascondi_colonna_lavorazioni_tempo_inst (self, widget, data=None):
        """nasconde le colonne della treeview lavorazioni"""
        self.__inset_remove_lavorazioni_column(12, "show_column_tempo_inst_lavorazioni", 7)
    def nascondi_colonna_lavorazioni_sicurezza (self, widget, data=None):
        """nasconde le colonne della treeview lavorazioni"""
        self.__inset_remove_lavorazioni_column(13, "show_column_sicurezza_lavorazioni", 8)

    def __inset_remove_lavorazioni_column (self, menuitem_nr, setting_name, col_nr):
        if self.popup_lavorazioni.items[menuitem_nr].get_active():
            settings[setting_name] = False
        elif not self.popup_lavorazioni.items[menuitem_nr].get_active():
            settings[setting_name] = True
        # imposta la visibilità delle colonne
        if not settings[setting_name]:
            self.treeview_lavorazioni.columns[col_nr].set_visible(False)
        else:
            self.treeview_lavorazioni.columns[col_nr].set_visible(True)
        return True

    def cell_lista_lavorazioni_edited( self, cell, path, new_text, col_nr):
        """Called when a text cell is edited.  It puts the new text in the model so that it is displayed properly.""" 
        if self.articolo_epu == None:
            return False
        model = self.treeview_lavorazioni.get_model()
        if col_nr in [5, 6, 7, 8, 9, 10]:
            new_text = self.__verifica_testo_immesso(new_text)
        if col_nr not in [9, 11]:
            model[path][col_nr] = str(new_text)
        tariffa = self.articolo_epu.tariffa
        if col_nr == 0:
            self.articolo_epu.tariffa = new_text
        elif col_nr == 1:
            self.articolo_epu.descrizione_codice = new_text
        elif col_nr == 2:
            self.articolo_epu.descrizione_voce = new_text
        elif col_nr == 3:
            self.articolo_epu.descrizione_estesa = new_text
        elif col_nr == 4:
            self.articolo_epu.unita_misura = new_text
        elif col_nr == 5:
            self.articolo_epu.costo_materiali = new_text
        elif col_nr == 6:
            self.articolo_epu.ricarico = new_text
        elif col_nr == 7:
            self.articolo_epu.tempo_inst = new_text
        elif col_nr == 8:
            self.articolo_epu.sicurezza = new_text
        elif col_nr == 9:
            self.articolo_epu.quantita = new_text
        elif col_nr == 10:
            self.articolo_epu.set_unit_price(float(new_text))
        elif col_nr == 11:
            self.articolo_epu.prezzo_totale = new_text
        elif col_nr == 12:
            self.articolo_epu.note = new_text
        result = self.preventivo.update_articolo_epu (tariffa, self.articolo_epu)
        # aggiorna il rigo della treeview
        iter = model.get_iter(path)
        art = self.preventivo.lavorazioni_rows_list (column= 'Tariffa', 
                              search_key = self.articolo_epu.tariffa,
                              convert_to_string=True)
        self.liststore_lavorazioni.insert_after(iter, art[0])
        self.liststore_lavorazioni.remove(iter)
        # seleziona il nuovo rigo (dopo la rimozione l'iter è il successivo, ovvero il nuovo rigo inserito)
        selection = self.treeview_lavorazioni.get_selection()
        selection.select_iter(iter)
        # aggiorna la maschera dell'articolo e del listino
        self.Aggiorna_maschera_articolo(self)
        self.Aggiorna_treeview_listino(self)
        self.Aggiorna_BarraStato_default()
        return True

################################################################################
###  DATI GENERALI -  ##########################################################
################################################################################

    def crea_treeview_dati_generali (self, treeview_obj, tipo_colonna):
        """Funzione per creare la treeview dei capitoli e categorie nella tab 'Dati generali'"""
        if tipo_colonna == "Supercapitolo":
            treeview_obj.liststore = self.liststore_supercapitolo
        elif tipo_colonna == "Capitolo":
            treeview_obj.liststore = self.liststore_capitolo
        elif tipo_colonna == "Subcapitolo":
            treeview_obj.liststore = self.liststore_subcapitolo
        elif tipo_colonna == "Supercategoria":
            treeview_obj.liststore = self.liststore_supercategoria
        elif tipo_colonna == "Categoria":
            treeview_obj.liststore = self.liststore_categoria
        elif tipo_colonna == "Subcategoria":
            treeview_obj.liststore = self.liststore_subcategoria
        treeview_obj.set_model(treeview_obj.liststore)
        treeview_obj.columns = [None]*4
        treeview_obj.columns[0] = Gtk.TreeViewColumn('No.')
        treeview_obj.columns[1] = Gtk.TreeViewColumn('Nome')
        treeview_obj.columns[2] = Gtk.TreeViewColumn('Descrizione')
        treeview_obj.columns[3] = Gtk.TreeViewColumn('Aumento Prezzi')
        for n in range(4):
            # add columns to treeview
            treeview_obj.append_column(treeview_obj.columns[n])
            # create a CellRenderers to render the data
            treeview_obj.columns[n].cell = Gtk.CellRendererText()
            # add the cells to the columns
            treeview_obj.columns[n].pack_start(treeview_obj.columns[n].cell, True)
            # set the cell attributes to the appropriate liststore column
            treeview_obj.columns[n].add_attribute(treeview_obj.columns[n].cell, 'text', n)
            # make treeview searchable
            treeview_obj.set_search_column(n)
            # set only the cell as editable
            treeview_obj.columns[n].cell.set_property('editable', True)
            treeview_obj.columns[n].cell.connect('edited', self.cell_treeview_dg_edited, n, tipo_colonna)

    def cell_treeview_dg_edited (self, cell, path, new_text, col_nr, tipo_colonna):
        """Callback quando viene editata una cella della treeview dei dati generali"""
        if tipo_colonna == "Supercategoria":
            model = self.treeview_dg_supercategorie.get_model()
        elif tipo_colonna == "Categoria":
            model = self.treeview_dg_categorie.get_model()
        elif tipo_colonna == "Subcategoria":
            model = self.treeview_dg_subcategorie.get_model()
        elif tipo_colonna == "Supercapitolo":
            model = self.treeview_dg_supercapitoli.get_model()
        elif tipo_colonna == "Capitolo":
            model = self.treeview_dg_capitoli.get_model()
        elif tipo_colonna == "Subcapitolo":
            model = self.treeview_dg_subcapitoli.get_model()
        if col_nr == 3:
            new_text = str(self.__verifica_testo_immesso(new_text))
        elif col_nr == 0: # la colonna 0 è il 'prymary key' relativa al rigo di database, non modificare!
            return False
        model[path][col_nr] = new_text
        iter = model.get_iter(path)
        key = model.get_value(iter, 0) 
        nome = model.get_value(iter, 1) 
        desc = model.get_value(iter, 2) 
        aumento = model.get_value(iter, 3)
        # eseguo l'aggiornamento del database
        self.preventivo.update_capitoli_categorie (key, tipo_colonna, nome, 
            Descrizione=desc, Aumento_prezzi=aumento)
        # aggiorno l'interfaccia grafica
        self.Aggiorna_treeview_capitoli(self)
        self.Aggiorna_treeview_categorie(self)
        self.Aggiorna_maschera_dati_generali(self)
        return True

    def Attiva_pulsanti_dati_generali_anagrafica(self, widget, data=None):
        """Callback del widget textview della sezione dati generali - quando si inizia
           la digitazione di nuovo testo (segnale "event") il widget attiva i pulsanti
           applica e annulla per memorizzare il testo"""
        pulsante_annulla = self.builder.get_object("button18")
        pulsante_applica = self.builder.get_object("button19")
        pulsante_annulla.set_sensitive(True)
        pulsante_applica.set_sensitive(True)

    def Modifica_dati_generali_anagrafica(self, widget, data=None):
        """Salva i dati anagrafici nel 'tab' Dati Generali"""
        start = self.dg_nome_lavoro.get_start_iter()
        end = self.dg_nome_lavoro.get_end_iter()
        nome_lavoro = self.dg_nome_lavoro.get_text(start, end, False)
        start = self.dg_cliente.get_start_iter()
        end = self.dg_cliente.get_end_iter()
        cliente = self.dg_cliente.get_text(start, end, False)
        start = self.dg_indirizzo.get_start_iter()
        end = self.dg_indirizzo.get_end_iter()
        indirizzo = self.dg_indirizzo.get_text(start, end, False)
        start = self.dg_comune.get_start_iter()
        end = self.dg_comune.get_end_iter()
        comune = self.dg_comune.get_text(start, end, False)
        start = self.dg_provincia.get_start_iter()
        end = self.dg_provincia.get_end_iter()
        provincia = self.dg_provincia.get_text(start, end, False)
        start = self.dg_redattore.get_start_iter()
        end = self.dg_redattore.get_end_iter()
        redattore = self.dg_redattore.get_text(start, end, False)
        self.preventivo.update_dati_generali (nome=nome_lavoro, cliente=cliente, 
                                              ricarico=None, manodopera=None, 
                                              sicurezza=None, indirizzo=indirizzo, 
                                              comune=comune, provincia=provincia, 
                                              redattore=redattore, valuta=None)
        self.Aggiorna_maschera_dati_generali(self)

    def Modifica_dati_generali_ricarico(self, widget, data=None):
        """Salva il ricarico nel tab 'Dati Generali'"""
        ric = float(self.dg_ricarico.get_value())
        if self.message(self, "domanda", _("Vuoi modificare il ricarico generale di tutti gli articoli esistenti?")):
            self.preventivo.update_ricarico_epu (ric)

    def Modifica_dati_generali_manodopera(self, widget, data=None):
        """Salva il manodopera nel tab 'Dati Generali'"""
        if self.message(self, "domanda", _("Vuoi modificare il valore della manodopera di tutti gli articoli esistenti?")):
            mo = self.__verifica_dato_immesso (self, self.dg_manodopera)
            self.preventivo.update_dati_generali (manodopera=mo)

    def Modifica_dati_generali_sicurezza(self, widget, data=None):
        """Salva il ricarico nel tab 'Dati Generali'"""
        sicurezza = float(self.dg_sicurezza.get_value())
        if self.message(self, "domanda", _("Vuoi modificare l'incidenza della sicurezza per tutti gli articoli esistenti?")):
            self.preventivo.update_sicurezza_epu (sicurezza)

    def Modifica_dati_generali_valuta(self, widget, data=None):
        """Cambia la valuta nel tab 'Dati Generali'"""
        valuta = self.dg_valuta.get_text()
        if self.message(self, "domanda", _("Vuoi modificare la valuta corrente con la nuova valuta '%s'?" % valuta)):
            self.preventivo.update_dati_generali (valuta=valuta)

    def pulsante_aggiungi_Supercategoria(self, widget, data=None):
        """Inserisce una nuova Supercategoria"""
        self.preventivo.insert_capitoli_categorie("Supercategoria", "Nuova Supercategoria")
        self.Aggiorna_liststore_categorie(self)
        self.Aggiorna_treeview_categorie(self)
    def pulsante_aggiungi_Categoria(self, widget, data=None):
        """Inserisce una nuova categoria"""
        self.preventivo.insert_capitoli_categorie("Categoria", "Nuova Categoria")
        self.Aggiorna_liststore_categorie(self)
        self.Aggiorna_treeview_categorie(self)
    def pulsante_aggiungi_Subcategoria(self, widget, data=None):
        """Inserisce una nuova Subcategoria"""
        self.preventivo.insert_capitoli_categorie("Subcategoria", "Nuova Subcategoria")
        self.Aggiorna_liststore_categorie(self)
        self.Aggiorna_treeview_capitoli(self)
    def pulsante_aggiungi_Supercapitolo(self, widget, data=None):
        """Inserisce una nuova Supercapitolo"""
        self.preventivo.insert_capitoli_categorie("Supercapitolo", "Nuovo Supercapitolo")
        self.Aggiorna_liststore_capitoli(self)
        self.Aggiorna_treeview_capitoli(self)
    def pulsante_aggiungi_Capitolo(self, widget, data=None):
        """Inserisce una nuova capitolo"""
        self.preventivo.insert_capitoli_categorie("Capitolo", "Nuovo Capitolo")
        self.Aggiorna_liststore_capitoli(self)
        self.Aggiorna_treeview_capitoli(self)
    def pulsante_aggiungi_Subcapitolo(self, widget, data=None):
        """Inserisce una nuova Subcapitolo"""
        self.preventivo.insert_capitoli_categorie("Subcapitolo", "Nouvo Subcapitolo")
        self.Aggiorna_liststore_capitoli(self)
        self.Aggiorna_treeview_capitoli(self)

    def pulsante_rimuovi_Supercategoria(self, widget, data=None):
        """rimuovi una nuova Supercategoria"""
        if self.__subroutine_rimuovi_cap_cat(self.treeview_dg_supercategorie, "Supercategoria"):
            # aggiorna l'interfaccia grafica
            self.Aggiorna_liststore_categorie(self)
            self.Aggiorna_treeview_categorie(self)
    def pulsante_rimuovi_Categoria(self, widget, data=None):
        """rimuovi una categoria"""
        if self.__subroutine_rimuovi_cap_cat(self.treeview_dg_categorie, "Categoria"):
            # aggiorna l'interfaccia grafica
            self.Aggiorna_liststore_categorie(self)
            self.Aggiorna_treeview_categorie(self)
    def pulsante_rimuovi_Subcategoria(self, widget, data=None):
        """rimuovi una Subcategoria"""
        if self.__subroutine_rimuovi_cap_cat(self.treeview_dg_subcategorie, "Subcategoria"):
            # aggiorna l'interfaccia grafica
            self.Aggiorna_liststore_categorie(self)
            self.Aggiorna_treeview_categorie(self)
    def pulsante_rimuovi_Supercapitolo(self, widget, data=None):
        """rimuovi un Supercapitolo"""
        if self.__subroutine_rimuovi_cap_cat(self.treeview_dg_supercapitoli, "Supercapitolo"):
            # aggiorna l'interfaccia grafica
            self.Aggiorna_liststore_capitoli(self)
            self.Aggiorna_treeview_capitoli(self)
    def pulsante_rimuovi_Capitolo(self, widget, data=None):
        """rimuovi un Capitolo"""
        if self.__subroutine_rimuovi_cap_cat(self.treeview_dg_capitoli, "Capitolo"):
            # aggiorna l'interfaccia grafica
            self.Aggiorna_liststore_capitoli(self)
            self.Aggiorna_treeview_capitoli(self)
    def pulsante_rimuovi_Subcapitolo(self, widget, data=None):
        """rimuovi un Subcapitolo"""
        if self.__subroutine_rimuovi_cap_cat(self.treeview_dg_subcapitoli, "Subcapitolo"):
            # aggiorna l'interfaccia grafica
            self.Aggiorna_liststore_capitoli(self)
            self.Aggiorna_treeview_capitoli(self)

    def __subroutine_rimuovi_cap_cat(self, treeviw_obj, tipo):
        """"""
        selection = treeviw_obj.get_selection()
        model, paths = selection.get_selected_rows()
        if len(paths) == 0:
            if self.message(self, "avviso", _("Nessun/a %s selezionata da rimuovere") % tipo):  
                return False
        iter = model.get_iter(paths[0])
        value = int(model.get_value(iter, 0))
        if self.message(self, "domanda", _("Vuoi cancellare il/la %s '%d'?") % (tipo, value)):
            # istruzione di cancellazione: 
            self.preventivo.delete_capitoli_categorie (tipo, value)
            return True
        else:
            return False

################################################################################
###  BARRA DI STATO DELL'APPLICAZIONE ##########################################
################################################################################
#Nota: di default la barra di stato deve visualizzare il totale generale del computo!

    def BarraStato(self, widget, testo):
        """Funzione per Aggiornare la Barra di Stato"""
        BarraStato = self.builder.get_object("statusbar1")
        contesto = BarraStato.push(0, testo)

    def Aggiorna_BarraStato_default(self):
        """Aggiornarmento di default della Barra di Stato"""
        # nella barra di stato visualizza il prezzo totale e la valuta
        self.BarraStato(self, "{0}: {1} {2}".format(Preventa.STR_TOT_G,
                        self.preventivo.calcolo_prezzi (self), 
                        self.preventivo.dati_generali_list()[3]))
        # visualizzare il nome della categoria corrente???
        # self.preventivo.get_categorie_name_from_id (self.capitoli[0], self.capitoli[1], self.capitoli[2])

################################################################################
###  FINESTRA MESSAGGI UTENTE ##################################################
################################################################################
    def message(self, widget, tipo, testo, data=None):
        """display a message windows"""
        buttons = Gtk.ButtonsType.OK_CANCEL
        if tipo == "info":
           tipo = Gtk.MessageType.INFO
           buttons = Gtk.ButtonsType.CLOSE
        elif tipo == "avviso":
           tipo = Gtk.MessageType.WARNING
        elif tipo == "domanda":
           tipo = Gtk.MessageType.QUESTION
        elif tipo == "errore":
           tipo = Gtk.MessageType.ERROR
           buttons = Gtk.ButtonsType.CLOSE
        else:
           tipo == Gtk.MessageType.INFO
           buttons = Gtk.ButtonsType.CLOSE
        message = Gtk.MessageDialog(parent=None, flags=0, type=tipo, buttons = buttons, message_format=testo)
        message.format_secondary_text(_("Premere un pulsante per continuare"))
        response = message.run()
        if response == Gtk.ResponseType.OK:
           response = True
        else:
           response = False 
        message.destroy()
        return response

################################################################################
# FINESTRA PER LA VISUALIZZAZIONE DEI REPORT SICUREZZA E MANODOPERA ############
################################################################################
class ReportWindow(Gtk.Window):
    def __init__(self, title, preventivo, parent_window = None):
        # Create a new dialog window for the scrolled window to be
        # packed into. 
        window = Gtk.Dialog()
        window.connect("destroy", self.destroy)
        window.set_title(title)
        window.set_border_width(0)
        window.set_size_request(600, 400)
        window.set_transient_for(parent_window)
        window.set_destroy_with_parent(True)
        # create a notebook
        notebook = Gtk.Notebook()
        scrolled_window1 = Gtk.ScrolledWindow()
        scrolled_window1.set_policy(Gtk.PolicyType.AUTOMATIC,
                                    Gtk.PolicyType.AUTOMATIC)
        scrolled_window2 = Gtk.ScrolledWindow()
        scrolled_window2.set_policy(Gtk.PolicyType.AUTOMATIC,
                                    Gtk.PolicyType.AUTOMATIC)
        label1 = Gtk.Label(_('Manodopera'))
        label2 = Gtk.Label(_('Sicurezza'))
        notebook.append_page(scrolled_window1, tab_label=label1)
        notebook.append_page(scrolled_window2, tab_label=label2)
        # The dialog window is created with a vbox packed into it.
        window.vbox.pack_start(notebook, True, True, 0)
        scrolled_window1.show()
        scrolled_window2.show()
        notebook.show()
        label1.show()
        label2.show()
        # imposta i liststore e le treeview CATEGORIE
        self.liststore_mo = Gtk.ListStore(str, str, str, str, str)
        self.liststore_sic = Gtk.ListStore(str, str, str, str, str)
        # recupera la valuta
        currency = preventivo.dati_generali_list()[3]
        # elementi da inserire nel notebook
        treeview1 = Gtk.TreeView(model=self.liststore_mo)
        treeview2 = Gtk.TreeView(model=self.liststore_sic)
        self.crea_treeview(treeview1, currency)
        self.crea_treeview(treeview2, currency, column3='Oneri Sicurezza')
        # pack the label into the scrolled window
        scrolled_window1.add_with_viewport(treeview1)
        scrolled_window2.add_with_viewport(treeview2)
        treeview1.show()
        treeview2.show()
        # Add a "close" button to the bottom of the dialog
        button = Gtk.Button("close")
        button.connect_object("clicked", self.destroy, window)
        # this makes it so the button is the default.
        button.set_can_default(True)
        window.action_area.pack_start(button, True, True, 0)
        # This grabs this button to be the default button. Simply hitting
        # the "Enter" key will cause this button to activate.
        button.grab_default()
        button.show()
        # crea la formattazione del testo
        self.get_data(preventivo)
        # mostra la finestra
        window.show()

    def destroy(self, widget):
        widget.destroy()
        return

    def crea_treeview (self, treeview_obj, currency, column3='Tempo (ore)'):
        """Funzione per creare la treeview dei capitoli e categorie"""
        liststore = treeview_obj.get_model()
        treeview_obj.columns = [None]*5
        treeview_obj.columns[0] = Gtk.TreeViewColumn('No.')
        treeview_obj.columns[1] = Gtk.TreeViewColumn('Nome Categoria/Capitolo')
        treeview_obj.columns[2] = Gtk.TreeViewColumn(column3)
        treeview_obj.columns[3] = Gtk.TreeViewColumn('Costo ({0})'.format(currency))
        treeview_obj.columns[4] = Gtk.TreeViewColumn('Inc.%')
        for n in range(5):
            # add columns to treeview
            treeview_obj.append_column(treeview_obj.columns[n])
            # create a CellRenderers to render the data
            treeview_obj.columns[n].cell = Gtk.CellRendererText()
            # add the cells to the columns
            treeview_obj.columns[n].pack_start(treeview_obj.columns[n].cell, True)
            # make treeview searchable
            treeview_obj.set_search_column(n)
            # set the cell attributes to the appropriate liststore column
            treeview_obj.columns[n].add_attribute(treeview_obj.columns[n].cell, 'text', n)

    def get_data(self, preventivo):
        # dimensiona le variabili e le liste
        lista_report1 = list()
        lista_report2 = list()
        # stampare report incidenza della manodopera
        lista_report1.extend(preventivo.print_report_inc_manodopera())
        # stampare report incidenza della manodopera su lista dei capitoli di epu
        lista_report1.extend(preventivo.print_report_inc_mo_lavorazioni ())
        # stampare report incidenza della sicurezza
        lista_report2.extend(preventivo.print_report_inc_sicurezza())
        # stampare report incidenza della sicurezza su lista dei capitoli di epu
        lista_report2.extend(preventivo.print_report_inc_sic_lavorazioni())
        # inserisce i dati nel modello della treeview
        for report in lista_report1:
            #('TOT', "", _("Totale Complessivo Manodopera (%s):") % valuta, tmp_tot, totale_globale, incid_perc)
            tipo, n, nome, tempo, prezzo, inc = report
            nome = nome.replace("\n", "")
            self.liststore_mo.append([n, nome, tempo, prezzo, inc])
            if tipo == 'TOT':
                self.liststore_mo.append(['', '', '-', '-', '-'])
        for report in lista_report2:
            #('SUP', "%s" % sup[0], "%s\n" % sup[1], sic_tot, totale, incid_perc)
            tipo, n, nome, sic, prezzo, inc = report
            nome = nome.replace("\n", "")
            self.liststore_sic.append([n, nome, sic, prezzo, inc])
            if tipo == 'TOT':
                self.liststore_sic.append(['', '', '-', '-', '-'])

################################################################################
###  PARTE PER BARRA DI AVANZAMENTO ############################################
################################################################################
class ProgressBar:
    """Mostra la barra di avanzamento"""
    def __init__(self, builder, passi, testo=None, window_mode=False, modal=False):
        self.builder = builder
        self.window = None
        self.activity_mode = False
        #self.timer = 0
        self.step = 0.1
        if not window_mode:
            # connetti e mostra l'oggetto barra d'avanzamento della finestra
            self.progressbar = self.builder.get_object("progressbar1")
            self.progressbar.show()
            # workaround per far si che compaia la progressbar
            while Gtk.events_pending():
                Gtk.main_iteration()
        elif window_mode: 
            # creazione della finestra della barra di avanzamento
            self.window = Gtk.Window(Gdk.WindowType.TOPLEVEL)
            self.window.connect("destroy", self.destroy)
            self.window.set_title(_("Barra di avanzamento"))
            self.window.set_border_width(0)
            self.window.set_default_size(600, 100)
            self.window.set_position(Gtk.WindowPosition.CENTER)
            self.window.set_resizable(False)
            self.window.set_modal(modal)
            # CREA UN BOX VERTICALE
            vbox = Gtk.VBox(False, 4)
            vbox.set_border_width(5)
            self.window.add(vbox)
            vbox.show()
            # Create a centering alignment object
            align = Gtk.Alignment.new(0.5, 0.5, 0, 0)
            align.set_padding(0, 0, 5, 5)
            vbox.pack_start(align, False, False, 5)
            align.show()
            # Create the ProgressBar
            self.progressbar = Gtk.ProgressBar()
            align.add(self.progressbar)
            self.progressbar.show()
            # mostra la finestra
            self.window.show()
            # workaround per far si che la costruzione della windows sia completata
            while Gtk.events_pending():
                Gtk.main_iteration()
        # imposta il testo della progressbar
        if testo is None: 
           testo = str()
        self.progressbar.set_text(testo)
        # imposta la modalità frazione o 'attivita'
        if type(passi) is int and passi > 0:
            self.step = 0.1/(float(passi)/10)
            self.progressbar.set_fraction(0.0)
        else:
            self.activity_mode = True
            # Add a timer callback to update the value of the progress bar
            self.progressbar.set_pulse_step(0.1)
        # cambia il cursore del mouse da normale a quello d'attesa
        self.cursor_wait = Gdk.Cursor(Gdk.CursorType.WATCH)
        self.gdk_window = self.progressbar.get_parent_window()
        if self.gdk_window is not None:
            self.gdk_window.set_cursor(self.cursor_wait)

    def avanza(self, new_text = None):
        """Avanza la barra di avanzamento"""
        if new_text is not None:
            # imposta il testo della progressbar
            self.progressbar.set_text(new_text)
        if not self.activity_mode:
            new_val = self.progressbar.get_fraction() + self.step
            # Set the new value
            if new_val <= 1.0:
                self.progressbar.set_fraction(new_val)
            else:
                self.progressbar.set_fraction(1.0)
        else:
            self.progressbar.pulse()
        while Gtk.events_pending():
            Gtk.main_iteration()
        # As this is a timeout function, return TRUE so that it
        # continues to get called
        return True

    def destroy(self, data=None):
        """distruggi la finestra della barra di avanzamento"""
        if self.window is not None:
            self.window.destroy()
        else: 
            self.progressbar.hide()
        if self.activity_mode:
            pass
            #GObject.source_remove(self.timer)
        #self.timer = 0
        # torna a impostare il cursore del mouse normale
        if self.gdk_window is not None:
            self.gdk_window.set_cursor(None)
        return True

# Update the value of the progress bar so that we get some movement
def progress_timeout(pbobj):
    pbobj.pulse()
    while Gtk.events_pending():
        Gtk.main_iteration()
    return True
